var _lnx$home;

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }

// namespace
var lnx = {
  exitHandled: false,
  // app entry point called via window onload event
  init: function init() {
    lnx.config.init();

    if (!lnx.config.ignoreLMS) {
      lnx.scormApi.lmsInit();
    } // an included js that preloads images - optional, remove in low bandwidth circumstances


    if (lnx_preload && lnx_preload.run) {
      lnx_preload.run();
    }

    lnx.audio.init();
    lnx.util.init();
    lnx.overlayMan.init();
  },
  // app exit point
  exit: function exit(closeWin) {
    // lmsFinish will only be called more than once
    // if the LMS return false to a previous LMSFinish call
    if (!this.exitHandled) {
      if (lnx.scormApi.lmsFinish()) {
        this.exitHandled = true;
      }
    }

    try {
      lnx.assessment.fullDestroy();
      lnx.nav.destroy();
      lnx.util.destroy();
    } catch (e) {}

    if (closeWin) {
      top.window.close();
    }
  }
};
lnx.cache = {
  cache: {},
  setValue: function setValue(k1, k2, v) {
    if (!this.cache[k1]) {
      this.cache[k1] = {};
    }

    this.cache[k1][k2] = v;
  },
  getValue: function getValue(k1, k2) {
    if (!this.cache[k1]) return null;
    return this.cache[k1][k2];
  }
}; // obj handles communications with lms api obj
// dependency on SCORM_12_APIWrapper.js which defines:
// doLMSInitialize, doLMSFinish, doLMSCommit, doLMSGetValue, doLMSSetValue

lnx.scormApi = {
  CMI: {
    LESSON_STATUS: "cmi.core.lesson_status",
    STUDENT_NAME: "cmi.core.student_name",
    STUDENT_ID: "cmi.core.student_id",
    SESSION_TIME: "cmi.core.session_time",
    SUSPEND_DATA: "cmi.suspend_data",
    COMPLETED_VAL: "completed",
    NOT_ATTEMPTED_VAL: "not attempted",
    INCOMPLETE_VAL: "incomplete"
  },
  intervalNum: null,
  initialized: false,
  courseComplete: false,
  crsCompleteThisSession: false,
  allowMultiCompletions: true,
  suspendData: {},
  testCompletedSuccessfully: false,
  lmsInit: function lmsInit() {
    try {
      var r = doLMSInitialize();
      lnx.util.log("LMSInitialize : " + r);

      if (r !== "true") {
        lnx.util.onError("Failed to initialize with LMS");
        return false;
      } else {
        this.initialized = true;
        this.lmsMaintainSession();
        var data = this.lmsGet(this.CMI.SUSPEND_DATA);
        this.suspendData = this.parseSuspendData(data);

        if (this.suspendData && this.suspendData.status === this.CMI.COMPLETED_VAL) {
          this.courseComplete = true;
        }

        lnx.util.log("LMSInitialize suspend_data: " + data); // LRN mandated functionality 
        // Rely on susspend data value to test if course is 'really' completed	
        // call get for the purpose of logging current value
        // call set with value of incomplete as mandated by LRN to avoid duplicate completion records

        this.lmsGet(this.CMI.LESSON_STATUS);
        this.lmsSet(this.CMI.LESSON_STATUS, this.CMI.INCOMPLETE_VAL);
        return true;
      }
    } catch (e) {
      lnx.util.log("lmsInit error thrown: " + e);
      return false;
    }
  },
  lmsFinish: function lmsFinish() {
    try {
      if (this.intervalNum) {
        clearInterval(this.intervalNum);
      }
    } catch (e) {
      lnx.util.log("lmsFinish, clearInterval error: " + e);
    }

    try {
      // session time set in doLMSFinish
      // As mandatad by LRN docs
      if (this.crsCompleteThisSession) {
        this.lmsSet(this.CMI.LESSON_STATUS, this.CMI.COMPLETED_VAL);
      } else {
        this.lmsSet(this.CMI.LESSON_STATUS, this.CMI.INCOMPLETE_VAL);
      }

      this.uploadSuspendData(false, true); //this.lmsCommit(); // called in SCORM_12_APIWrapper.js as per LRN requirements

      var r = doLMSFinish();
      lnx.util.log("LMSFinish: " + r);

      if (r !== "true") {
        return false;
      } else {
        this.initialized = false;
        return true;
      }
    } catch (e) {
      lnx.util.log("lmsFinish error thrown: " + e);
      return false;
    }
  },
  lmsCommit: function lmsCommit() {
    try {
      var r = doLMSCommit();
      lnx.util.log("LMSCommit: " + r);

      if (r !== "true") {
        return false;
      } else {
        return true;
      }
    } catch (e) {
      lnx.util.log("lmsCommit error thrown: " + e);
      return false;
    }
  },
  lmsGet: function lmsGet(name) {
    try {
      var r = doLMSGetValue(name);
      lnx.util.log("Get: " + name + " : " + r);
      return r;
    } catch (e) {
      lnx.util.log("lmsGet error thrown: " + e);
      return "";
    }
  },
  lmsSet: function lmsSet(name, value) {
    try {
      var r = doLMSSetValue(name, value);
      lnx.util.log("Set: " + name + " : " + value + " -- " + r);

      if (r !== "true") {
        return false;
      } else {
        return true;
      }
    } catch (e) {
      lnx.util.log("lmsSet error thrown: " + e);
      return false;
    }
  },
  lmsCompleteCourse: function lmsCompleteCourse(pass) {
    try {
      // As per LRN docs, only set completion record if course not previously completed
      // we rely on value in suspend_data to determine if course has previoulsy been completed
      if (pass && (!this.courseComplete || this.allowMultiCompletions)) {
        this.crsCompleteThisSession = true; // this call could/should be defered to course exit as per LRN docs
        // leaving it in for present as back up in case of unconventional window exit/non exit	

        var r = this.lmsSet(this.CMI.LESSON_STATUS, this.CMI.COMPLETED_VAL);

        if (r) {
          // LRN implementation is non-standard with regard to LMSCommit calls -
          // not permited until course exit
          // must assume data has/will be successfully persised
          this.courseComplete = true;
          this.suspendData.status = this.CMI.COMPLETED_VAL;
          this.uploadSuspendData(true, true); //r = this.lmsCommit(); // added to account for LRN sync call failing when window closed directly 2) removed 1/9/21 at lrn request as temp fix

          lnx.util.log("lmsCompleteCourse lmsCommit result: " + r); // log whether this call has really 'stuck'

          lnx.util.log("lmsCompleteCourse Get cmi.suspend_data --");
          this.lmsGet(this.CMI.SUSPEND_DATA);
          return true;
        } else {
          if (lnx.config.ignoreLMS) {
            return;
          } // must inform user that they may not actually get credit


          lnx.util.onError("LMS failed to confirm Knowledge Check pass/course completion", true);
        }
      } else {
        lnx.util.log("lmsCompleteCourse test passed: " + pass);
        return true;
      }
    } catch (e) {
      lnx.util.log("lmsCompleteCourse error thrown: " + e);
      return;
    }
  },
  lmsLogTestResult: function lmsLogTestResult(pass) {
    this.testCompletedSuccessfully = pass;
  },
  getTestCompletedSuccessfully: function getTestCompletedSuccessfully() {
    return this.testCompletedSuccessfully;
  },
  lmsMaintainSession: function lmsMaintainSession() {
    var time = 1000 * 60 * 20;

    try {
      this.intervalNum = setInterval(lnx.scormApi.sessionKeeper, time);
    } catch (e) {
      lnx.util.log("lmsMaintainSession error: " + e);
    }
  },
  //IE does not pass args to setInterval so need this helper func
  sessionKeeper: function sessionKeeper() {
    lnx.scormApi.uploadSuspendData(false, true); //lnx.scormApi.lmsGet(lnx.scormApi.CMI.STUDENT_ID);
  },
  parseSuspendData: function parseSuspendData(data) {
    var a, b, i, len;
    var o = {};

    try {
      if (data && typeof data === "string") {
        a = data.split(",");

        if (a && a.length) {
          for (i = 0, len = a.length; i < len; i++) {
            b = a[i].split("=");

            if (b && b.length === 2) {
              o[b[0]] = b[1];
            }
          }
        }
      }
    } catch (e) {
      lnx.util.onError(e, false, "parseSuspendData");
      return null;
    }

    return o;
  },
  serializeSuspendData: function serializeSuspendData() {
    var s = "",
        p;
    var d = this.suspendData;

    for (p in d) {
      s += p + "=" + d[p] + ",";
    }

    if (s.length) {
      s = s.slice(0, -1);
    }

    return s;
  },
  getSuspendData: function getSuspendData() {
    return this.suspendData;
  },
  uploadSuspendData: function uploadSuspendData(noBookMarkUpdate, noCommit) {
    if (!noBookMarkUpdate) {
      this.suspendData.bookmark = lnx.nav.getCurrNavId(true, this.crsCompleteThisSession);
    }

    var data = this.serializeSuspendData();
    var r = this.lmsSet(this.CMI.SUSPEND_DATA, data);

    if (!noCommit) {
      r = this.lmsCommit();
    }

    return r;
  },
  getSuspendDataValue: function getSuspendDataValue(val) {
    try {
      if (this.suspendData) {
        return this.suspendData[val];
      }
    } catch (e) {
      lnx.util.onError(e, false, "getSuspendDataValue: " + val);
    }

    return null;
  },
  setSuspendDataValue: function setSuspendDataValue(prop, val) {
    if (prop) {
      this.suspendData[prop] = val;
    }

    return this.suspendData;
  },
  getIsInitialized: function getIsInitialized() {
    return this.initialized;
  },
  getIsCourseComplete: function getIsCourseComplete() {
    return this.courseComplete;
  },
  setMultiCompletionsStatus: function setMultiCompletionsStatus(status) {
    this.allowMultiCompletions = status;
  } //    updateLMSBookmark: function(){
  //        this.suspendData.bookmark = lnx.nav.getCurrNavId(false, this.crsCompleteThisSession);
  //        var data = this.serializeSuspendData();
  //        this.lmsSet(this.CMI.SUSPEND_DATA, data);
  // }

};
lnx.config = {
  isIpad: /ipad/i.test(navigator.userAgent),
  isIE: navigator.appName === "Microsoft Internet Explorer",
  isIE8orLess: navigator.appName === "Microsoft Internet Explorer" && !document.createElement("video").canPlayType,
  isIE6: false,
  isIELessThan8: false,
  useHTML5: true,
  showDebugWin: /debug=true/i.test(top.window.location.search),
  debugWithCourseComplete: /debugwithcoursecomplete=true/i.test(top.window.location.search.toLowerCase()),
  debugOutput: null,
  autoAudio: true,
  noAudio: true,
  isR2L: false,
  courseId: "",
  //UNDO set to false in release version
  showAlerts: false,
  //UNDO set to false in release version
  ignoreLMS: /http\:\/\/localhost\/devfon/i.test(top.window.location) || /http\:\/\/(www\.)?learnex\./i.test(top.window.location) || /ignorelms=true/i.test(top.window.location.search.toLowerCase()),
  screenShotMode: /mode=screenshot/i.test(top.window.location),
  overrideLinearNav: /overrideLinearNav=true/i.test(top.window.location),
  winDim: {
    w: 1000,
    h: 840
  },
  showEdits: /showEdits/i.test(top.window.location),
  editNumber: null,
  showScreen: null,
  init: function init() {
    var self = this;

    if (this.showEdits && /editNumber/i.test(top.window.location.search)) {
      this.editNumber = extractHTMLParam("editNumber");
    }

    if (this.ignoreLMS && this.screenShotMode) {
      addScreenShotCss();
    }

    function extractHTMLParamValue(val) {
      var regex = new RegExp(val, "i");
      var s = top.window.location.search.split("&");

      for (var i = 0; i < s.length; i++) {
        if (regex.test(s[i])) {
          s = s[i].split("=");
          return s[1];
        }
      }

      return null;
    }

    function setShowScreenVal() {
      var t = extractHTMLParamValue("showScreen");

      if (t) {
        var reg = /\d{0,3}_?(C_\d{1,3})/;
        var r = t.match(reg);

        if (r) {
          return r[1];
        } else {
          reg = /\d{0,3}_?(toc_\d{1,3})/;
          r = t.match(reg);

          if (r) {
            lnx.view.showToc();
          }
        }
      }

      return null;
    }

    function addScreenShotCss() {
      var elm = document.createElement("link");
      elm.setAttribute("rel", "stylesheet");
      elm.setAttribute("href", "screenshots.css");
      elm.setAttribute("media", "all");
      document.head.appendChild(elm);
    }

    this.showScreen = setShowScreenVal();

    if (this.showDebugWin) {
      this.showAlerts = true;
    } //this.useHTML5 = !this.isIE;


    if (this.isIE) {
      var ver = this.msIeVersion();
      this.isIE6 = ver === 6;
      this.isIELessThan8 = ver !== 0 && ver < 8;
    }

    try {
      // hide flash video player if not ie
      if (!this.isIE) {
        var elm = document.getElementById("flashWarehouse");

        if (elm) {
          elm.className = "hide";
        }
      }
    } catch (e) {} // UNDO remove
    // this.showDebugWin = true;


    try {
      if (!this.showEdits) {
        top.window.resizeTo(this.winDim.w, this.winDim.h);
      }
    } catch (e) {
      // sometimes resizing lms frameset can generate a security access error 
      // call non-error throwing func after delay
      window.setTimeout(lnx.util.resizeWin, 2000);
    } // seperate resize call from debug setup so we dont loose both with one error


    try {
      if (this.showDebugWin && !this.debugOutput) {
        this.debugOutput = document.getElementById("debugOutput");
        this.debugOutput.style.display = "block";
        lnx.util.log("Debug Session...");
      }
    } catch (e) {
      lnx.util.onError(e, false, "config.init - showDebugWin");
    }

    if (this.debugWithCourseComplete) {
      lnx.scormApi.courseComplete = true;
    }
  },
  // ms code - http://support.microsoft.com/kb/167820
  msIeVersion: function msIeVersion() {
    try {
      var ua = window.navigator.userAgent;
      var msie = ua.indexOf("MSIE ");

      if (msie > 0) {
        // If Internet Explorer, return major version number
        return parseInt(ua.substring(msie + 5, ua.indexOf(".", msie)));
      } else {
        // If another browser, return 0
        return 0;
      }
    } catch (e) {
      return 0;
    }
  }
}; // adapted from padilicious.com script

lnx.swipeEvent = {
  triggerElementID: null,
  fingerCount: 0,
  startX: 0,
  startY: 0,
  curX: 0,
  curY: 0,
  deltaX: 0,
  deltaY: 0,
  horzDiff: 0,
  vertDiff: 0,
  minLength: 100,
  swipeLength: 0,
  swipeAngle: null,
  swipeDirection: null,
  init: function init() {
    lnx.util.updateEventListener(document, "touchstart", this.touchStart);
    lnx.util.updateEventListener(document, "touchmove", this.touchMove);
    lnx.util.updateEventListener(document, "touchend", this.touchEnd);
  },
  touchStart: function touchStart(event, passedName) {
    lnx.swipeEvent.fingerCount = event.touches.length;

    if (lnx.swipeEvent.fingerCount == 1) {
      // get the coordinates of the touch
      lnx.swipeEvent.startX = event.touches[0].pageX;
      lnx.swipeEvent.startY = event.touches[0].pageY; // store the triggering element ID

      lnx.swipeEvent.triggerElementID = passedName;
    } else {
      // more than one finger touched so cancel
      lnx.swipeEvent.touchCancel(event);
    }
  },
  touchMove: function touchMove(event) {
    //event.preventDefault();
    if (event.touches.length == 1) {
      lnx.swipeEvent.curX = event.touches[0].pageX;
      lnx.swipeEvent.curY = event.touches[0].pageY;
    } else {
      lnx.swipeEvent.touchCancel(event);
    }
  },
  touchEnd: function touchEnd(event) {
    //event.preventDefault();
    // check to see if more than one finger was used and that there is an ending coordinate
    if (lnx.swipeEvent.fingerCount == 1 && lnx.swipeEvent.curX != 0) {
      // use the Distance Formula to determine the length of the swipe
      lnx.swipeEvent.swipeLength = Math.round(Math.sqrt(Math.pow(lnx.swipeEvent.curX - lnx.swipeEvent.startX, 2) + Math.pow(lnx.swipeEvent.curY - lnx.swipeEvent.startY, 2))); // if the user swiped more than the minimum length, perform the appropriate action

      if (lnx.swipeEvent.swipeLength >= lnx.swipeEvent.minLength) {
        lnx.swipeEvent.caluculateAngle();
        lnx.swipeEvent.determineSwipeDirection();
        lnx.swipeEvent.processingRoutine();
        lnx.swipeEvent.touchCancel(event); // reset the variables
      } else {
        lnx.swipeEvent.touchCancel(event);
      }
    } else {
      lnx.swipeEvent.touchCancel(event);
    }
  },
  touchCancel: function touchCancel(event) {
    // reset the variables back to default values
    lnx.swipeEvent.fingerCount = 0;
    lnx.swipeEvent.startX = 0;
    lnx.swipeEvent.startY = 0;
    lnx.swipeEvent.curX = 0;
    lnx.swipeEvent.curY = 0;
    lnx.swipeEvent.deltaX = 0;
    lnx.swipeEvent.deltaY = 0;
    lnx.swipeEvent.horzDiff = 0;
    lnx.swipeEvent.vertDiff = 0;
    lnx.swipeEvent.swipeLength = 0;
    lnx.swipeEvent.swipeAngle = null;
    lnx.swipeEvent.swipeDirection = null;
    lnx.swipeEvent.triggerElementID = null;
  },
  caluculateAngle: function caluculateAngle() {
    var X = lnx.swipeEvent.startX - lnx.swipeEvent.curX;
    var Y = lnx.swipeEvent.curY - lnx.swipeEvent.startY;
    var Z = Math.round(Math.sqrt(Math.pow(X, 2) + Math.pow(Y, 2))); //the distance - rounded - in pixels

    var r = Math.atan2(Y, X); //angle in radians (Cartesian system)
    // get angle in degrees

    lnx.swipeEvent.swipeAngle = Math.round(r * 180 / Math.PI);

    if (lnx.swipeEvent.swipeAngle < 0) {
      lnx.swipeEvent.swipeAngle = 360 - Math.abs(lnx.swipeEvent.swipeAngle);
    }
  },
  determineSwipeDirection: function determineSwipeDirection() {
    if (lnx.swipeEvent.swipeAngle <= 10 && lnx.swipeEvent.swipeAngle >= 0) {
      lnx.swipeEvent.swipeDirection = "left";
    } else if (lnx.swipeEvent.swipeAngle <= 360 && lnx.swipeEvent.swipeAngle >= 350) {
      lnx.swipeEvent.swipeDirection = "left";
    } else if (lnx.swipeEvent.swipeAngle >= 170 && lnx.swipeEvent.swipeAngle <= 190) {
      lnx.swipeEvent.swipeDirection = "right";
    } else if (lnx.swipeEvent.swipeAngle > 10 && lnx.swipeEvent.swipeAngle < 170) {
      lnx.swipeEvent.swipeDirection = "down";
    } else {
      lnx.swipeEvent.swipeDirection = "up";
    }
  },
  processingRoutine: function processingRoutine() {
    //var swipedElement = document.getElementById("screenFrame");
    var swipedElement = document.body;

    if (lnx.swipeEvent.swipeDirection === "left") {
      lnx.nav.next();
    } else if (lnx.swipeEvent.swipeDirection === "right") {
      lnx.nav.previous();
    }
  }
};
lnx.util = {
  stringer: null,
  xmlObj: null,
  xmlDocs: [],
  init: function init(file) {
    if (!file) file = "./course.xml?nocache=";
    this.getXML(file + new Date().getTime(), this.onXMLLoad.bind(this));
  },
  processFile: function processFile(xml) {
    this.xmlDocs.push(xml);

    if (lnx.config.showEdits && this.xmlDocs.length < 2) {
      this.getXML("./edits.xml?nocache=" + new Date().getTime(), this.onXMLLoad.bind(this));
    } else {
      lnx.localization.init(this.xmlDocs[0]);
      lnx.nav.init(this.xmlDocs[0]);
      if (lnx.config.showEdits) lnx.edits.init(this.xmlDocs[0], this.xmlDocs[1]);
    }
  },
  getXML: function getXML(url, f) {
    try {
      var x = this.xmlObj = this.getRequestObject();
      x.onreadystatechange = f;
      x.open("GET", url);
      x.send();
    } catch (e) {
      lnx.util.onError(e, true, "getXML");
    }
  },
  getRequestObject: function getRequestObject() {
    if (window.XMLHttpRequest) {
      return new XMLHttpRequest();
    }

    try {
      return new ActiveXObject("MSXML2.XMLHTTP.6.0");
    } catch (e) {
      try {
        return new ActiveXObject("MSXML2.XMLHTTP.3.0");
      } catch (e2) {
        throw new Error("XMLHTTP not supported");
      }
    }
  },
  onXMLLoad: function onXMLLoad() {
    // in ie8 where enable native XMLHTTP support is disabled
    // we have to revert to activeX obj
    // in this case this != the request obj in callback func
    // hence we use prop ref - lnx.util.xmlObj
    if (lnx.util.xmlObj.readyState == 4) {
      if (lnx.util.xmlObj.status == 200) {
        var xml = lnx.util.xmlObj.responseXML;

        if (!xml || xml && xml.parseError && xml.parseError.errorCode !== 0) {
          lnx.util.onError("Network error\nPlease restart course", true);
        } else {
          // no nomalize in ie for XML docs
          if (xml.normalize) {
            xml.normalize();
          }

          lnx.util.stripWhiteSpace(xml);
          this.processFile(xml);
        }
      } else {
        lnx.util.onError("Network error, status: " + lnx.util.xmlObj.status + "\nPlease restart course", true);
      }
    }
  },
  XMLToString: function XMLToString(node) {
    if (node.xml) {
      return node.xml;
    }

    this.stringer || (this.stringer = new XMLSerializer());
    return this.stringer.serializeToString(node);
  },
  // IE property preserveWhiteSpace defaults to false 
  // 'whitespace' text nodes are therefore ignored when walking doc tree.
  // In IE, therefore, stripWhiteSpace never gets a chance to remove same. 
  // If u inspect serialized doc after processing, whitespace 
  // formatting remains in IE (as its not considered a node - c above) 
  // but not in safari, firefox etc
  // unknown/lost url for attribution of this function
  stripWhiteSpace: function stripWhiteSpace(n) {
    for (var i = 0; i < n.childNodes.length; i++) {
      var c = n.childNodes[i];

      if (c.nodeType == 3 && !/\S/.test(c.nodeValue)) {
        n.removeChild(c);
        i--;
      }

      if (c.nodeType == 1) {
        this.stripWhiteSpace(c);
      }
    }

    return n;
  },
  getElmsByTagAndAttrib: function getElmsByTagAndAttrib(elm, tag, attrib) {
    var res = [];
    var elms = elm.getElementsByTagName(tag);

    if (elms.length) {
      var l = elms.length;

      for (var i = 0; i < l; i++) {
        if (elms[i].getAttribute(attrib)) {
          res.push(elms[i]);
        }
      }
    }

    return res;
  },
  onWinErr: function onWinErr(msg, url, line) {
    this.onError(msg + ", " + url + ", " + line);
    return false;
  },
  onError: function onError(e, force, info) {
    var nl = "\n";
    var msg = "Error details:" + nl;

    if (!info) {
      info = "";
    }

    if (_typeof(e) === "object") {
      msg += "name: " + e.name + nl;
      msg += "message: " + e.message + nl;
      msg += "description: " + e.description + nl;
      msg += "number: " + (e.number & 0xffff) + nl;
      msg += info;
    } else {
      msg = "Error: " + e + nl + info;
    }

    this.log(msg);

    if (!force && !lnx.config.showAlerts) {
      //(!( force || lnx.config.showAlerts ))
      return;
    } // need to cater for iPad


    alert(msg); //UNDO remove
    //debugger;
  },
  log: function log(msg) {
    try {
      if (window.console) {
        window.console.log(msg);
      }

      if (lnx.config.showDebugWin) {
        lnx.config.debugOutput.insertAdjacentText("beforeEnd", msg + "\n\n");
        lnx.config.debugOutput.doScroll("down");
      }
    } catch (e) {}
  },
  // adds or removes event listeners 
  // caters for ie legacy event model
  updateEventListener: function updateEventListener(evtTargs, type, listener, remove) {
    var temp, elm, method; // allow method to be called with single or multiple targets

    if (evtTargs.constructor !== Array) {
      evtTargs = [evtTargs];
    } // determine if we are adding or removing listeners and handle ie 8


    if (document.addEventListener) {
      method = remove ? "removeEventListener" : "addEventListener";
    } else {
      method = remove ? "detachEvent" : "attachEvent";
      type = "on" + type;
    } // note assumption - currently not testing validity of 'non-string' elms


    for (var i = 0, len = evtTargs.length; i < len; i++) {
      elm = evtTargs[i];

      if (typeof elm === "string") {
        elm = temp = document.getElementById(elm);

        if (!elm) {
          lnx.util.onError("Failed to find element in updateEventListener, " + temp);
          continue;
        }
      } // update listener


      elm[method](type, listener, false);
    }
  },
  resizeWin: function resizeWin() {
    try {
      top.window.resizeTo(lnx.config.winDim.w, lnx.config.winDim.h);
    } catch (e) {
      lnx.util.onError(e, false, "util.resizeWin - resizeTo");
    }
  },
  stopVOverflow: function stopVOverflow(d) {
    if (!d) {
      console.log("ERROR: No element passed to stopVOverflow");
      return;
    }

    if (d.scrollHeight > d.clientHeight) {
      var sz = parseInt(getComputedStyle(d).fontSize);

      while (sz > 11) {
        sz -= 1;
        d.style.fontSize = sz + "px";

        if (d.scrollHeight <= d.clientHeight) {
          break;
        }
      }
    }
  },
  stopHOverflow: function stopHOverflow(d) {
    var lh = parseInt(getComputedStyle(d).lineHeight);
    var i = 1;
    var fs = parseInt(getComputedStyle(d).fontSize);

    while (parseInt(getComputedStyle(d).height) > lh) {
      d.style.fontSize = fs - i++ + "px";

      if (i > 100) {
        break;
      }
    }
  },
  destroy: function destroy() {
    this.stringer = null;
    this.xmlObj = null;
  }
};
lnx.edits = {
  editViewer: null,
  editList: null,
  edits: null,
  divs: [],
  screenInfo: [],
  available: false,
  curListItem: null,
  listItems: null,
  course: null,
  init: function init(course, edits) {
    this.available = true;
    this.course = course; //addEditKey(document.getElementById("innerContainer"));

    this.editViewer = document.getElementById("innerContainer").appendChild(document.createElement("div"));
    this.editViewer.id = "editViewer";
    this.editList = document.body.appendChild(document.createElement("div"));
    this.editList.id = "editList";
    this.editList.addEventListener("click", this.onListSelect.bind(this));
    this.editList.style.display = "block";
    var fb = course.querySelectorAll("[class='feedback']");

    for (var i = 0; i < fb.length; i++) {
      //fb[i].classList.add("showFeedback");
      fb[i].setAttribute("class", fb[i].getAttribute("class") + " showFeedback");
    }

    var s = "";
    s += "<div><p>Info: </p><div></div></div>";
    s += "<div><p>Old Version</p><div></div></div>";
    s += "<div><p>Abbott Edit</p><div></div></div>";
    s += "<div><p>New Version</p><div></div></div>";
    s += "<div><p>English</p><div></div></div>";
    this.editViewer.innerHTML = s;

    for (var i = 0; i < 5; i++) {
      this.divs.push(this.editViewer.childNodes[i].childNodes[1]);
    }

    var list = [],
        self = this;
    this.edits = edits.documentElement.childNodes; //this.edits.forEach(processEdit); //forEach doesn't work for nodeLists in IE

    for (var i = 0; i < this.edits.length; i++) {
      processEdit(this.edits[i]);
    }

    lnx.nav.getOrderedScreenList(list);
    addMappingToEdit(list, this.edits);
    addToEditList(list, this.edits);

    if (lnx.config.editNumber) {
      var e = this.editList.querySelector('tr[number="' + lnx.config.editNumber + '"');

      if (e) {
        e = {
          target: e
        };
        this.onListSelect(e);
      }
    }

    var p = this.editList.appendChild(document.createElement("p"));
    p.innerHTML = '<a href="../../reports/allLangs.html?lang=' + edits.documentElement.getAttribute("langId") + '" target="blank">Edits Report, this languages</a>';
    p = this.editList.appendChild(document.createElement("p"));
    p.innerHTML = '<a href="../../reports/allLangs.html" target="blank">Edits Report, all languages</a>';

    function processEdit(edit) {
      var cp = edit.getAttribute("csspath"),
          xp = edit.getAttribute("xpath"),
          type = edit.getAttribute("type"),
          rows = edit.getAttribute("rows"),
          match = edit.getAttribute("match"),
          number = edit.getAttribute("number"),
          node = course.querySelector(cp);

      if (!node) {
        // IE has problem with passing XML ids to QS
        var r = cp.match(/#\S+/g);

        for (var i = 0; i < r.length; i++) {
          r[i] = r[i].replace(/">$/, "");
          cp = cp.replace(r[i], "[id='" + r[i].substring(1) + "']");
        }
      }

      node = course.querySelector(cp);

      if (type === "deletion") {
        var elm = course.createElement("span");

        if (navigator.vendor) {
          elm.innerHTML = "&#x20;&#x20;[DEL]";
        } else {
          elm.appendChild(course.createTextNode("_[DEL]_"));
        }

        node.appendChild(elm);
        node = elm;
      }

      if (!node.classList) {
        var pre = node.getAttribute("class") ? node.getAttribute("class") + " " : "";
        node.setAttribute("class", pre + "edit_" + type);
      } else {
        node.classList.add("edit_" + type);
      }

      node.setAttribute("number", number);
      node.setAttribute("onclick", "lnx.edits.onSelect(" + number + ")");
      var id = xp && xp.match(/\[@id='(.+?)']/)[1];

      if (!id) {
        id = cp.match(/\[id='(.+?)']/)[1];
      }

      edit.setAttribute("screen", id);
      list.push({
        id: id,
        rows: rows,
        match: match,
        number: number,
        edit: edit
      });
    }

    function addMappingToEdit(list, edits) {
      for (var i = 0; i < list.length; i++) {
        list[i].edit.setAttribute("orderNumber", i + 1);
      }
    }

    function addToEditList(list, edits) {
      this.editList.innerHTML = createTable(list, edits); //this.editList.childNodes[1].insertAdjacentHTML("beforeEnd", ("<li sid='" + list[i].id + "'>" + content + "</li>"));            
    }

    function createTable(list, edits) {
      var space = "&nbsp;&nbsp;",
          sp2 = space + space + space + space,
          s = "<table><thead><tr><th colspan='4'><div class='editKey'>Edit key: &nbsp;&nbsp;&nbsp;&nbsp;<div>&#x00A0;</div><span>Revised</span><div>&#x00A0;</div><span>New</span><div>&#x00A0;</div><span>Deleted</span></div></th></tr><tr><th>Edit #</th><th>Screen #</th><th>Row #</th><th>&nbsp;&nbsp;Match</th></tr></thead><tbody>";

      for (var i = 0; i < list.length; i++) {
        var match = list[i].match === "true" ? "&#x2714;" : list[i].match === "" ? "" : "&#x2716;";
        var num = (i < 9 ? "&nbsp;" : "") + (i + 1);
        var idx = list[i].idx;
        idx = idx < 10 ? space + idx : idx;
        s += "<tr " + "sid='" + list[i].id + "' number='" + list[i].number + "' orderNumber='" + (i + 1) + "'><td>" + num + "</td><td>" + idx + space + "(" + list[i].hIdx + ")</td><td>" + sp2 + list[i].rows.replace(/,/g, ", ") + "</td><td>" + match + "</td></tr>";
      }

      s += "</tbody></table>";
      return s;
    }

    function addEditKey(elm) {
      var html = "<div class='editKey'><div></div><span>Revised Content</span><div></div><span>New Content</span><div></div><span>Deleted Content</span></div>";
      elm.insertAdjacentHTML("beforeEnd", html);
    }
  },
  handleKey: function handleKey(k) {
    if (!this.available) return;
    var items = this.editList.querySelectorAll("tbody tr"),
        i = 1;

    if (k === "ArrowDown") {
      if (this.curListItem) {
        if (this.curListItem.nextSibling) {
          i = parseInt(this.curListItem.getAttribute("orderNumber"));
          i++;
        }

        this.onListSelect(null, i, true);
      } else {
        this.findNextPrevScreenWithEdit();
      }
    } else if (k === "ArrowUp") {
      if (this.curListItem) {
        if (this.curListItem.previousSibling) {
          i = parseInt(this.curListItem.getAttribute("orderNumber"));
          i--;
        } else {
          i = items.length;
        }

        this.onListSelect(null, i, true);
      } else {
        this.findNextPrevScreenWithEdit(true);
      }
    }
  },
  findNextPrevScreenWithEdit: function findNextPrevScreenWithEdit(prev) {
    var navId = lnx.nav.getNextPrevNavIds(prev);

    if (!navId) {
      navId = this.getFisrtLastEditNavId(prev);
    }

    if (lnx.nav.getCurrNavId() !== navId) {
      lnx.nav.navigate(navId, false);

      if (prev) {
        // need to jump to last edit if more than one
        var sid = this.curListItem.getAttribute("sid");
        var nextSib = this.curListItem.nextSibling;

        while (nextSib) {
          if (!(sid == nextSib.getAttribute("sid"))) {
            break;
          } else {
            this.updateList(nextSib);
            this.onSelect(nextSib.getAttribute("number"));
            nextSib = nextSib.nextSibling;
          }
        }
      }
    }
  },
  getFisrtLastEditNavId: function getFisrtLastEditNavId(last) {
    var items = this.editList.querySelectorAll("tbody tr");
    var item = last ? items[items.length - 1] : items[0];
    return item.getAttribute("sid");
  },
  setUp: function setUp() {
    if (this.available) this.onSelect();
  },
  tearDown: function tearDown() {
    if (this.available) {
      this.editViewer.style.display = "none";
    }
  },
  onListSelect: function onListSelect(e, i, viaKey) {
    var targ, navId, number;

    if (e) {
      navId = e.target.getAttribute("sid") || e.target.parentNode.getAttribute("sid");
      if (!navId) return;
      targ = e.target.hasAttribute("sid") ? e.target : e.target.parentNode;
    }

    this.updateList(targ, i);

    if (navId || viaKey) {
      navId = navId ? navId : this.curListItem.getAttribute("sid");
      number = this.curListItem.getAttribute("number");
      if (lnx.nav.getCurrNavId() !== navId) lnx.nav.navigate(navId, true);
      this.onSelect(number, true);
    }
  },
  updateList: function updateList(cur, i, clear) {
    var prevListItem = this.curListItem;

    if (clear) {
      if (prevListItem) prevListItem.style.outline = "";
      this.curListItem = null;
      return;
    }

    if (!cur) {
      this.curListItem = this.editList.querySelectorAll("tbody tr")[i - 1];
    } else {
      this.curListItem = cur;
    }

    if (prevListItem) prevListItem.style.outline = "";
    this.curListItem.style.outline = "1px solid black";
  },
  onSelect: function onSelect(num, viaList) {
    var self = lnx.edits,
        v = self.editViewer,
        e = self.edits,
        d = self.divs,
        s = "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;",
        match,
        edit,
        elms;
    elms = document.getElementById("screenFrame").querySelectorAll("fieldset.show [number]");

    if (!elms.length) {
      // not a question screen, make more general check
      elms = document.getElementById("screenFrame").querySelectorAll("[number]");
    }

    if (!elms.length) {
      this.updateList(null, null, true);
      return;
    }

    if (!num) {
      num = parseInt(elms[0].getAttribute("number"));
    }

    for (var i = 0; i < elms.length; i++) {
      if (elms[i].getAttribute("number") === num.toString()) {
        elms[i].style.outline = "1px solid black";
      } else {
        elms[i].style.outline = "";
      }
    }

    v.style.display = "block";
    edit = e[num - 1];
    match = edit.getAttribute("match");
    match = match === "true" ? "&#x2714;" : match === "" ? "" : "&#x2716;", d[0].innerHTML = "Edit #: " + edit.getAttribute("orderNumber") + s + "Type: " + edit.getAttribute("type") + s + "Match: " + match + s + "Row(s): " + edit.getAttribute("rows");
    d[1].innerHTML = edit.childNodes[0].textContent;
    d[2].innerHTML = edit.childNodes[2].textContent;
    d[3].innerHTML = edit.childNodes[1].textContent;
    d[4].innerHTML = edit.childNodes[3].textContent;

    if (!viaList) {
      self.onListSelect.call(this, null, parseInt(edit.getAttribute("orderNumber")));
    }
  }
};
lnx.localization = {
  stringMap: {},
  init: function init(doc) {
    var s = doc.getElementsByTagName("strings")[0];

    for (var i = 0, len = s.childNodes.length; i < len; i++) {
      if (s.childNodes[i].nodeType === 1) {
        this.stringMap[s.childNodes[i].tagName] = s.childNodes[i].firstChild.nodeValue;
      }
    }
  },
  getLocalString: function getLocalString(id) {
    return this.stringMap[id];
  }
}; // this object provides an interface for messages from 
// embedded flash swf files

lnx.flashInterface = {
  flashVideoIsReady: false,
  // invoked by flash player when it starts to plays a video
  playing: function playing(id) {
    lnx.util.log("flashplaying");
    lnx.audio.stopAudio();
  },
  // invoked by initial host swf when it loads
  ready: function ready() {
    this.flashVideoIsReady = true;
  },
  // log error messages from swfs
  onError: function onError(e) {
    lnx.util.onError(e, false, "via flashInterface");
  }
};
lnx.nav = {
  // nav target on test retake
  TEST_RETAKE_ID: "Q_1",
  //"C_32",
  // shortcut refs
  content: null,
  toc: null,
  screens: null,
  //xml node containers
  navNodes: [],
  // list of leaf navigation nodes, keys added to also allow accesse as a map
  navNodeMap: {},
  // map of all navigation nodes inc branch nodes
  screensMap: {},
  // map of all content screens
  templateMap: {},
  allNodes: [],
  // list of all nodes inc. branch nodes - used for forced linear navigation feature
  navIndex: 0,
  currNavId: "",
  // indicate when app is at first or last screen
  terminals: {
    isFirst: false,
    isLast: false
  },
  farIndex: 0,
  farNavId: null,
  isLinearNav: false,
  completeAllScreens: false,
  isAutoCompletion: false,
  keys: {
    "ArrowDown": "ArrowDown",
    "Down": "ArrowDown",
    "ArrowUp": "ArrowUp",
    "Up": "ArrowUp",
    "ArrowRight": "ArrowRight",
    "Right": "ArrowRight",
    "ArrowLeft": "ArrowLeft",
    "Left": "ArrowLeft"
  },
  tocPanel: null,
  currNodeType: null,
  mustCompleteMap: {
    "hasVirtualScreenOverlay": true,
    "scenarioWithVirtualScreens": true,
    "dialogueWithVirtualScreens": true,
    "verticalScenario": true,
    "dialogue": true,
    "stages": true,
    "flashCard": true
  },
  screenLocked: false,
  currNode: null,
  isFrameWork2: false,
  helpRefIds: null,
  audioTout: null,
  jumpTo: null,
  homeId: null,
  init: function init(doc) {
    // let user exit course if we have not got scorm api ref
    if (!lnx.config.ignoreLMS && !lnx.scormApi.getIsInitialized()) {
      var prompMsg = lnx.localization.getLocalString("s1");

      if (!window.confirm(prompMsg)) {
        top.window.close();
        return;
      }
    } // if ipad add swipe event handling


    if (lnx.config.isIpad) {
      lnx.swipeEvent.init();
    }

    this.content = doc;
    var cr = doc.getElementsByTagName("course")[0];
    this.jumpTo = cr.getAttribute("jumpto");
    var t = parseInt(cr.getAttribute("audiotout"));
    this.audioTout = isNaN(t) ? null : t * 1000;
    this.isFrameWork2 = cr.getAttribute("framework") === "2";
    this.frameWorkNum = cr.getAttribute("framework");

    if (this.isFrameWork2) {
      this.helpRefIds = [];
      this.helpRefIds.push(cr.getAttribute("helpId"));
    }

    this.isLinearNav = cr.getAttribute("linearnav") === "true" && !lnx.scormApi.getIsCourseComplete() && !lnx.config.showEdits;

    if (lnx.config.overrideLinearNav) {
      this.isLinearNav = false;
    }

    if (this.isLinearNav) {
      this.completeAllScreens = cr.getAttribute("completeallscreens") === "true";
    }

    this.isAutoCompletion = cr.getAttribute("autocompletion") === "true" && !lnx.scormApi.getIsCourseComplete();
    this.alwayshowRef = cr.getAttribute("alwayshowRef");
    lnx.scormApi.setMultiCompletionsStatus(!(cr.getAttribute("allowmulticompletions") === "false"));
    lnx.config.noAudio = !(cr.getAttribute("noaudio") === "false");
    lnx.config.isR2L = cr.getAttribute("isR2L") === "true";
    lnx.config.courseId = cr.getAttribute("id");
    this.toc = doc.getElementsByTagName("toc")[0];
    this.screens = doc.getElementsByTagName("screens")[0];
    this.homeId = cr.getAttribute("homeId"); //this.createTemplateMap(doc.getElementsByTagName("templates")[0]);

    this.parseToc(this.toc);
    this.createScreensMap(this.screens);
    this.assignHandlers();
    var hideAudio = lnx.config.noAudio;
    document.addEventListener("keyup", this.onKeyUp.bind(this));
    lnx.view.init(this.toc.childNodes, this.onNavigate, lnx.audio.getAudioOn(), hideAudio, this.isFrameWork2);

    if (this.alwayshowRef) {
      var n = this.navNodeMap[this.alwayshowRef];
      var n = this.screens.querySelector("#" + this.alwayshowRef); //var gp = n.parentNode.parentNode;

      var title = "Resources"; //gp.getAttribute("title");

      refGPnavId = this.alwayshowRef;
      lnx.view.refPanel.init(n, this.screensMap, title, refGPnavId);
    }

    this.tocPanel.init(this.toc);
    lnx.progBar.init(); // display first screen

    this.navigate(this.getStartNavId());
  },
  onKeyUp: function onKeyUp(e) {
    var self = this,
        key = normalizeKey(e.key);

    if (key === "ArrowUp" && e.shiftKey && e.altKey && this.jumpTo) {
      if (this.jumpToScreen(this.jumpTo)) {
        return;
      }
    }

    if (key === "ArrowDown" || key === "ArrowUp") {
      lnx.edits.handleKey.call(lnx.edits, key);
    } else if (key === "ArrowRight") {
      this.next();
    } else if (key === "ArrowLeft") {
      this.previous(e.shiftKey);
    }

    function normalizeKey(k) {
      return self.keys[k];
    }
  },
  jumpToScreen: function jumpToScreen(screen) {
    if (this.navNodeMap[screen]) {
      var navId = screen;
      this.farIndex = this.navNodes[navId];
      this.farNavId = navId;
      this.navigate(navId);
      return true;
    }

    return false;
  },
  getStartNavId: function getStartNavId() {
    var navId = null; // grab potential bookmark

    var data = lnx.scormApi.getSuspendData(); // first check is their a navid in the url querystring

    if (lnx.config.showScreen && this.navNodeMap[lnx.config.showScreen]) {
      navId = lnx.config.showScreen;
      this.farIndex = this.navNodes[navId];
    } else if (data && data.bookmark && this.navNodeMap[data.bookmark]) {
      navId = data.bookmark;
      this.farIndex = this.navNodes[navId];
    } else {
      // no bookmark so return navId of first toc node
      navId = this.navNodes[0].getAttribute("navId");
    }

    this.farNavId = navId;
    return navId;
  },
  getCurrNavId: function getCurrNavId(notLast, setFirst) {
    try {
      var result = "";

      if (!this.currNavId) {
        return this.navNodes[0].getAttribute("navId");
      }

      result = this.currNavId; // notLast can be specified - for bookmarking purposes 
      // so we do not return user to test feedback screen 
      // check if currNavId is last screen and return id for first screen if this is the case

      if (this.currNavId && notLast) {
        if (this.navNodes[this.currNavId] === this.navNodes.length - 1) {
          result = this.navNodes[0].getAttribute("navId");
        }
      } // if setFirst is true reset bookmark to first screen


      if (setFirst) {
        result = this.navNodes[0].getAttribute("navId");
      }

      return result;
    } catch (e) {
      lnx.util.onError(e, false, "getCurrNavId");
      return "";
    }
  },
  getNextPrevNavIds: function getNextPrevNavIds(prev) {
    var n = this.navNodes,
        id = this.currNavId,
        idx = this.navNodes[id],
        tId;

    while (idx > -1 && idx < n.length) {
      id = n[idx].getAttribute("navId");
      tId = n[idx].getAttribute("targetId");

      if (!this.screensMap[id]) {
        // must be a virtual question screen
        if (tId && (this.screensMap[tId].querySelector("[number]") || this.screensMap[tId].getAttribute("number"))) {
          return id;
        }
      } else if (this.screensMap[id].querySelector("[number]") || this.screensMap[id].getAttribute("number")) {
        return id;
      }

      idx = prev ? idx - 1 : idx + 1;
    }
  },
  getOrderedScreenList: function getOrderedScreenList(list) {
    //var a = [], b = [];
    for (var i = 0; i < list.length; i++) {
      list[i].idx = this.navNodes[list[i].id] + 1;
      list[i].hIdx = getHidx(this.navNodes[list[i].idx - 1]); // a.push(list[i].number);
    }

    list.sort(order); // for(var i=0;i<list.length;i++){           
    //     console.log(a[i], " : ", list[i].number, list[i].id);
    // }

    return list;

    function getHidx(n) {
      var p = n.parentNode,
          r = [];

      while (p) {
        r.unshift(getPosInList(p.childNodes, n));
        n = p;
        p = n.tagName.toLowerCase() === "nav" ? p.parentNode : null;
      }

      return r.join(".");

      function getPosInList(cn, n) {
        for (var i = 0; i < cn.length; i++) {
          if (cn[i] === n) {
            return i + 1;
          }
        }
      }
    }

    function order(a, b) {
      if (a.id === b.id) {
        // var c = a.rows ? a.rows.split(",")[0] : 0,
        //     d = b.rows ? b.rows.split(",")[0] : 0;
        // return c - d;
        return a.number - b.number;
      }

      return a.idx - b.idx;
    }
  },
  parseToc: function parseToc(n) {
    for (var i = 0, len = n.childNodes.length, navId; i < len; i++) {
      if (n.childNodes[i].nodeType !== 1) {
        continue;
      }

      navId = n.childNodes[i].getAttribute("navId"); // map all navigatable nodes including non-leaves

      this.navNodeMap[navId] = n.childNodes[i]; // property added for forced linear navigation feature
      // TO DO refactor various node maps/lists

      this.allNodes.push(n.childNodes[i]); // use navId as a hash for quick lookup of a nodes index

      this.allNodes[navId] = this.allNodes.length - 1;

      if (n.childNodes[i].hasChildNodes()) {
        this.parseToc(n.childNodes[i]);
      } else {
        // list of leaf navigatable nodes only
        // these nodes ultimately map to screens
        this.navNodes.push(n.childNodes[i]); // use navId as a hash for quick lookup of a nodes index

        this.navNodes[navId] = this.navNodes.length - 1;
      }
    }
  },
  createScreensMap: function createScreensMap(n) {
    for (var i = 0, len = n.childNodes.length; i < len; i++) {
      if (n.childNodes[i].nodeType === 1) {
        this.screensMap[n.childNodes[i].getAttribute("id")] = n.childNodes[i];
      }
    }
  },
  createTemplateMap: function createTemplateMap(n) {
    for (var i = 0, len = n.childNodes.length; i < len; i++) {
      if (n.childNodes[i].nodeType === 1) {
        this.templateMap[n.childNodes[i].nodeName] = n.childNodes[i].firstChild;
      }
    }
  },
  assignHandlers: function assignHandlers() {
    var exitBtn = document.getElementById("exitBtn");
    lnx.util.updateEventListener(exitBtn, "click", this.onExitCourse);
    var audioBtn = document.getElementById("audioBtn");
    lnx.util.updateEventListener(audioBtn, "click", this.onAudioUpdate); // catch pagehide events so we can kill any current audio

    lnx.util.updateEventListener(window, "pagehide", this.onPageHide);

    if (this.alwayshowRef && this.helpRefIds) {
      var helpBtn = document.getElementById("helpBtn");
      helpBtn.setAttribute("data-navId", "resources");
      lnx.util.updateEventListener(helpBtn, "click", function () {
        lnx.view.refPanel.show();
      });
    }
  },
  onPageHide: function onPageHide(e) {
    lnx.audio.stopAudio();
  },
  onExitCourse: function onExitCourse(e) {
    // top.window.close()?
    lnx.exit(true);
    e = e || window.event;
    e.stopPropagation ? e.stopPropagation() : e.cancelBubble = true;
  },
  onAudioUpdate: function onAudioUpdate(e) {
    if (lnx.nav.getIsScreenLocked()) return;
    e = e || window.event;
    var target = e.target || e.srcElement;
    var isOn = lnx.audio.getAudioOn();
    isOn = !isOn;
    lnx.audio.setAudioOn(isOn);

    if (!isOn) {
      lnx.audio.stopAudio();
    }

    lnx.view.updateAudioIcon(target, isOn);
    e.stopPropagation ? e.stopPropagation() : e.cancelBubble = true;
  },
  onNavigate: function onNavigate(e) {
    // if(typeof e === "string"){
    //     lnx.nav.navigate(e);
    //     return;
    // }
    e = e || window.event;
    var target = e.target || e.srcElement;
    lnx.nav.navigate(target.getAttribute("data-navId"));
    e.stopPropagation ? e.stopPropagation() : e.cancelBubble = true;
    /*e.preventDefault ?
    e.preventDefault() : ( e.returnValue = false );*/
  },
  onCertify: function onCertify() {
    lnx.scormApi.lmsCompleteCourse(true);
  },
  onTestResult: function onTestResult(pass) {
    lnx.scormApi.lmsCompleteCourse(pass);
    lnx.scormApi.lmsLogTestResult(pass);
    this.next();
  },
  onRetakeTest: function onRetakeTest(id) {
    id = id || this.TEST_RETAKE_ID;
    this.navigate(id);
  },
  next: function next() {
    this.navigate("next");
  },
  previous: function previous(shift) {
    this.navigate("prev", false, shift);
  },
  goToScreen: function goToScreen(i) {
    var n = this.navNodes[i];

    if (n) {
      this.navigate(n.getAttribute("navId"));
    }
  },
  navigate: function navigate(navId, viaEdits, isShift) {
    var node,
        idx = this.navIndex;
    var nodesLength = this.navNodes.length;
    var self = this;
    lnx.edits.tearDown();
    lnx.view.refPanel.show(false);

    if (!navId) {
      // exit as required arg is missing
      // user has almsost certainly clicked between navigation buttons on navigation container
      // which has single listener attached and utilizes bubbling to catch nav clicks 
      return;
    }

    if (!this.currNode) {// must be first visit to a screen;
      // this.currNode = this.navNodes[0];
      // this.navIndex = 0;
    }

    var prevNode = this.currNode;
    var prevType = this.currNodeType;

    if (navId === "prev") {
      if (!this.navIndex) {
        // navIndex == 0 when at first screen
        if (!isShift) {
          // prevent reversing (and unlocking of linear nav) unless shift is down
          return;
        }

        node = this.navNodes[this.navNodes.length - 1];
        this.navIndex = this.navNodes.length - 1;
      } else {
        node = this.navNodes[--this.navIndex];
      }
    } else if (navId === "next") {
      if (!(this.navIndex + 1 < nodesLength)) {
        // node = this.navNodes[0];
        // this.navIndex = 0;
        node = this.navNodes[this.navIndex];
        return;
      } else {
        node = this.navNodes[++this.navIndex];
      }
    } else {
      // passed 'ordinary' navId so find its matching node
      node = this.navNodeMap[navId]; // make sure we've got a node before continuing

      if (!node) {
        // check is it custom link
        if (navId.substring(0, 5) === "link:") {
          window.open(navId.substring(5));
          return;
        }

        lnx.util.onError("Failed to find navNode for navId: " + navId);
        return;
      }

      while (node.hasChildNodes()) {
        // it's a nav screen ancestor so drill down
        node = node.firstChild;
      } // use named prop added to array to find screen's index without looping


      var tIndex = this.navNodes[node.getAttribute("navId")]; //special case added - never exit if screen to navigate to is home screen

      if (this.isLinearNav && tIndex > this.farIndex + 1 && tIndex !== this.navNodes[this.homeId]) {
        // check if overlay reference panel should be shown
        if (this.alwayshowRef === node.getAttribute("navId")) {
          lnx.view.refPanel.show();
        }

        return;
      }

      this.navIndex = tIndex; // sanity check

      if (!this.navIndex && this.navIndex !== 0) {
        lnx.util.onError("undefined screen index for navId: " + navId); // reset to current index

        this.navIndex = idx;
        return;
      }
    } // ignore navigation event if screen is 'must complete' and not completed
    // if((!isPriorNode()) && this.isLinearNav && prevNode && (prevNode.getAttribute("mustComplete") === "true")){
    //     if(!this.isScreenComplete(prevNode.getAttribute("navId"))){
    //         this.navIndex = idx;
    //         return;
    //     }
    // }
    // ignore navigation event if screen is 'must complete' and not completed


    if (!isPriorNode() && this.isLinearNav && (prevNode && prevNode.getAttribute("mustComplete") === "true" || this.completeAllScreens) && this.screenLocked //(((prevNode && prevNode.getAttribute("mustComplete") === "true") || !prevNode) || this.completeAllScreens)
    ) {
      if (prevNode && !this.isScreenComplete(prevNode.getAttribute("navId"))) {
        // if(!prevNode || !this.isScreenComplete(prevNode.getAttribute("navId"))){
        this.navIndex = idx;
        return;
      }
    } // check if nav action should be ignored - screen will display content as a virtual screen/nav event


    if (navId === "next" && this.insertVirtual(prevType, navId)) {
      this.navIndex = idx;
      this.screenLocked = getMustCompleteBeforeAdvance();

      if (this.screenLocked) {
        lnx.view.disableFwdButton();
      }

      return;
    }

    if (navId === "prev" && this.insertVirtual(prevType, navId, node.getAttribute("type"))) {
      this.navIndex = idx;
      return;
    } // keep record of current navId for bookmarking


    this.currNavId = node.getAttribute("navId");
    this.currNodeType = node.getAttribute("type");
    this.currNode = node;

    if (this.navIndex > this.farIndex) {
      this.farIndex = this.navIndex;
      this.farNavId = this.currNavId;
      lnx.scormApi.uploadSuspendData(false, true);
    } // kill any audio playing on old screen


    lnx.audio.stopAudio();
    var isLast = this.updateTerminalState(this.navIndex, nodesLength);
    if (isLast && this.isAutoCompletion) lnx.scormApi.lmsCompleteCourse(true); // if linear navigation style pass in index of furthest point in allNodes array

    var linear = this.isLinearNav ? {
      i: this.allNodes[this.farNavId],
      n: this.allNodes
    } : null;
    this.screenLocked = getMustCompleteBeforeAdvance();
    lnx.view.update(node, this.terminals, this.screensMap, linear, this.screenLocked, navId);
    if (!viaEdits) lnx.edits.setUp();
    lnx.overlayMan.hide();
    lnx.nav.tocPanel.hide();
    lnx.progBar.positionSlider(this.navIndex);

    function getMustCompleteBeforeAdvance() {
      if (!self.isLinearNav) return false;

      if (node.getAttribute("mustComplete") === "true" || self.completeAllScreens) {
        if (lnx.scormApi.getIsCourseComplete()) return false;
        if (isPriorNode()) return false;
        if (self.isScreenComplete(self.currNavId)) return false;
        /**/

        if (self.completeAllScreens && !hasAudio()) {
          return false;
        }

        if (!self.currNode) {
          // must be first screen
          return false;
        }

        return true;
      } else {
        return false;
      }
    }

    function isPriorNode() {
      return self.navIndex < self.farIndex;
    }

    function hasAudio() {
      var screen = self.getScreenFromNodeId(self.currNavId);

      if (!screen) {
        return false;
      }

      if (lnx.config.noAudio) {
        return false;
      }

      var s = screen.outerHTML;
      var r = /data-(auto)?Audio="(\w+)\"/i.exec(s);

      if (r && r[2] != "0") {
        return true;
      } // if( /data-autoAudio="\w+\"/.test( s )){
      //     return true;
      // }
      //  if( /data-audio="\w+\"/.test( s )){
      //     return true;
      // }


      return false;
    }
  },
  insertVirtual: function insertVirtual(type, dir) {
    if (!this.mustCompleteMap[type]) return false; // Abbott now want this to always operate
    // if(lnx.scormApi.getIsCourseComplete()) return false;
    // if(!(this.navIndex > this.farIndex)) return false;

    var screen = lnx.view.getCurrentScreen();

    if (!(screen && screen.insertVirtualScreen)) {
      // problem, allow navigation anyway
      return false;
    }

    if (!screen.insertVirtualScreen(dir)) return false;

    if (screen.getResId) {
      lnx.view.updateDebugBox(null, screen.getResId());
    }

    return true;
  },
  isScreenComplete: function isScreenComplete(id) {
    return lnx.cache.getValue("complete", id);
  },
  getIsScreenLocked: function getIsScreenLocked() {
    return this.screenLocked;
  },
  setIsScreenLocked: function setIsScreenLocked(v) {
    return this.screenLocked = v;
  },
  getScreenFromNodeId: function getScreenFromNodeId(id) {
    return this.screensMap[id];
  },
  getCurrentNode: function getCurrentNode() {
    return this.currNode;
  },
  getCurrentScreenId: function getCurrentScreenId() {
    if (!this.currNode) return null;
    return this.currNode.getAttribute("navId");
  },
  getCurrentNavIds: function getCurrentNavIds(id) {
    var n;

    if (id) {
      n = id;
    } else if (!this.currNode) {
      return null;
    } else {
      n = this.currNode;
    }

    var a = [null, null, null];
    a[2] = n.getAttribute("navId");
    n = this.currNode.parentNode;
    if (n) a[1] = n.getAttribute("navId");
    n = n.parentNode;
    if (n) a[0] = n.getAttribute("navId");
    return a;
  },
  getTopicsStatus: function getTopicsStatus() {
    var results = {};
    var m = this.toc.querySelectorAll("nav[required]");
    var isLocked = this.isLinearNav && !lnx.scormApi.getIsCourseComplete();
    var lastTopicComplete = false; // var homeScreenIndex = this.navNodes[this.homeId];
    // var autoAvailable = this.farIndex >= homeScreenIndex; 

    for (var i = 0; i < m.length; i++) {
      var mo = results[m[i].getAttribute("navId")] = {
        topics: {},
        complete: true,
        available: true,
        id: m[i].getAttribute("navId"),
        index: i + 1
      };
      var t = m[i].childNodes;
      var atLeastOneTopicAvailble = false;

      for (var j = 0; j < t.length; j++) {
        var tId = t[j].getAttribute("navId");
        var lastScreenIndexInTopic = this.navNodes[t[j].lastChild.getAttribute("navId")];
        var firstScreenIndexInTopic = this.navNodes[t[j].firstChild.getAttribute("navId")];
        var to = mo.topics[tId] = {};
        to.complete = lastScreenIndexInTopic <= this.farIndex ? true : false;
        to.available = firstScreenIndexInTopic <= this.farIndex ? true : false;
        to.firstScreenId = t[j].firstChild.getAttribute("navId");

        if (!to.available && (i == 0 && j == 0 || !isLocked)) {
          to.available = true;
        } //if previous topic completed, make this one available


        if (!to.available && lastTopicComplete) {
          to.available = true;
        }

        if (to.available) {
          atLeastOneTopicAvailble = true;
        }

        if (!to.complete) {
          mo.complete = false;
          lastTopicComplete = false;
        } else {
          lastTopicComplete = true;
        }
      }

      if (!atLeastOneTopicAvailble) {
        mo.available = false;
      }
    }

    return results;
  },
  isFarIndexBeforeHomeScreen: function isFarIndexBeforeHomeScreen() {
    var homeScreenIndex = this.navNodes[this.homeId];
    return this.farIndex < homeScreenIndex;
  },
  goToHomeScreen: function goToHomeScreen() {
    this.navigate(this.homeId);
  },
  updateTerminalState: function updateTerminalState(idx, len) {
    // function assumes > 1 screen
    if (idx === 0) {
      // first screen
      this.terminals.isFirst = true;
      this.terminals.isLast = false;
    } else if (idx + 1 === len) {
      // last screen
      this.terminals.isLast = true;
      this.terminals.isFirst = false;
    } else {
      // neither first or last screen
      this.terminals.isLast = this.terminals.isFirst = false;
    }

    return this.terminals.isLast;
  },
  destroy: function destroy() {
    var exitBtn = document.getElementById("exitBtn");
    lnx.util.updateEventListener(exitBtn, "click", this.onExitCourse, true);
    var audioBtn = document.getElementById("audioBtn");
    lnx.util.updateEventListener(audioBtn, "click", this.onAudioUpdate, true);
    lnx.util.updateEventListener(window, "pagehide", this.onPageHide, true);
    this.content = this.toc = this.screens = this.navNodes = this.navNodeMap = this.screensMap = this.templateMap = null;
    lnx.view.destroy(this.onNavigate);
  }
};
lnx.nav.tocPanel = {
  overlay: null,
  closeBtn: null,
  payLoad: null,
  html: "",
  currNodes: [],
  initialized: false,
  init: function init(n) {
    var self = this;
    var btn = document.body.querySelector(".tocBtn");
    if (!btn) return;
    btn.onclick = this.show.bind(this);
    this.overlay = document.body.querySelector("div.overlay.tocPanel");

    this.overlay.querySelector("img").onclick = function () {
      self.hide();
    }; // this.overlay.onclick = function(e){
    //     self.selection(e);
    // };


    this.overlay.addEventListener("click", this.selection.bind(this));
    this.payLoad = this.overlay.querySelector("div:nth-of-type(2)");
    this.parseToc(n, 1);
    this.payLoad.innerHTML = this.html;
    this.initialized = true;
  },
  parseToc: function parseToc(n, level) {
    for (var i = 0, len = n.childNodes.length; i < len; i++) {
      if (n.childNodes[i].nodeType !== 1) {
        continue;
      }

      var nd = n.childNodes[i];
      this.html += '<p class="tocListL' + level + '" navId="' + nd.getAttribute("navId") + '">' + nd.getAttribute("title") + '</p>';

      if (n.childNodes[i].hasChildNodes() && level < 2) {
        this.parseToc(n.childNodes[i], 2);
      }
    }
  },
  selection: function selection(e) {
    var navId = e.target.getAttribute("navId");

    if (navId) {
      lnx.nav.navigate(navId);
      this.hide();
    }
  },
  show: function show() {
    if (!this.initialized) return;

    if (this.currNodes.length) {
      this.currNodes[0].classList.remove("tocItemOn");
      this.currNodes[1].classList.remove("tocItemOn");
    }

    var ids = lnx.nav.getCurrentNavIds();
    var p = this.payLoad.querySelector('p[navId="' + ids[0] + '"]');
    p && p.classList.add("tocItemOn");
    this.currNodes[0] = p;
    p = this.payLoad.querySelector('p[navId="' + ids[1] + '"]');
    p && p.classList.add("tocItemOn");
    this.currNodes[1] = p;
    this.overlay.style.display = "block";
  },
  hide: function hide() {
    if (!this.initialized) return;
    this.overlay.style.display = "none";
  }
}; // this object handles rendering of navigation menu, content title and
// and content screens

lnx.view = {
  // Strings corresponding to main html nav and view element id values
  ELM_IDS: ["mainNav", "subNav", "screenNav", "screenNavFooter", "screenFrame", "contentTitle", "moduleTitle"],
  ELM_IDS2: ["fwdBtn", "backBtn", "fwdBtnF", "backBtnF", "screenFrame", "contentTitle", "moduleTitle"],
  // holds refs to main html navigation and view elements
  elements: {},
  courseTitleElm: null,
  // arrays of xml navigation nodes used to build navigation menu
  mainNavNodes: [],
  subNavNodes: [],
  screenNavNodes: [],
  // map of screen types
  screenTypes: null,
  // ref to div element within which screens of content display 
  screenFrame: null,
  // ref to current screen js object
  currScreen: null,
  // cache for fwd btn refs
  disabledFwdBtns: null,
  allwaysShowRefNode: null,
  toc: null,
  contentTitleFontSize: null,
  debugBox: null,
  init: function init(tocNodes, navHandler, isAudioOn, hideAudio, isFrameWork2) {
    // each type has a corresponding js object which handles interaction etc
    this.screenTypes = {
      "test": lnx.assessment,
      "feedback": lnx.assessmentFeedback,
      "testIntro": lnx.assessmentIntro,
      "slideshow": lnx.slideShow,
      "scenario": lnx.scenario,
      "accordian": lnx.accordian,
      "panel": lnx.panel,
      "video": lnx.video,
      "multiPanel": lnx.multiPanel,
      "learnHow": lnx.learnHow,
      "matching": lnx.matching,
      "imgSequence": lnx.imgSequence,
      "basic": lnx.basic,
      "learningCheck": lnx.learningCheck,
      "selectReveal": lnx.selectReveal,
      "selectRevealEmail": lnx.selectRevealEmail,
      "hasVirtualScreenOverlay": lnx.hasVirtualScreenOverlay,
      "acknowledge": lnx.acknowledge,
      "autoVideo": lnx.autoVideo,
      "clickAndPopup": lnx.clickAndPopup,
      "clickAndPopupSelection": lnx.clickAndPopupSelection,
      "videoScenario": lnx.videoScenario,
      "scenarioWithVirtualScreens": lnx.scenarioWithVirtualScreens,
      "introPanel": lnx.introPanel,
      "moveUp": lnx.moveUp,
      "steps": lnx.steps,
      "basicWithVideo": lnx.basicWithVideo,
      "dialogueWithVirtualScreens": lnx.dialogueWithVirtualScreens,
      "certification": lnx.certification,
      "verticalScenario": lnx.verticalScenario,
      "dialogue": lnx.dialogueWithVirtualScreens,
      "stages": lnx.stages,
      "home": lnx.home,
      "flashCard": lnx.flashCard
    };

    if (isFrameWork2) {
      this.ELM_IDS = this.ELM_IDS2;
    }

    this.debugBox = document.getElementById("debugBox");

    if (lnx.config.ignoreLMS && !lnx.config.screenShotMode && this.debugBox && !lnx.config.hideDebugBox) {
      this.debugBox.style.display = "block";
    } // get refs to main html doc elements and store in elements object


    for (var i = 0, len = this.ELM_IDS.length; i < len; i++) {
      this.elements[this.ELM_IDS[i]] = document.getElementById(this.ELM_IDS[i]);
    } // shortcut reference


    this.screenFrame = this.elements.screenFrame;
    this.setCourseTitle();
    this.contentTitleFontSize = window.getComputedStyle(this.elements.contentTitle).fontSize;
    this.toc = this.buildMenu(tocNodes, navHandler, isFrameWork2);

    if (!isAudioOn || hideAudio) {
      var audioBtn = document.getElementById("audioBtn");
      this.updateAudioIcon(audioBtn, isAudioOn, hideAudio);
    }

    lnx.home.createTocModal();
  },
  showToc: function showToc(hide) {
    // if(hide === false){
    //     toc.style.display = "none";
    // } else {
    //     toc.style.display = "block";
    // }
    //lnx.nav.navigate(lnx.nav.homeId);
    lnx.home.showTocModal();
  },
  setAlwaysShowRefOn: function setAlwaysShowRefOn(id) {//this.allwaysShowRefNode = this.elements.mainNav.querySelector('li[data-navid="' + id + '"]');
  },
  updateDebugBox: function updateDebugBox(node, resId) {
    var d = this.debugBox;

    if (d) {
      if (resId) {
        d.innerHTML = resId;
        return;
      }

      d.innerHTML = node.getAttribute("resId");

      if (node.getAttribute("type") === "dialogueWithVirtualScreens") {
        d.innerHTML = lnx.dialogueWithVirtualScreens.getResId();
      }
    }
  },
  setCourseTitle: function setCourseTitle() {
    this.courseTitleElm = document.getElementById("courseTitle");
    this.courseTitleElm.innerHTML = lnx.localization.getLocalString("s9");
    var a = ["tocBtn", "helpBtn", "refBtn", "audioBtn", "exitBtn"];

    for (var i = 0; i < a.length; i++) {
      var elm = document.getElementById(a[i]);

      if (!elm) {
        continue;
      }

      elm.setAttribute("title", lnx.localization.getLocalString("s" + (i + 14)));
    }
  },
  updateTitle: function updateTitle(node) {
    var title = node.getAttribute("displayTitle") || node.getAttribute("title");
    var mTitle;
    var fontClass = null;

    while (node && !title) {
      node = node.parentNode;
      title = node.getAttribute("displayTitle") || node.getAttribute("title");
      fontClass = node.getAttribute("fontSize");
    }

    if (title) {
      var tElm = this.elements.contentTitle;
      tElm.style.fontSize = this.contentTitleFontSize;

      if (node.parentNode) {
        mTitle = node.parentNode.getAttribute("title") || "";
      }

      this.elements.moduleTitle && (this.elements.moduleTitle.innerHTML = mTitle);

      if (fontClass) {
        title = "<span class=\"" + fontClass + "\">" + title + "</span>";
      }

      tElm.innerHTML = title;
      var lh = parseInt(window.getComputedStyle(tElm).lineHeight);
      var i = 1;
      var fs = parseInt(this.contentTitleFontSize);

      while (parseInt(window.getComputedStyle(tElm).height) > lh) {
        tElm.style.fontSize = fs - i++ + "px";

        if (i > 100) {
          break;
        }
      }
    }
  },
  updateCourseTitleColor: function updateCourseTitleColor(idx) {
    this.courseTitleElm.className = ".titleText titleTextColor" + idx;
  },
  buildMenu: function buildMenu(tocNodes, navHandler, isFrameWork2) {
    // not using this toc menu for framework 3
    //return;
    if (isFrameWork2) {
      return this.buildMenu2(tocNodes, navHandler, isFrameWork2);
    }

    this.buildMainNavMenu(tocNodes);
    this.buildSubNavMenu(this.mainNavNodes[0].childNodes);
    this.buildScreensNavMenu(this.subNavNodes[0].childNodes);
    this.updateTitle(this.mainNavNodes[0].childNodes[0]);
    var evtTargs = [this.elements.mainNav, this.elements.subNav, this.elements.screenNav, this.elements.screenNavFooter]; // add event listeners

    lnx.util.updateEventListener(evtTargs, "click", navHandler);
  },
  buildMenu2: function buildMenu2(tocNodes, navHandler, isFrameWork2) {
    var self = this;
    var tocCon = this.buildVerticalMenu(tocNodes);
    var toc = tocCon.parentNode;
    var mods = tocCon.querySelectorAll(".moduleItem");
    var curMod = null;
    var openToc = document.getElementById("tocBtn");
    var closeToc = toc.querySelectorAll(".tocHeader > img")[1];
    lnx.util.updateEventListener(tocCon, "click", viewHandler);
    lnx.util.updateEventListener([openToc, closeToc], "click", showToc);
    var navBtns = document.documentElement.querySelectorAll("#player .navBtn");
    lnx.util.updateEventListener(Array.prototype.slice.call(navBtns), "click", viewHandler);
    lnx.util.updateEventListener(toc.querySelector("div.popUnderlay"), "click", onClickUnderlay);
    return toc;

    function onClickUnderlay(e) {
      if (e.target === e.currentTarget) {
        showToc(e);
      }
    }

    function showToc(e) {
      // if(e.target.getAttribute("data-show")){
      //     toc.style.display = "block";
      // } else {
      //     toc.style.display = "none";
      // }
      //lnx.nav.navigate(lnx.nav.homeId);
      lnx.home.showTocModal(e);
    }

    function viewHandler(e) {
      var targ = e.target;

      if (targ.nodeName.toLowerCase() === "img") {
        targ = targ.parentNode;
      }

      var navId = targ.getAttribute("data-navId");

      if (navId && navId.substring(0, 1).toLowerCase() === "a") {
        if (curMod && curMod.getAttribute("data-navid") === navId) {
          updateTocView(null, curMod);
          updateIcon(curMod, false);
          curMod = null;
          return;
        }

        for (var i = 0, len = mods.length; i < len; i++) {
          if (mods[i].getAttribute("data-navid") === navId) {
            updateTocView(mods[i], curMod);
            updateIcon(curMod, false);
            curMod = mods[i];
            updateIcon(curMod, true);
            break;
          }
        }
      } else {
        navHandler(e);
      }
    }

    function updateTocView(show, hide) {
      show && (show.nextElementSibling.style.display = "block");
      hide && (hide.nextElementSibling.style.display = "none");
    }

    function updateIcon(elm, show) {
      if (elm) {
        var img = elm.querySelector("img");

        if (img) {
          var url = show ? "images/modIconD.png" : "images/modIcon.png";
          img.src = url;
        }
      }
    }
  },
  buildVerticalMenu: function buildVerticalMenu(tocNodes) {
    var mHTML = '<div class="moduleCon"><div class="moduleItem" data-navId="$a"><img src="images\/modIcon.png"/>$b<\/div><div class="topicCon">';
    var tHTML = '<div class="topicItem" data-navId="$a">$b<hr/><\/div>';
    var eHTML = '<\/div><\/div>';
    var s = '';

    for (var i = 0; i < tocNodes.length; i++) {
      s += genModule(tocNodes[i]);
    } //console.log(s);


    var tocCon = window.document.querySelector(".tocContainer");
    tocCon.innerHTML = s;
    return tocCon;

    function genModule(n) {
      var s = mHTML.replace("$a", n.getAttribute("navId")).replace("$b", n.getAttribute("title"));
      s += genTopics(n.childNodes);
      s += eHTML;
      return s;
    }

    function genTopics(n) {
      var s = '';

      for (var i = 0; i < n.length; i++) {
        if (n[i].nodeType === n[i].COMMENT_NODE) {
          continue;
        }

        s += tHTML.replace("$a", n[i].getAttribute("navId")).replace("$b", n[i].getAttribute("title"));
      }

      return s;
    }
  },
  buildMainNavMenu: function buildMainNavMenu(nodes) {
    var NAV_LEVEL = 1;
    var iHTML = this.genHTML(nodes, this.mainNavNodes, NAV_LEVEL);
    this.elements.mainNav.innerHTML = iHTML; // workaround for ie 8's lack of last-child css
    //this.elements.mainNav.lastChild.style.border = "none";
  },
  buildSubNavMenu: function buildSubNavMenu(nodes) {
    var NAV_LEVEL = 2;
    var iHTML = this.genHTML(nodes, this.subNavNodes = [], NAV_LEVEL);
    this.elements.subNav.innerHTML = iHTML;
  },
  buildScreensNavMenu: function buildScreensNavMenu(nodes, firstElm) {
    var NAV_LEVEL = 3; // images referenced in css

    var prevHTML = "<div data-navId='prev' class='screenBtnPrev'  unselectable='on'></div>";
    var nextHTML = "<div data-navId='next' class='screenBtnNext'  unselectable='on'></div>";
    var f = lnx.config.isR2L ? "genR2LScreenHTML" : "genHTML";
    var iHTML = prevHTML + this[f](nodes, this.screenNavNodes = [], NAV_LEVEL) + nextHTML; // header nav buttons

    this.elements.screenNav.innerHTML = iHTML; // footer nav buttons

    this.elements.screenNavFooter.innerHTML = iHTML;
  },
  genHTML: function genHTML(nodes, list, level) {
    var navId,
        iHTML = "";
    var pageNum = 0;

    for (var i = 0; i < nodes.length; i++) {
      if (nodes[i].nodeType !== 1 || !(navId = nodes[i].getAttribute("navId"))) {
        continue;
      }

      switch (level) {
        case 1:
          {
            iHTML += "<li data-navId='" + navId + "' class='mainNavLi mainNavLiColor" + i + "'>" + nodes[i].getAttribute("title") + "<\/li>";
            break;
          }

        case 2:
          {
            iHTML += "<li data-navId='" + navId + "' class='subNavLi'>" + nodes[i].getAttribute("title") + "<\/li>";
            break;
          }

        case 3:
          {
            iHTML += "<div data-navId='" + navId + "' class='screenBtn'>" + ++pageNum + "<\/div>";
            break;
          }
      }

      list.push(nodes[i]);
    }

    return iHTML;
  },
  genR2LScreenHTML: function genR2LScreenHTML(nodes, list, level) {
    // ToDo add second loop as pageNum will be incorrect if xml includes non element nodes
    var navId,
        iHTML = "",
        pageNum = nodes.length;

    for (var i = nodes.length - 1; i >= 0; i--) {
      if (nodes[i].nodeType !== 1 || !(navId = nodes[i].getAttribute("navId"))) {
        continue;
      }

      iHTML += "<div data-navId='" + navId + "' class='screenBtn'>" + pageNum-- + "<\/div>";
      list.push(nodes[i]);
    }

    return iHTML;
  },
  update: function update(node, terminals, screensMap, linear, disableFwdBtn, navId) {
    if (disableFwdBtn) {
      this.disableFwdButton();
    }

    this.updateScreenView(node, screensMap, navId);
    this.updateNavView(node, terminals, linear, disableFwdBtn);
  },
  disableFwdButton: function disableFwdButton() {
    var audioBtn = document.getElementById("audioBtn");
    this.updateAudioIcon(audioBtn, true);
    lnx.audio.setAudioOn(true);
    var e = lnx.view.elements;
    e.fwdBtn.classList.add("fwdBtnDisabled");
    e.fwdBtnF.classList.add("fwdBtnDisabled");
    this.disabledFwdBtns = [e.fwdBtn, e.fwdBtnF];
  },
  onScreenComplete: function onScreenComplete() {
    if (this.disabledFwdBtns) {
      if (lnx.nav.isFrameWork2) {
        if (!lnx.nav.terminals.isLast) {
          this.disabledFwdBtns[0].classList.remove("fwdBtnDisabled");
          this.disabledFwdBtns[1].classList.remove("fwdBtnDisabled");
        } else {
          // don't null disabledFwdBtns array
          return;
        }
      } else {
        this.disabledFwdBtns[0].className = this.disabledFwdBtns[1].className = "screenBtnNext";
      }
    }

    this.disabledFwdBtns = null;
  },
  getCurrentScreen: function getCurrentScreen() {
    return this.currScreen;
  },
  updateScreenView: function updateScreenView(node, screensMap, origNavId) {
    var type = node.getAttribute("type");
    var navId = node.getAttribute("navId");
    this.currScreen && this.currScreen.destroy(type, navId);
    this.currScreen = this.screenTypes[type];

    if (!this.currScreen) {
      lnx.util.onError("No screenType for type attrib: " + type);
      return;
    } // allow a screen to render its own content accross multiple screens
    // currently only assessment does this


    if (!this.currScreen.hasContent()) {
      var iHTML = this.getScreenContent(node, screensMap);
      this.screenFrame.innerHTML = iHTML;
      this.screenFrame.style.opacity = "0";
      var tl = new TimelineMax();
      tl.to(this.screenFrame, 1, {
        opacity: 1
      });
    } // rename to process rather than init


    this.screenTypes[type].init(node, this.screenFrame.firstChild, this.screenFrame, origNavId);
    lnx.audio.autoPlayAudio(node, this.screenFrame.firstChild);
  },
  // returns string of html markup keyed off toc node attribute
  getScreenContent: function getScreenContent(node, screensMap) {
    // Use a targetId(when available) or a navId to map a navigation node to a content screen.
    // Currently only the assessment uses targetIds.
    // targetIds allow for multiple navigation nodes to point to a single target content screen, in this case a form.
    // The individual form questions are essentially virtual screens - they appear in the navigation xml 
    // but do not map to seperate content screen elements. 
    // A targetId therefore allows a group of xml navigation nodes point to the same resource.
    // screens are ultimatley be rendered as html markup
    var sId = node.getAttribute("targetId") ? node.getAttribute("targetId") : node.getAttribute("navId");

    if (!sId) {
      // may be a screen
      if (node && node.getAttribute("class").indexOf("helpPage") !== -1) {
        sId = node.getAttribute("id");
      }
    }

    var n = screensMap[sId];

    if (!n) {
      return "";
    } else {
      var r = lnx.util.XMLToString(n);
      return r;
    }
  },
  updateNavView: function updateNavView(node, terminals, linear, disableFwdBtn) {
    if (lnx.nav.isFrameWork2) {
      this.updateNavView2(node, terminals, linear, disableFwdBtn);
      return;
    }

    var subNavNode = node.parentNode;
    var mainNavNode = subNavNode.parentNode;
    var navId = mainNavNode.getAttribute("navId");
    var elms = this.elements.mainNav.childNodes;
    var elmsFooter;
    var mainNavColorIndex; // is item enabled	

    var enabled = false;
    this.disabledFwdBtns = null;

    for (var i = elms.length - 1; i > -1; i--) {
      var nId = elms[i].getAttribute("data-navId");

      if (linear && !enabled && linear.n[nId] < linear.i) {
        enabled = true;
      }

      if (nId === navId) {
        elms[i].className = "mainNavLiOn mainNavLiColor" + i;
        mainNavColorIndex = i;
      } else {
        var cn = "mainNavLi mainNavLiColor" + i;

        if (linear && !enabled) {
          cn = "mainNavLiDisabled";
        }

        elms[i].className = cn;
      }
    }

    this.buildSubNavMenu(mainNavNode.childNodes);
    navId = subNavNode.getAttribute("navId");
    elms = this.elements.subNav.childNodes;
    enabled = false;

    for (var i = elms.length - 1; i > -1; i--) {
      var nId = elms[i].getAttribute("data-navId");

      if (linear && !enabled && linear.n[nId] < linear.i) {
        enabled = true;
      }

      if (nId === navId) {
        elms[i].className = "subNavLiOn" + (lnx.config.courseId === "aac" ? " bgColor" + mainNavColorIndex : "");
      } else {
        var cn = "subNavLi subNavLi" + mainNavColorIndex;

        if (linear && !enabled) {
          cn = "subNavLiDisabled";
        }

        elms[i].className = cn;
      }
    }

    this.buildScreensNavMenu(subNavNode.childNodes);
    navId = node.getAttribute("navId");
    elms = this.elements.screenNav.childNodes;
    elmsFooter = this.elements.screenNavFooter.childNodes;
    var prevIdx, nextIdx;
    enabled = false; // screenNavFooter elements mirror screenNav elements

    for (var i = elms.length - 1; i > -1; i--) {
      if (elms[i].getAttribute("data-navId") === "prev") {
        prevIdx = i;
        continue;
      }

      if (elms[i].getAttribute("data-navId") === "next") {
        nextIdx = i;
        continue;
      }

      var nId = elms[i].getAttribute("data-navId");

      if (linear && !enabled && linear.n[nId] < linear.i) {
        enabled = true;
      }

      if (nId === navId) {
        elms[i].className = elmsFooter[i].className = "screenBtnOn";
      } else {
        /*if (elms[i].className === "screenBtnOn") {*/
        var cn = "screenBtn";

        if (linear && !enabled) {
          cn = "screenBtnDisabled";
        }

        elms[i].className = elmsFooter[i].className = cn;
      }
    }

    if (terminals.isFirst) {
      elms[prevIdx].className = elmsFooter[prevIdx].className = "hide";
      elms[nextIdx].className = elmsFooter[nextIdx].className = "screenBtnNext";
    } else if (terminals.isLast) {
      elms[nextIdx].className = elmsFooter[nextIdx].className = "hide";
      elms[prevIdx].className = elmsFooter[prevIdx].className = "screenBtnPrev";
    } else {
      elms[prevIdx].className = elmsFooter[prevIdx].className = "screenBtnPrev";
      elms[nextIdx].className = elmsFooter[nextIdx].className = "screenBtnNext";
    } // show disabled fwd btn for screens whose activity must be completed before u can navigate


    if (disableFwdBtn && !terminals.isLast) {
      elms[nextIdx].className = elmsFooter[nextIdx].className = "screenBtnNextDisabled";
      this.disabledFwdBtns = [elms[nextIdx], elmsFooter[nextIdx]];
    } // uncomment following to update color of course tilte to match section color
    //this.updateCourseTitleColor(mainNavColorIndex);


    this.updateTitle(node);

    if (this.allwaysShowRefNode && this.allwaysShowRefNode.classList.contains("mainNavLiDisabled")) {
      this.allwaysShowRefNode.classList.remove("mainNavLiDisabled");
      this.allwaysShowRefNode.classList.add("mainNavLi");
    }
  },
  updateNavView2: function updateNavView2(node, terminals, linear, disableFwdBtn) {
    this.updateTitle(node);
    this.updateDebugBox(node);
    var e = lnx.view.elements;

    if (terminals.isFirst) {
      e.backBtn.classList.add("backBtnDisabled");
      e.backBtnF.classList.add("backBtnDisabled");
      e.fwdBtn.classList.remove("fwdBtnDisabled");
      e.fwdBtnF.classList.remove("fwdBtnDisabled");
    } else if (terminals.isLast) {
      e.backBtn.classList.remove("backBtnDisabled");
      e.backBtnF.classList.remove("backBtnDisabled");
      e.fwdBtn.classList.add("fwdBtnDisabled");
      e.fwdBtnF.classList.add("fwdBtnDisabled");
    } else {
      e.backBtn.classList.remove("backBtnDisabled");
      e.backBtnF.classList.remove("backBtnDisabled");
      e.fwdBtn.classList.remove("fwdBtnDisabled");
      e.fwdBtnF.classList.remove("fwdBtnDisabled");
    } // show disabled fwd btn for screens whose activity must be completed before u can navigate


    if (disableFwdBtn && !terminals.isLast) {
      e.fwdBtn.classList.add("fwdBtnDisabled");
      e.fwdBtnF.classList.add("fwdBtnDisabled");
      this.disabledFwdBtns = [e.fwdBtn, e.fwdBtnF];
    }
  },
  updateAudioIcon: function updateAudioIcon(target, isOn, hide) {
    var audioOn = target.getAttribute("data-audioOn");
    var isBgImg = audioOn.substring(0, 3) === "url";

    if (isOn) {
      if (isBgImg) {
        target.style.backgroundImage = audioOn;
      } else {
        target.src = audioOn;
      }
    } else {
      var audioOff = target.getAttribute("data-audioOff");

      if (isBgImg) {
        target.style.backgroundImage = audioOff;
      } else {
        target.src = audioOff;
      }
    }

    if (hide) {
      target.style.display = "none";
    }
  },
  clearScreenFrame: function clearScreenFrame() {
    this.screenFrame.innerHTML = "";
  },
  destroy: function destroy(navHandler) {
    var evtTargs = [this.elements.mainNav, this.elements.subNav, this.elements.screenNav, this.elements.screenNavFooter]; // remove event listeners if elements defined - only need to check first

    evtTargs[0] && lnx.util.updateEventListener(evtTargs, "click", navHandler, true);
    this.courseTitleElm = this.elements = this.screenFrame = this.currScreen = this.screenTypes = null;
    this.mainNavNodes = this.subNavNodes = this.screenNavNodes = null; // clear screen in case window.close fails

    var elm = document.getElementById("body");
    elm && (elm.innerHTML = "");
  }
};
lnx.view.refPanel = {
  overlay: null,
  closeBtn: null,
  active: false,
  init: function init(node, screensMap, title, refId) {
    var self = this;
    this.overlay = document.getElementById("player").appendChild(document.createElement("div"));
    this.overlay.className = "overlay refPanel";
    this.overlay.innerHTML = '<div class="refCloseBtn"><img src="images/tocClose.svg"></div><div class="refContent"></div>';

    this.overlay.querySelector(".refCloseBtn").onclick = function () {
      self.show(false);
    }; //this.overlay.querySelector('.refTitle').innerHTML = title;


    this.overlay.querySelector('.refContent').innerHTML = lnx.view.getScreenContent(node, screensMap); // var s = this.overlay.querySelector('.refContent ul').innerHTML;
    // this.overlay.querySelector('.refContent').innerHTML = s;

    lnx.view.setAlwaysShowRefOn(refId); //this.show();       
  },
  show: function show(s) {
    if (!this.overlay) return;

    if (s === undefined) {
      s = !this.active;
      this.active = s;
    } else {
      this.active = s === false ? false : true;
    }

    this.overlay.style.display = s === false ? "none" : "block";
  }
};
lnx.audio = {
  audioPlayer: null,
  flashAudioPlayer: null,
  audioOn: true,
  tOutToken: null,
  period: null,
  delayTimeoutToken: null,
  init: function init() {
    this.setAudioOn(!(lnx.scormApi.getSuspendDataValue("audio") === "false"));

    if (window.Audio) {
      this.audioPlayer = new Audio();
      this.audioPlayer.addEventListener("ended", this.onAudioEnd);
    } //this.flashAudioPlayer = document.getElementById("flashAudioPlayer");

  },
  getAudioOn: function getAudioOn() {
    return this.audioOn;
  },
  setAudioOn: function setAudioOn(setOn) {
    this.audioOn = setOn;
    lnx.scormApi.setSuspendDataValue("audio", "" + setOn);
  },
  autoPlayAudio: function autoPlayAudio(node, screenElm) {
    if (!this.audioOn || lnx.config.noAudio) {
      return;
    }

    var name;

    if (screenElm && (name = screenElm.getAttribute("data-autoAudio"))) {
      this.playAudio(name);
    }
  },
  playAudio: function playAudio(name, delay) {
    if (lnx.config.noAudio) return;
    this.stopAudio();

    if (lnx.nav.isLinearNav && lnx.nav.completeAllScreens) {
      if (this.period === null) {
        this.period = lnx.nav.audioTout;
      }

      if (this.period) {
        this.tOut = setTimeout(this.onAudioEnd, this.period);
      }
    }

    if (!this.audioOn) {
      return;
    }

    if (name === "0") {
      // placeholder value when no audio file named in xml
      // ie html5 audio player seems to have problem if you feed
      // too many dummy file names to it
      return;
    }

    var suffix = "audio/";
    var prefix = ".mp3";
    var file = suffix + name + prefix;

    if (this.audioPlayer && this.audioPlayer.play) {
      if (typeof delay === "number") {
        this.delayTimeoutToken = setTimeout(function () {
          lnx.audio.playAudio(name);
        }, delay);
        return;
      } //try{


      this.audioPlayer.src = file;
      var p = this.audioPlayer.play();

      if (p !== undefined) {
        p.catch(function (error) {//console.log("In audio promise catch: " + error);
        });
      } // catch( e ){
      //  lnx.util.onError( e, false, "playAudio - html5" );
      // }

    } else {
      try {
        this.flashAudioPlayer.playFlashAudio(file);
      } catch (e) {
        lnx.util.onError(e, false, "playAudio - flash");
      }
    }
  },
  stopAudio: function stopAudio() {
    if (lnx.config.noAudio) return;
    clearTimeout(this.tOut);
    clearTimeout(this.delayTimeoutToken);

    try {
      if (this.audioPlayer && this.audioPlayer.pause) {
        // html 5 audio in ie9 seems to throw errors when pause function is called depending on audio state
        // testing for audio paused does not prevent following clause from running
        // debugger shows paused value as true when error is generated so following line should not run but does!
        // testing for 'not paused' prevents error but alows audio to continue when it should'nt
        // only seems issue when u rapidly update audio source
        // does not seem to apply to safari
        if (this.audioPlayer.paused !== true) {
          this.audioPlayer.pause();
        }
      } else {
        this.flashAudioPlayer.stopFlashAudio();
      }
    } catch (e) {
      lnx.util.onError(e, false, "stopAudio");
    }
  },
  onAudioEnd: function onAudioEnd(e) {
    var self = lnx.audio;
    clearTimeout(self.tOut);

    if (lnx.nav.getIsScreenLocked()) {//return;
    }

    var screen = lnx.view.getCurrentScreen();
    var isVirtual = !!screen.insertVirtualScreen;

    if (isVirtual && screen.waitForUnlock) {
      if (screen.waitForUnlock() && !screen.isFinalScreen()) {
        return;
      }
    }

    lnx.nav.setIsScreenLocked(false);
    lnx.view.onScreenComplete();

    if (isVirtual && !screen.isFinalScreen()) {
      return;
    }

    lnx.cache.setValue("complete", lnx.nav.getCurrentScreenId(), true);
  }
}; // following objects are the various screen types the app supports
// all screen objects must implement the following interface:
// screen interface:
// obj.init( 
//			 node, - xml toc node
// 			 screenElm, - screen content as html element
//			 frameElm - html element (a div) which parents the screen content 
//	)
// obj.hasContent()
// ojb.destroy( type )
// basic is generally a simple text screen without interactivity

lnx.basic = {
  init: function init(node, screenElm, frameElm) {},
  hasContent: function hasContent() {
    return false;
  },
  isComplete: function isComplete() {
    return true;
  },
  destroy: function destroy(type) {}
};
lnx.video = {
  // bug in ios safari, can't reparent video element
  // use innerHTML to create new video elements
  // reposition single flash object in IE above underlay element 
  init: function init(node, screenElm) {
    var useHTML5 = lnx.config.useHTML5;
    var vidType = useHTML5 ? ".mp4" : ".swf";
    var vidFolder = "video/";
    var holder = document.getElementById("videoUnderlay");
    var data = holder.getAttribute("data-videoInfo");
    var src = vidFolder + data + vidType;
    var vid;

    if (useHTML5) {
      // create html5 video element
      holder.innerHTML = this.getVideoHTML(data, src);
      vid = holder.firstChild;
      lnx.util.updateEventListener(vid, "play", this.onPlay);
    } else {
      // use existing flash object
      try {
        vid = document.getElementById("videoPlayer");
        vid.loadNewMovie(src, false);
      } catch (e) {
        lnx.util.onError(e, false, "video - flash");
      }

      holder.innerHTML = "";
    } // position video above underlay


    var u = document.getElementById("videoUnderlay");
    vid.style.position = "absolute"; // ie6 and 7 positioning fix - these versions do not correctly calculate offsets consistently
    // only tutorial video effected - temp quick fix

    var ot = u.offsetTop;

    if (data === "tutorial" && lnx.config.isIELessThan8 && ot < 100 && !document.querySelectorAll) {
      ot = 220;
    }

    vid.style.top = ot + "px";
    vid.style.left = u.offsetLeft + "px";
    vid.className = data === "tutorial" ? "videoPlayerTutShow" : "videoPlayerShow";
  },
  onPlay: function onPlay(e) {
    e = e || window.event;
    var target = e.target || e.srcElement;
    lnx.audio.stopAudio();
    /*if( target.paused ){
    	target.play && target.play();
    } else {
    	target.pause && target.pause();
    }*/

    e.stopPropagation ? e.stopPropagation() : e.cancelBubble = true;
    e.preventDefault ? e.preventDefault() : e.returnValue = false;
    return false;
  },
  getVideoHTML: function getVideoHTML(data, src) {
    var poster = "images/" + data + ".jpg";
    var iHTML = "<video controls='controls' poster='" + poster + "' type='video/mp4'>";
    iHTML += "<source src='" + src + "' type='video/mp4' />";
    iHTML += "</video>";
    return iHTML;
  },
  hasContent: function hasContent() {
    return false;
  },
  destroy: function destroy() {
    var useHTML5 = lnx.config.useHTML5;
    var vid;

    try {
      if (useHTML5) {
        vid = document.getElementById("videoUnderlay").firstChild;
        lnx.util.updateEventListener(vid, "play", this.onPlay, true);
      } else {
        vid = document.getElementById("videoPlayer");
        vid.unLoadMovie();
        vid.className = "videoPlayerHide";
      }
    } catch (e) {
      lnx.util.onError(e, false, "video.destroy - useHTML5: " + useHTML5);
    }
  }
};
lnx.introPanel = {
  items: [],
  current: null,
  init: function init(node, screenElm, frameElm) {
    this.items = Array.prototype.slice.call(screenElm.querySelectorAll(".hPanel"));

    if (this.items.length) {
      lnx.util.updateEventListener(this.items, "click", this.onSelection);
    }
  },
  onSelection: function onSelection(e) {
    e = e || window.event; // shortcut ref

    var self = lnx.panel;
    var elm = e.currentTarget;

    if (self.current && elm === self.current) {
      animate(self.current.querySelector(".panelF"), false);
      elm = null;
    } else {
      animate(elm.querySelector(".panelF"), true);
      self.current && animate(self.current.querySelector(".panelF"), false);
    }

    self.current = elm;
    e.stopPropagation ? e.stopPropagation() : e.cancelBubble = true;
    e.preventDefault ? e.preventDefault() : e.returnValue = false;
    return false;

    function animate(e, up) {
      if (!e) return;
      var tl = new TimelineMax();

      if (up) {
        //e.style.top = "0px";                
        tl.to(e, .6, {
          top: "0px"
        });
      } else {
        //e.style.top = "265px"
        tl.to(e, .6, {
          top: "265px"
        });
      }
    }
  },
  hasContent: function hasContent() {
    return false;
  },
  destroy: function destroy() {
    lnx.util.updateEventListener(this.items, "click", this.onSelection, true);
    this.current = null;
    this.items = [];
  }
};
lnx.panel = {
  items: [],
  current: null,
  init: function init(node, screenElm, frameElm) {
    // get nodeList of div elms for traversal
    var set = screenElm.getElementsByTagName("div");
    var tallest = 0,
        tabs = []; // collect set of clickable panel elements

    for (var i = 0, len = set.length; i < len; i++) {
      if (set[i].getAttribute("data-optionNum")) {
        this.items.push(set[i]);
        tabs.push(set[i].querySelector("div"));
        var h = parseInt(window.getComputedStyle(tabs[tabs.length - 1]).height);

        if (h > tallest) {
          tallest = h;
        }
      }
    }

    for (var i = 0; i < tabs.length; i++) {
      tabs[i].style.height = tallest + "px";
    }

    if (this.items.length) {
      lnx.util.updateEventListener(this.items, "click", this.onSelection);
    }
  },
  onSelection: function onSelection(e) {
    e = e || window.event;
    var target = e.target || e.srcElement; // cater for ie8 lack of currentTarget re bubbling

    var elm,
        audio,
        noAudio = false;

    if (e.currentTarget) {
      elm = e.currentTarget;
      audio = e.currentTarget.getAttribute("data-audio");
    } else {
      if (!target.getAttribute("data-optionNum")) {
        var t = target.parentNode;

        while (t.parentNode) {
          if (t.getAttribute("data-optionNum")) {
            elm = t;
            audio = t.getAttribute("data-audio");
            break;
          }

          t = t.parentNode;
        }
      } else {
        elm = target;
        audio = target.getAttribute("data-audio");
      }
    } // shortcut ref


    var self = lnx.panel; // multiple classes per className
    // workaround for lack of classList property in IE

    var child;

    if (self.current && self.current === elm) {
      // toggle current items display
      child = self.current.getElementsByTagName("p")[1];

      if (/hide/.test(child.className)) {
        child.className = child.className.replace(/hide/, "show");
      } else {
        child.className = child.className.replace(/show/, "hide");
        noAudio = true;
      }
    } else {
      // show new selection and hide (possible) previous selection
      elm.getElementsByTagName("p")[1].className = elm.getElementsByTagName("p")[1].className.replace(/hide/, "show");

      if (self.current) {
        self.current.getElementsByTagName("p")[1].className = self.current.getElementsByTagName("p")[1].className.replace(/show/, "hide");
      }
    } // only play audio if new item clicked
    // stop audio if existing selection is reselected 'off'


    if (noAudio) {//lnx.audio.stopAudio(); //no panel audio in this course so let main audio finish 
    } else if (audio) {//lnx.audio.playAudio(audio);
    }

    self.current = elm;
    e.stopPropagation ? e.stopPropagation() : e.cancelBubble = true;
    e.preventDefault ? e.preventDefault() : e.returnValue = false;
    return false;
  },
  hasContent: function hasContent() {
    return false;
  },
  destroy: function destroy() {
    lnx.util.updateEventListener(this.items, "click", this.onSelection, true);
    this.current = null;
    this.items = [];
  }
};
lnx.accordian = {
  items: [],
  current: null,
  init: function init(node, screenElm, frameElm) {
    // get nodeList of div elms for traversal
    var set = screenElm.querySelectorAll("div.clickable"); // collect clickable div lems

    for (var i = 0, len = set.length; i < len; i++) {
      if (set[i].getAttribute("data-optionNum")) {
        this.items.push(set[i]);
      }
    }

    if (this.items.length) {
      lnx.util.updateEventListener(this.items, "click", this.onSelection);
    }
  },
  onSelection: function onSelection(e) {
    e = e || window.event;
    var target = e.target || e.srcElement; // cater for ie8 lack of currentTarget re bubbling

    var elm;

    if (e.currentTarget) {
      elm = e.currentTarget;
    } else {
      if (!target.getAttribute("data-optionNum")) {
        var t = target.parentNode;

        while (t.parentNode) {
          if (t.getAttribute("data-optionNum")) {
            elm = t;
            break;
          }

          t = t.parentNode;
        }
      } else {
        elm = target;
      }
    } // shortcut ref


    var self = lnx.accordian;
    var info; // multiple classes per className
    // following is workaround for lack of classList property

    if (self.current && self.current === elm) {
      // toggle current items display
      info = self.current.querySelector("div:nth-of-type(2)");

      if (/hide/.test(info.className)) {
        info.className = info.className.replace(/hide/, "show");
      } else {
        info.className = info.className.replace(/show/, "hide");
      }
    } else {
      // show new selection and hide (possible) previous selection
      info = elm.querySelector("div:nth-of-type(2)");
      info.className = info.className.replace(/hide/, "show");

      if (self.current) {
        info = self.current.querySelector("div:nth-of-type(2)");
        info.className = info.className.replace(/show/, "hide");
      }
    }

    self.current = elm;
    e.stopPropagation ? e.stopPropagation() : e.cancelBubble = true;
    e.preventDefault ? e.preventDefault() : e.returnValue = false;
    return false;
  },
  hasContent: function hasContent() {
    return false;
  },
  destroy: function destroy() {
    lnx.util.updateEventListener(this.items, "click", this.onSelection, true);
    this.current = null;
    this.items = [];
  }
};
lnx.scenario = {
  options: [],
  isSubtype: false,
  screenElm: null,
  init: function init(node, screenElm, frameElm) {
    // store whether scenario is subtype or not
    this.isSubtype = !!node.getAttribute("subType"); // store a reference to the screen's parent html element for use in event handler

    this.screenElm = screenElm; // get nodeList of div elms for traversal

    var set = screenElm.getElementsByTagName("div"); // collect set of scenario options - options have data-optionNum attribute

    for (var i = 0, len = set.length; i < len; i++) {
      if (set[i].getAttribute("data-optionNum")) {
        this.options.push(set[i]);
      }
    }

    if (this.options.length) {
      lnx.util.updateEventListener(this.options, "click", this.onSelection);
    }
  },
  onSelection: function onSelection(e) {
    e = e || window.event;
    var target = e.target || e.srcElement; // ref to host object

    var self = lnx.scenario; // cater for ie8 which has no currentTarget re bubbling

    var opt, audio, t;

    if (e.currentTarget) {
      opt = e.currentTarget.getAttribute("data-optionNum");
      audio = e.currentTarget.getAttribute("data-audio");
    } else {
      opt = target.getAttribute("data-optionNum");

      if (!opt) {
        t = target.parentNode;

        while (t.parentNode) {
          if (opt = t.getAttribute("data-optionNum")) {
            audio = t.getAttribute("data-audio");
            break;
          }

          t = t.parentNode;
        }
      } else {
        audio = target.getAttribute("data-audio");
      }
    }

    opt = parseInt(opt, 10); // we have 2 types of scenario
    // TO DO: clear out magic strings

    if (self.isSubtype) {
      var elm2,
          elm = document.getElementById("cs_1_left_txt");
      var img = document.getElementById("csImg");
      img.src = self.screenElm.getAttribute("data-image2");

      if (opt === 1) {
        elm2 = document.getElementById("cs_2_a");
        elm.innerHTML = elm2.innerHTML;
      } else if (opt === 2) {
        elm2 = document.getElementById("cs_2_b");
        elm.innerHTML = elm2.innerHTML;
      } else if (opt === 3) {
        elm2 = document.getElementById("cs_2_c");
        elm.innerHTML = elm2.innerHTML;
      }
    } else {
      if (opt >= 1 && opt <= 4) {
        document.getElementById("cs_1").className = "hide";
        document.getElementById("cs_2").className = "show";
      } else if (opt == 5) {
        document.getElementById("cs_2_right_1").className = "show";
        document.getElementById("cs_2_right_2").className = "hide";
      } else if (opt == 6) {
        document.getElementById("cs_2_right_2").className = "show";
        document.getElementById("cs_2_right_1").className = "hide";
      }
    }

    if (audio) {
      lnx.audio.playAudio(audio);
    }

    e.stopPropagation ? e.stopPropagation() : e.cancelBubble = true;
    e.preventDefault ? e.preventDefault() : e.returnValue = false;
    return false;
  },
  hasContent: function hasContent() {
    return false;
  },
  destroy: function destroy(type) {
    lnx.util.updateEventListener(this.options, "click", this.onSelection, true);
    this.options = [];
    this.isSubtype = false;
    this.screenElm = null;
  }
};
lnx.slideShow = {
  thumbs: [],
  current: null,
  active: false,
  useHTML5: false,
  init: function init(node, screenElm) {
    this.useHTML5 = lnx.config.useHTML5; // get nodeList of img elms for traversal

    var vset = lnx.util.getElmsByTagAndAttrib(screenElm, "img", "data-thumbs"); // collect set of thumbnails

    for (var i = 0, len = vset.length; i < len; i++) {
      this.thumbs.push(vset[i]);
    }

    if (this.thumbs.length) {
      lnx.util.updateEventListener(this.thumbs, "click", this.onClick);
      lnx.util.updateEventListener(this.thumbs, "mouseover", this.onMouseOver);
      lnx.util.updateEventListener(this.thumbs, "mouseout", this.onMouseOut);
    }
  },
  onClick: function onClick(e) {
    e = e || window.event;
    var target = e.target || e.srcElement;
    lnx.audio.stopAudio();

    if (lnx.slideShow.current === target) {// replay movie
    } else {
      target.src = target.getAttribute("data-down");
      lnx.slideShow.current = target;
      var tmbs = lnx.slideShow.thumbs; // ie8 has no nextElementSibling but nextSibling selects text nodes in ie9

      var nextElementSibling = target.nextElementSibling !== undefined ? "nextElementSibling" : "nextSibling";

      for (var i = 0, len = tmbs.length; i < len; i++) {
        if (tmbs[i] !== target) {
          // display thumbnail in up state
          tmbs[i].src = tmbs[i].getAttribute("data-up");
          var sib = tmbs[i][nextElementSibling];

          if (sib && sib.tagName.toLowerCase() === "p") {
            sib.className = "ssThumbTitle";
          }
        } else {
          // display thumbnail in down state
          var sib = tmbs[i][nextElementSibling];

          if (sib && sib.tagName.toLowerCase() === "p") {
            sib.className = "ssThumbTitleOn";
          } // hide placeholder image where video displays


          var img = document.getElementById("ssInitImg");
          img && (img.style.display = "none"); // display video	

          lnx.slideShow.displayVideo(tmbs[i].getAttribute("data-targetVid"));
        }
      }
    }

    e.stopPropagation ? e.stopPropagation() : e.cancelBubble = true;
    e.preventDefault ? e.preventDefault() : e.returnValue = false;
    return false;
  },
  onMouseOver: function onMouseOver(e) {
    e = e || window.event;
    var target = e.target || e.srcElement;

    if (lnx.slideShow.current !== target) {
      target.src = target.getAttribute("data-roll");
    }

    e.stopPropagation ? e.stopPropagation() : e.cancelBubble = true;
    e.preventDefault ? e.preventDefault() : e.returnValue = false;
    return false;
  },
  onMouseOut: function onMouseOut(e) {
    e = e || window.event;
    var target = e.target || e.srcElement;

    if (lnx.slideShow.current !== target) {
      target.src = target.getAttribute("data-up");
    }

    e.stopPropagation ? e.stopPropagation() : e.cancelBubble = true;
    e.preventDefault ? e.preventDefault() : e.returnValue = false;
    return false;
  },
  getVideoHTML: function getVideoHTML(data, src) {
    var poster = "images/" + data + ".jpg";
    var iHTML = "<video controls='controls' poster='" + poster + "' type='video/mp4'>";
    iHTML += "<source src='" + src + "' type='video/mp4' />";
    iHTML += "</video>";
    return iHTML;
  },
  displayVideo: function displayVideo(data) {
    var vid;
    var vidType = this.useHTML5 ? ".mp4" : ".swf";
    var vidFolder = "video/";
    var src = vidFolder + data + vidType;
    var holder = document.getElementById("videoUnderlay");

    if (this.useHTML5) {
      holder.innerHTML = this.getVideoHTML(data, src);
      holder.style.display = "block";
      vid = holder.firstChild;
      vid.className = "videoPlayerSSShow";
      vid.play();
    } else {
      // diplay flash object and load flash movie
      holder.innerHTML = "";
      holder.style.display = "block";
      vid = document.getElementById("videoPlayer");
      vid.className = "videoPlayerSSShow";

      try {
        vid.loadNewMovie(src, true);
      } catch (e) {
        lnx.util.onError(e, false, "slideShow - flash");
      } // position flash object above underlay


      vid.style.position = "absolute";
      var u = document.getElementById("videoUnderlay"); // ie6 and 7 positioning fix - these versions do not correctly calculate offsets consistently
      //  - temp quick fix

      var ot = u.offsetTop;
      var ol = u.offsetLeft;

      if (lnx.config.isIELessThan8 && ot < 100 && !document.querySelectorAll) {
        ot = 220;
        ol = 410;
      }

      vid.style.top = ot + "px";
      vid.style.left = ol + "px";
      vid.className = "videoPlayerSSShow";
      this.active = true;
    }
  },
  hasContent: function hasContent() {
    return false;
  },
  destroy: function destroy() {
    lnx.util.updateEventListener(this.thumbs, "click", this.onClick, true);
    lnx.util.updateEventListener(this.thumbs, "mouseover", this.onMouseOver, true);
    lnx.util.updateEventListener(this.thumbs, "mouseout", this.onMouseOut, true);

    if (this.active) {
      try {
        var vid = document.getElementById("videoPlayer");
        vid.unLoadMovie();
        vid.className = "videoPlayerHide";
      } catch (e) {
        lnx.util.onError(e, false, "slideShow.destroy - flash");
      }
    }

    this.active = false;
    this.current = null;
    this.useHTML5 = false;
    this.thumbs = [];
  }
};
lnx.assessment = {
  //constants
  STATE_NOTINITED: 0,
  STATE_CONTINUE: 1,
  STATE_RETURN: 2,
  STATE_EXIT: 3,
  TEST_INCOMPLETE: 0,
  TEST_PASS: 1,
  TEST_FAIL: 2,
  pass: false,
  formHolder: null,
  submitBtn: null,
  reset: false,
  questions: [],
  results: [],
  currentQ: 0,
  questionIds: [],
  qsCountDown: null,
  complete: false,
  passScore: 8,
  retakeId: null,
  frame: null,
  submitMsg: null,
  typeString: "test",
  status: 0,

  /*this.TEST_INCOMPLETE*/
  state: 0,

  /*this.STATE_NOTINITED*/
  init: function init(node, screenElm, frameElm) {
    var prevState = this.state;

    switch (prevState) {
      case this.STATE_NOTINITED:
        {
          this.setup(node, screenElm, frameElm);
          this.state = this.STATE_CONTINUE;
          break;
        }

      case this.STATE_CONTINUE:
        {
          this.updateQuestion(node);
          break;
        }

      case this.STATE_EXIT:
        {
          this.add();
          this.updateQuestion(node);
          this.state = this.STATE_CONTINUE;
          break;
        }
    }
  },
  setup: function setup(node, screenElm, frameElm) {
    this.submitMsg = document.getElementById("submitMsg");
    var passScr = parseInt(screenElm.getAttribute("data-passScore"), 10);

    if (!isNaN(passScr)) {
      this.passScore = passScr;
    }

    this.retakeId = screenElm.getAttribute("data-retakeId");
    this.frame = frameElm;
    this.submitBtn = document.getElementById("submitBtn");
    this.submitBtn && (this.submitBtn.value = lnx.localization.getLocalString("s11")); //Submit

    this.disableSubmitBtn(true);
    var fset = screenElm.getElementsByTagName("fieldset"); // UNDO remove
    //var ans = [1,0,0,2,1,0,0,0,1,1];
    // add fieldset elements to our custom form objects question property

    for (var i = 0, len = fset.length; i < len; i++) {
      this.questions.push(fset[i]); // UNDO remove
      //var q = fset[i].getElementsByTagName("input")[ans[i]].checked = true;
      // create hash map on array for quick lookup of a question's indexy by its id 

      var id = fset[i].getAttribute("id");
      this.questions[id] = i;
      this.questionIds.push(id);
    } // UNDO remove
    //this.submitBtn.disabled = false;


    this.qsCountDown = this.questionIds.slice(0); // default fieldset display is none, show first

    this.updateQuestion(node); //this.questions[0].className = "show";
    // note our formHolder reference is acutally a div that wraps the actual form

    this.formHolder = screenElm;
    lnx.util.updateEventListener(document.getElementById("form_1"), "submit", this.onSubmit);
    lnx.util.updateEventListener(document.getElementById("form_1"), "click", this.onChange);
  },
  hasContent: function hasContent() {
    return this.state !== this.STATE_NOTINITED;
  },
  // handle radio button change event
  onChange: function onChange(e) {
    e = e || window.event;
    var target = e.target || e.srcElement;
    var self = lnx.assessment; // ie 8 workaround, not catching change event therefore using click
    // do not want to recieve non radio button click events

    if (target.type && (target.type.toLowerCase() === "radio" || target.type.toLowerCase() === "checkbox")) {
      var t = target;

      while (t && t.nodeName.toLowerCase() != "fieldset") {
        t = t.parentNode;
      } // get question id from fieldset


      var id = t.id; // check if all questions are answered

      if (!self.complete) {
        for (var i = 0, len = self.qsCountDown.length; i < len; i++) {
          if (id === self.qsCountDown[i]) {
            self.qsCountDown.splice(i, 1);

            if (!self.qsCountDown.length) {
              self.complete = true;
            }

            break;
          }
        } // if all questions answered, enable submit button


        if (self.complete) {
          self.disableSubmitBtn(false);
          self.submitMsg && (self.submitMsg.innerHTML = "&nbsp;");
        } else {
          var txt = "&nbsp;"; // do not display message re unanswered questions if only last question unanswered

          if (!(self.qsCountDown.length === 1 && self.qsCountDown[0] === self.questionIds[self.questionIds.length - 1])) {
            for (i = 0, len = self.qsCountDown.length; i < len; i++) {
              //TO DO update to remove dependency on format of Q id
              var num = self.qsCountDown[i].slice(2);
              txt += num + (i + 1 !== len ? ",&nbsp;&nbsp;" : "&nbsp;&nbsp;");
            }

            var prefixTxt = len > 1 ? lnx.localization.getLocalString("s3") + "&nbsp;&nbsp;" : lnx.localization.getLocalString("s4") + "&nbsp;&nbsp;"; //Question(s)

            txt = prefixTxt + txt + "&nbsp;" + lnx.localization.getLocalString("s5"); //"not answered";
          }

          self.submitMsg && (self.submitMsg.innerHTML = txt);
        }
      }
    }

    e.stopPropagation ? e.stopPropagation() : e.cancelBubble = true;
    target.blur();
  },
  //handle form submit event
  onSubmit: function onSubmit(e) {
    e = e || window.event;
    var target = e.target || e.srcElement;
    var self = lnx.assessment;
    var res = self.calcResult();
    self.status = res.pass ? self.TEST_PASS : self.TEST_FAIL;
    self.pass = self.status === self.TEST_PASS; // update to get form by tag

    var form = document.getElementById("form_1");
    form.reset();

    if (self.submitMsg) {
      self.submitMsg.innerHTML = lnx.localization.getLocalString("s2"); //"All questions remain unanswered";
    }

    self.complete = false;
    self.qsCountDown = self.questionIds.slice(0);
    self.disableSubmitBtn(true); //TO DO: fire custom event

    lnx.nav.onTestResult(self.pass);
    e.stopPropagation ? e.stopPropagation() : e.cancelBubble = true;
    e.preventDefault ? e.preventDefault() : e.returnValue = false;
    return false;
  },
  disableSubmitBtn: function disableSubmitBtn(disable) {
    // cater for ie 8 which does'nt recognise disabled pseudoclass
    if (disable) {
      this.submitBtn.disabled = true;
      this.submitBtn.className = "submitButtonDisabled";
      this.submitBtn.style.cursor = "default";
    } else {
      this.submitBtn.disabled = false;
      this.submitBtn.className = "submitButtonEnabled";
      this.submitBtn.style.cursor = "pointer";
    }
  },
  // show new question content
  updateQuestion: function updateQuestion(node) {
    var navId = node.getAttribute("navId");
    this.questions[this.currentQ].className = "hide"; // the xml node's navId attribute maps to the html id of the fieldset
    // use this to find the index of the new question

    this.currentQ = this.questions[navId];
    this.questions[this.currentQ].className = "show";
  },
  // re-attach form to document
  add: function add() {
    // should pass in frame as param
    // integrate this func with updateQuestion func
    this.frame.innerHTML = "";
    this.frame.appendChild(this.formHolder);
  },
  calcResult: function calcResult() {
    var score = 0; // stores a set of results for this assessment

    this.results = []; // get nodeList of fieldset elms for traversal

    var fset = this.formHolder.getElementsByTagName("fieldset");
    var txtCon = lnx.config.isIE8orLess ? "innerText" : "textContent"; // check user response against correct answer for each question

    for (var i = 0, len = fset.length; i < len; i++) {
      // result obj for individual Questions in assessment
      var r = {},
          fail = false,
          found = false;
      r.options = [];
      r.isCorrect = false;
      r.question = fset[i].getElementsByTagName("legend")[0].innerHTML; //r.feedback = fset[i].querySelector("p.feedback")[txtCon];
      // cater for ie 6

      r.feedback = lnx.util.getElmsByTagAndAttrib(fset[i], "div", "data-fb")[0].innerHTML;
      r.audioFb = fset[i].getAttribute("data-audio");
      r.userAns = [];
      var correctAns = fset[i].getAttribute("data-correctAns").split(",");
      var rbs = fset[i].getElementsByTagName("input");

      for (var j = 0, k = rbs.length; j < k; j++) {
        //only interested in radio buttons or checkbox
        if (rbs[j].type.toLowerCase() != "radio" && rbs[j].type.toLowerCase() != "checkbox") {
          continue;
        } // record the option text for feedback screen


        r.options.push(rbs[j].parentNode[txtCon]);

        if (rbs[j].checked) {
          lnx.util.log(rbs[j].name + " checked"); // record users answer
          //r.userAns = j;

          var ua = {
            num: j,
            isCorrect: false
          };
          found = false;

          for (var m = 0; m < correctAns.length; m++) {
            if (rbs[j].value === correctAns[m]) {
              correctAns.splice(m, 1);
              found = true;
              ua.isCorrect = true;
              break;
            }
          }

          if (!found) {
            fail = true;
          }

          r.userAns.push(ua);
        }
      }

      if (!correctAns.length && !fail) {
        score++;
        r.isCorrect = true;
      }

      this.results.push(r);
    }

    return {
      result: score,
      pass: score >= this.passScore
    };
  },
  // return assessment results
  getResults: function getResults() {
    return this.results;
  },
  // return the current assessment status - incomplete, pass, fail
  getStatus: function getStatus() {
    return this.status;
  },
  // clears form of current selections
  doReset: function doReset() {
    this.reset = true;
    this.status = this.TEST_INCOMPLETE;
    this.disableSubmitBtn(true);
  },
  // retake the assessment
  retake: function retake() {
    lnx.nav.onRetakeTest(this.retakeId);
  },
  destroy: function destroy(type) {
    if (this.typeString === type) {
      // when the new screen type is also 'test' 
      // ignore destroy call as the type equivalence indicates
      // we are moving between question screens i.e. we are still within the test
      return;
    }

    if (lnx.config.isIE6) {
      // ie6 does not remember selections when form is removed from doc
      // so clear our record of questions answered so that submit button
      // is not incorrectly shown
      this.qsCountDown = this.questionIds.slice(0);

      try {
        this.submitMsg && (this.submitMsg.innerHTML = lnx.localization.getLocalString("s2")); //"All questions remain unanswered";
      } catch (e) {}
    }

    this.state = this.STATE_EXIT;

    if (this.frame.firstChild) {
      this.frame.removeChild(this.frame.firstChild);
    }
  },
  // called on page close to release resources held in doc fragment
  fullDestroy: function fullDestroy() {
    try {
      var elm = document.getElementById("form_1");

      if (elm) {
        lnx.util.updateEventListener(elm, "submit", this.onSubmit, true);
        lnx.util.updateEventListener(elm, "click", this.onChange, true);
      }

      this.formHolder = this.submitBtn = this.questions = this.results = null;
      this.currentQ = this.questionIds = this.qsCountDown = this.frame = this.submitMsg = null;
    } catch (e) {}
  }
};
lnx.assessmentIntro = {
  resourceId: null,
  init: function init(node, screenElm, frameElm) {
    this.resourceId = screenElm.getAttribute("data-resourceId");
    var elm = document.getElementById(this.resourceId);
    elm && (elm.value = lnx.localization.getLocalString("s10")); // Knowledge Check

    if (this.resourceId) {
      lnx.util.updateEventListener(elm, "click", this.onStartTest);
    } else {
      lnx.util.onError("Failed to find resourceId in lnx.assessmentIntro.init");
    }

    return this;
  },
  onStartTest: function onStartTest() {
    lnx.nav.next();
  },
  hasContent: function hasContent() {
    return false;
  },
  destroy: function destroy(type) {
    if (this.resourceId) {
      lnx.util.updateEventListener(document.getElementById(this.resourceId), "click", this.onStartTest, true);
    }
  }
};
lnx.assessmentFeedback = {
  fbAudioIds: null,
  retakeBtn: null,
  fbItems: null,
  curFbItem: null,
  init: function init(node, screenElm, frameElm) {
    var helper = lnx.assessment;

    if (!this.fbAudioIds) {
      this.fbAudioIds = []; // note order of audio ids is important for setDisplay()

      this.fbAudioIds.push(screenElm.getAttribute("data-audioNoResults"));
      this.fbAudioIds.push(screenElm.getAttribute("data-audioCorrect"));
      this.fbAudioIds.push(screenElm.getAttribute("data-audioIncorrect"));
    }

    this.retakeBtn = document.getElementById("retakeBtn");
    this.retakeBtn && (this.retakeBtn.value = lnx.localization.getLocalString("s12"));
    var elms = [];

    for (var i = 0; i < 3; i++) {
      // feedback0: TEST_INCOMPLETE, feedback1: TEST_PASS, feedback2: TEST_FAIL,
      elms.push(document.getElementById("feedback" + i));
    }

    this.setDisplay(elms, helper.getStatus(), this.fbAudioIds);
    lnx.util.updateEventListener(this.retakeBtn, "click", this.onRetake);
  },
  // display 1 of 3 feeback alternatives depending on assessment status
  setDisplay: function setDisplay(elms, status, audioIds) {
    // status values:   
    // 0 - TEST_INCOMPLETE, 1 - TEST_PASS, 2 - TEST_FAIL        
    for (var j = 0; j < 3; j++) {
      if (j == status) {
        elms[j].className = "show";
        if (audioIds[j]) lnx.audio.playAudio(audioIds[j]);
        break;
      }
    }

    if (status) {
      // only show results when test taken, eg status value of 1 or 2
      this.showResults();
    }
  },
  onRetake: function onRetake(e) {
    e = e || window.event;
    var target = e.target || e.srcElement;
    var helper = lnx.assessment;
    var self = lnx.assessmentFeedback; //self.clearResults();

    helper.doReset();
    helper.retake();
    e.stopPropagation ? e.stopPropagation() : e.cancelBubble = true;
  },

  /*clearResults : function(){
      
      this.curFbItem = null;
      lnx.view.clearScreenFrame();
  },*/
  showResults: function showResults() {
    this.fbItems = [];
    var iHTML = "";
    var results = lnx.assessment.getResults();

    for (var i = 0, len = results.length; i < len; i++) {
      var num = i + 1;
      var img = results[i].isCorrect ? "src='images/knowledgecheck_tick.gif'" : "src='images/knowledgecheck_cross.gif'";
      iHTML += "<div data-result='true' class='qResult' data-optionNum='" + num + "' data-audio='" + results[i].audioFb + "'><img " + img + " class='imgBlock' \/><span class='fQuestionTitle'>" + lnx.localization.getLocalString("s4") + " " + num + "</span></div>";
      iHTML += "<div class='hide qFeedback'><p class='fbQuestion'>" + results[i].question + "</p>";
      iHTML += "<div class='fOptions'>";
      var o = results[i].options;

      for (var j = 0, k = o.length; j < k; j++) {
        var style = "";

        for (var m = 0; m < results[i].userAns.length; m++) {
          var ua = results[i].userAns[m];

          if (ua.num === j) {
            style = ua.isCorrect ? " class='userAnsGreen'" : " class='userAnsRed'";
          }
        } // put space between option number and option text


        var txt = o[j].replace(/(.\.)/, "$1&nbsp;&nbsp;");
        iHTML += "<p" + style + ">" + txt + "</p>";
      }

      iHTML += "</div>"; //var fb = results[i].isCorrect ? "That's correct!" : "That's incorrect!";

      var fb = results[i].isCorrect ? lnx.localization.getLocalString("s6") : lnx.localization.getLocalString("s7"); //iHTML += "<p class='fbHeading'>Feedback: " + fb + "</p>";

      iHTML += "<p class='fbHeading'>" + lnx.localization.getLocalString("s8") + "&nbsp;" + fb + "</p>";

      if (!(results[i].feedback === "<hr class=\"hr1px\">" || results[i].feedback === ".")) {
        iHTML += "<div class='fbTxt'>" + results[i].feedback + "</div>";
      }

      iHTML += "</div>";
    }

    var container = document.getElementById("fbResultsContainer");
    container.innerHTML = iHTML; // cater for ie6, can't use querySelectorAll

    var set = lnx.util.getElmsByTagAndAttrib(container, "div", "data-result");

    for (i = 0, len = set.length; i < len; i++) {
      this.fbItems.push(set[i]);
    }

    if (this.fbItems.length) {
      lnx.util.updateEventListener(this.fbItems, "click", this.onClick);
    }
  },
  onClick: function onClick(e) {
    e = e || window.event;
    var target = e.target || e.srcElement;
    var self = lnx.assessmentFeedback;
    var elm;

    if (e.currentTarget) {
      elm = e.currentTarget;
    } else {
      if (!target.getAttribute("data-optionNum")) {
        var t = target.parentNode;

        while (t.parentNode) {
          if (t.getAttribute("data-optionNum")) {
            elm = t;
            break;
          }

          t = t.parentNode;
        }
      } else {
        elm = target;
      }
    } // ie8 has no nextElementSibling but nextSibling selects text nodes in ie9


    var nextElementSibling = elm.nextElementSibling !== undefined ? "nextElementSibling" : "nextSibling"; // may be multiple classes per className
    // following is workaround for lack of classList property

    if (self.curFbItem && self.curFbItem === elm) {
      // toggle current items display
      var sib = self.curFbItem[nextElementSibling];
      /*if( !sib ){
          lnx.util.onError("No nextElementSibling in showResults");
      }*/

      if (/hide/.test(sib.className)) {
        sib.className = sib.className.replace(/hide/, "show"); // play audio feedback
        //lnx.audio.playAudio( elm.getAttribute( "data-audio" )); // no audio feedback in this course
      } else {
        sib.className = sib.className.replace(/show/, "hide"); // closed item so stop audio feedback
        //lnx.audio.stopAudio(); // no audio feedback in this course
      }
    } else {
      // show new selection and hide (possible) previous selection
      elm[nextElementSibling].className = elm[nextElementSibling].className.replace(/hide/, "show"); // play audio feedback
      //lnx.audio.playAudio( elm.getAttribute( "data-audio" )); // no audio feedback in this course

      if (self.curFbItem && self.curFbItem[nextElementSibling]) {
        self.curFbItem[nextElementSibling].className = self.curFbItem[nextElementSibling].className.replace(/show/, "hide");
      }
    }

    self.curFbItem = elm;
    e.stopPropagation ? e.stopPropagation() : e.cancelBubble = true;
    e.preventDefault ? e.preventDefault() : e.returnValue = false;
    return false;
  },
  hasContent: function hasContent() {
    return false;
  },
  destroy: function destroy(type) {
    if (this.retakeBtn) {
      lnx.util.updateEventListener(this.retakeBtn, "click", this.onRetake, true);
      this.retakeBtn = null;
    }

    if (this.fbItems) {
      lnx.util.updateEventListener(this.fbItems, "click", this.onClick, true);
      this.fbItems = null;
    }

    this.curFbItem = null;
  }
};
lnx.multiPanel = {
  items: [],
  init: function init(node, screenElm, frameElm) {
    var params = JSON.parse(screenElm.getAttribute("data-params"));
    var scrId = screenElm.getAttribute("id"); //console.log(params);

    var p = screenElm.querySelectorAll(".panelsHolder > div");
    var o = screenElm.querySelectorAll(".panelsOverlayHolder > div > div");
    var c = screenElm.querySelectorAll(".panelsOverlayHolder > div");
    var num = p.length;
    var targets = [];
    var isComplete = false;
    var inCompleteList = [];
    setStyles(p, params);

    for (i = 0; i < num; i++) {
      var obj = {
        closed: false,
        animHeight: false,
        value: null,
        panel: null,
        overlay: null,
        container: null
      };

      if (p[i].getAttribute("data-animHeight") === "true") {
        obj.animHeight = true;
        obj.value = window.getComputedStyle(c[i]).height;
        c[i].style.height = "0px";
      } else {
        obj.value = window.getComputedStyle(c[i]).width;
        c[i].style.width = "0px";
      }

      obj.panel = p[i];
      obj.overlay = o[i];
      obj.container = c[i];
      targets.push(obj);
      inCompleteList.push(obj);
    }

    var pf = function pf(e) {
      for (var i = 0; i < targets.length; i++) {
        if (targets[i].animHeight) {
          if (targets[i].container.style.height !== "0px") {
            close(targets[i]);
          }
        } else {
          if (targets[i].container.style.width !== "0px") {
            close(targets[i]);
          }
        }
      }

      i = parseInt(e.currentTarget.getAttribute("data-overlayNum"));
      open(targets[i]);
    };

    var of = function of(e) {
      var n = parseInt(e.currentTarget.getAttribute("data-overlayNum"));
      close(targets[n]);
    };

    var close = function close(tg) {
      if (!tg.closed) {
        var imgUrl = window.getComputedStyle(tg.panel).backgroundImage.replace(/\.jpg/, "_closed.jpg");
        tg.panel.style.backgroundImage = imgUrl;
        tg.closed = true;
      }

      var tl = new TimelineMax();
      var p = tg.animHeight ? {
        height: "0px"
      } : {
        width: "0px"
      };
      tl.to(tg.container, .75, p);
    };

    var open = function open(tg) {
      var tl = new TimelineMax();
      var p = tg.animHeight ? {
        height: tg.value
      } : {
        width: tg.value
      };
      tl.to(tg.container, .75, p);

      if (!isComplete) {
        if (getIsComplete(tg)) {
          isComplete = true;
          lnx.cache.setValue("complete", scrId, true);
          lnx.view.onScreenComplete();
        }
      }
    };

    for (var i = 0; i < targets.length; i++) {
      targets[i].panel.setAttribute("data-overlayNum", i);
      targets[i].panel.addEventListener("click", pf, false);
      targets[i].overlay.setAttribute("data-overlayNum", i);
      targets[i].overlay.addEventListener("click", of, false);
    }

    function getIsComplete(tg) {
      var l = inCompleteList;

      if (l.length) {
        for (var i = 0; i < l.length; i++) {
          if (l[i] === tg) {
            l.splice(i, 1);

            if (!l.length) {
              return true;
            }

            break;
          }
        }
      } else {
        return true;
      }
    }

    function setStyles(p, d) {
      var o = {};
      o.diam = d[1];
      var top = 0,
          left = 0,
          cd;

      for (i = 0; i < p.length; i++) {
        cd = d[i + 1];
        p[i].style.width = cd[0] + "px";
        p[i].style.height = cd[1] + "px";
        var img = 'images/' + scrId + '_mp_' + (i + 1) + '.jpg';
        p[i].style.backgroundImage = "url(" + img + ")";
        p[i].style.top = top + "px";
        p[i].style.left = left + "px";

        if (o.diam[1] !== cd[1]) {
          top = cd[1] + 20;
        } else {
          top = 0;
          left += cd[0] + 20;
        }
      }
    } // remember for destruction


    targets.panelFunc = pf;
    targets.overlayFunc = of;
    this.items = targets;
  },
  hasContent: function hasContent() {
    return false;
  },
  destroy: function destroy() {
    var pf = this.items.panelFunc;
    var of = this.items.overlayFunc;

    for (var i = 0; i < this.items.length; i++) {
      this.items[i].panel.removeEventListener("click", pf, false);
      this.items[i].overlay.removeEventListener("click", of, false);
    }

    this.items = [];
  }
};
lnx.learningCheck = {
  items: [],
  id: null,
  cancelTimeout: null,
  init: function init(node, screenElm, frameElm) {
    var self = this;
    var id = screenElm.getAttribute("id");
    var ctnClass = ".LearningCheck";
    var btn = screenElm.querySelector(ctnClass + " > div > div");
    var img = screenElm.querySelector(ctnClass + " > div > div img");
    var btnTxt = btn.querySelector("p");
    var header = screenElm.querySelector(ctnClass + " > div");
    var panel = screenElm.querySelector(ctnClass + " > div:nth-child(2)");
    var txt = screenElm.querySelector(ctnClass + " > div:nth-child(2) > div:first-child");
    var txtAlts = screenElm.querySelectorAll(ctnClass + " > div:nth-child(2) > div:nth-of-type(2) > *");
    txtAlts = Array.prototype.slice.call(txtAlts);

    if (txtAlts.length < 6) {
      txtAlts[5] = {
        outerHTML: ""
      };
    }

    var correctAns = screenElm.getAttribute("data-correctAns");
    var prxBtn = screenElm.querySelector(".twoOneCol > div:nth-child(3)");
    var con = screenElm.querySelector(ctnClass);
    var qs = screenElm.querySelector(".twoOneCol > div:nth-child(1)");
    con.style.zIndex = "-1";
    var attempts = 0;
    var mustGetCorrect = node.getAttribute("subType") === "2" ? true : false;
    var resultAudio = screenElm.getAttribute("data-resultAudio");

    if (resultAudio) {
      resultAudio = resultAudio.split(",");
    } else {
      resultAudio = ["NotCorrect", "PartCorrect", "Correct"];
    }

    var delayAudio = null;
    var a = screenElm.querySelector("[data-delayAudio]");

    if (a) {
      delayAudio = a.getAttribute("data-delayAudio").split(",");
    }

    var doCourseCompletion = screenElm.getAttribute("data-doCourseCompletion") === "true";
    var isBinary = screenElm.querySelectorAll("label").length === 2;
    var open = false;
    var tl = new TimelineMax();
    var pHt = window.getComputedStyle(panel).height;
    panel.style.height = "0px"; // prxBtn.style.display = "none";
    // btn.style.display = "none";
    //btnTxt.innerHTML = lnx.localization.getLocalString("s13");
    //tl.to(btnTxt, .2, {text: lnx.localization.getLocalString("s13")});

    tl.to(header, .75, {
      top: "0px"
    }, 0);
    tl.to(btn, .75, {
      right: "15px"
    }, 0);
    tl.to(panel, .75, {
      height: pHt,
      onReverseComplete: onReverseComplete
    }, 0);
    tl.to(txt, .3, {
      opacity: 1
    });
    tl.pause();
    lnx.util.stopVOverflow(screenElm.querySelector("div.LearningCheckCon > div.twoOneCol > div:first-child"));

    function onReverseComplete() {
      con.style.zIndex = "-1";
      prxBtn.style.display = "block";
      btn.style.display = "none";

      if (attempts < 2) {
        var lCheck = document.getElementsByName('lCheck');

        for (var i = 0; i < lCheck.length; i++) {
          lCheck[i].checked = false;
        }
      } // prxBtn.style.display = "none";
      // btn.style.display = "none";

    }

    var f = function f(e) {
      clearTimeout(self.cancelTimeout);
      lnx.audio.stopAudio();

      if (open) {
        tl.reverse();
        open = false;
      } else {
        con.style.zIndex = "10";
        prxBtn.style.display = "none";
        btn.style.display = "block";
        var r = getResult(),
            t;
        attempts++;
        txt.innerHTML = "";
        var PART_CORRECT = 0;
        var idx = 1;
        var num = 2;

        if (r) {
          txt.style.paddingTop = 0;
          t = txtAlts[2].outerHTML + txtAlts[4].outerHTML + txtAlts[5].outerHTML;
          lnx.cache.setValue("complete", id, true);
          lnx.view.onScreenComplete();

          if (resultAudio) {
            lnx.audio.playAudio(resultAudio[2]);
          }

          if (doCourseCompletion) {
            lnx.scormApi.lmsCompleteCourse(true);
          }
        } else if (!isBinary && (attempts < 2 || mustGetCorrect)) {
          num = r === PART_CORRECT ? 1 : 0;
          t = txtAlts[num].outerHTML + txtAlts[3].outerHTML;
          idx = 0;
        } else {
          num = r === PART_CORRECT ? 1 : 0;
          txt.style.paddingTop = 0;
          t = txtAlts[num].outerHTML + txtAlts[4].outerHTML + txtAlts[5].outerHTML;
          lnx.cache.setValue("complete", id, true);
          lnx.view.onScreenComplete();
        }

        if (resultAudio) {
          lnx.audio.playAudio(resultAudio[num]);
        }

        if (delayAudio) {
          self.cancelTimeout = setTimeout(function () {
            lnx.audio.playAudio(delayAudio[idx]);
          }, 2750);
        }

        txt.innerHTML = t;

        if (r || isBinary || !r && attempts > 1 && !mustGetCorrect) {
          tl.clear();
          tl.to(btn, .1, {
            opacity: 0
          }, 0);
          tl.to(header, .75, {
            top: "0px"
          }, 0);
          tl.to(panel, .75, {
            height: pHt
          }, 0);
          tl.to(txt, .3, {
            opacity: 1
          });
          tl.play();
        } else {
          tl.play();
        }

        open = true;
      }
    };

    function getResult() {
      var result = null;
      var lCheck = document.getElementsByName('lCheck');
      var checked = [];

      for (var i = 0; i < lCheck.length; i++) {
        if (lCheck[i].checked) {
          checked.push(lCheck[i].value);

          if (correctAns.search(lCheck[i].value + "(,|$)") > -1) {
            result = 0;
          }
        }
      }

      checked = checked.sort().join(",");
      return correctAns === checked ? 1 : result;
    }

    btn.addEventListener("click", f, false);

    prxBtn.onclick = function () {
      f();
    }; // qs.onchange = function(){
    //     console.log('onchange');
    //     prxBtn.style.display = "block";
    //     btn.style.display = "block";
    // };


    this.items = [btn, f, prxBtn, qs];
  },
  hasContent: function hasContent() {
    return false;
  },
  destroy: function destroy() {
    this.items[0].removeEventListener("click", this.items[1], false);
    this.items[2].onclick = null; //this.items[3].onchange = null;

    this.items = [];
    clearTimeout(this.cancelTimeout);
  }
}, lnx.learnHow = {
  items: [],
  init: function init(node, screenElm, frameElm) {
    var id = screenElm.getAttribute("id");
    var isComplete = false;
    var ctnClass = ".learnHowContainer";

    if (node.getAttribute("subType") === "more") {
      // fixed error - test before release
      ctnClass = ".learnMoreContainer";
    }

    var btn = screenElm.querySelector(ctnClass + " > div > div");
    var img = screenElm.querySelector(ctnClass + " > div > div img");
    var btnTxt = btn.querySelector("p");
    var header = screenElm.querySelector(ctnClass + " > div");
    var panel = screenElm.querySelector(ctnClass + " > div:nth-child(2)");
    var txt = screenElm.querySelector(ctnClass + " > div:nth-child(2) > div");
    var open = false;
    var tl = new TimelineMax();
    var pHt = window.getComputedStyle(panel).height;
    panel.style.height = "0px";

    var f = function f(e) {
      if (!isComplete) {
        lnx.cache.setValue("complete", id, true);
        lnx.view.onScreenComplete();
        isComplete = true;
      }

      if (open) {
        tl.reverse();
      } else {
        if (tl.reversed()) {
          tl.play();
        } else {
          tl.to(btnTxt, .2, {
            opacity: 0
          });
          tl.to(header, .75, {
            top: "0px"
          }, 0);
          tl.to(btn, .75, {
            right: "20px"
          }, 0);
          tl.to(panel, .75, {
            height: pHt
          }, 0);
          tl.to(txt, .3, {
            opacity: 1
          });
          tl.to(img, 0, {
            src: "images/button_close.png"
          }, .2);
        }
      }

      open = !open;
    };

    btn.addEventListener("click", f, false);
    this.items = [btn, f];
  },
  hasContent: function hasContent() {
    return false;
  },
  destroy: function destroy() {
    this.items[0].removeEventListener("click", this.items[1], false);
    this.items = [];
  }
};
lnx.moveUp = {
  items: [],
  init: function init(node, screenElm, frameElm) {
    var id = screenElm.getAttribute("id");
    var isComplete = false;
    var ctnClass = ".moveUpContainer";
    var con = screenElm.querySelector(ctnClass);
    var btn = screenElm.querySelector(ctnClass + " > div > div");
    var img = screenElm.querySelector(ctnClass + " > div:nth-child(1) > div");
    var btnTxt = btn.querySelectorAll("p")[0];
    var btnClose = btn.querySelectorAll("p")[1];
    var header = screenElm.querySelector(ctnClass + " > div");
    var panel = screenElm.querySelector(ctnClass + " > div:nth-child(2)");
    var txt = screenElm.querySelector(ctnClass + " > div:nth-child(2) > div");
    var audio = screenElm.querySelector(ctnClass).getAttribute("data-audio"); // change height, location of div.moveUpContainer when closed to
    // allow links below to be clicked        

    var conClosedVals = {
      h: "70px",
      t: "399px",
      t2: "18px"
    };
    var conOrigVals = getOriginalVals();
    var self = this;
    lnx.util.stopVOverflow(screenElm.querySelector("div.moveUp > div:first-of-type"));
    lnx.util.stopVOverflow(screenElm.querySelector(".moveUpContainer > div:nth-of-type(2) > div"));
    updateConVals();
    var open = false;
    var tl = new TimelineMax();
    var pHt = window.getComputedStyle(panel).height;
    panel.style.height = "0px";

    var f = function f(e) {
      lnx.audio.stopAudio();

      if (!isComplete) {
        lnx.cache.setValue("complete", id, true);
        lnx.view.onScreenComplete();
        isComplete = true;
      }

      if (open) {
        tl.reverse();
      } else {
        updateConVals(true);
        lnx.audio.playAudio(audio, 1000);

        if (tl.reversed()) {
          tl.play();
        } else {
          tl.to(btnTxt, .2, {
            opacity: 0
          });
          tl.to(btnClose, .2, {
            opacity: 1
          }, .55);
          tl.to(btnClose, 0, {
            visibility: "visible"
          });
          tl.to(header, .75, {
            top: "-7px",
            onReverseComplete: updateConVals
          }, 0);
          tl.to(btn, .75, {
            right: "20px"
          }, 0);
          tl.to(btn, .75, {
            width: "26px"
          }, 0);
          tl.to(btn, .75, {
            height: "26px"
          }, 0);
          tl.to(btn, .75, {
            top: "14px"
          }, 0);
          tl.to(panel, .75, {
            height: pHt
          }, 0);
          tl.to(txt, .3, {
            opacity: 1
          }); //tl.to(img, 0, {src: "images/button_close.png"}, .2);
        }
      }

      open = !open;
    };

    btn.addEventListener("click", f, false);
    this.items = [btn, f];

    function getOriginalVals() {
      var st = window.getComputedStyle(con);
      return {
        h: st.height,
        t: st.top,
        t2: window.getComputedStyle(con.firstChild).top
      };
    }

    function updateConVals(isOpening) {
      var vals = isOpening ? conOrigVals : conClosedVals;
      con.style.height = vals.h;
      con.style.top = vals.t;
      con.firstChild.style.top = vals.t2;
    } // function stopVOverflow(){
    //     var d = screenElm.querySelector("div.moveUp > div:first-of-type");                
    //     if(d.scrollHeight > d.clientHeight){
    //         var sz = parseInt(getComputedStyle(d).fontSize);
    //         while(sz > 11){
    //             sz -= 1;
    //             d.style.fontSize = (sz + "px");
    //             if(d.scrollHeight <= d.clientHeight){
    //                 break;
    //             }
    //         }
    //     }
    // }

  },
  hasContent: function hasContent() {
    return false;
  },
  destroy: function destroy() {
    this.items[0].removeEventListener("click", this.items[1], false);
    this.items = [];
  }
};
lnx.matching = {
  items: [],
  doActivity: null,
  init: function init(node, screenElm, frameElm) {
    // read data-match attribute value list from xml
    // list order should match order of items in markup 
    // e.g. 1,3 maps to first item in left col and 3rd item in right
    var tl = new TimelineMax();
    var l = screenElm.querySelectorAll(".matching > div:nth-child(1) > div");
    var r = screenElm.querySelectorAll(".matching > div:nth-child(2) > div");
    var m = screenElm.querySelector(".matching");
    var completeMsg = screenElm.querySelector(".matching > div:nth-child(3)");
    var cv1 = screenElm.querySelector(".matchCanvas");
    var w = window.getComputedStyle(cv1).width;
    var ctx1 = cv1.getContext('2d');
    var cY1, cY2;
    var matchLines = [];
    var lineWidth = 5; //var colors = {fail:"#CC3359", success: "#7EBB00", orange: "#EC6A00"};

    var colors = {
      fail: "#aeb0bc",
      success: "#aeb0bc",
      orange: "#EC6A00",
      green: "#01AF40",
      alt: "#aeb0bc"
    };
    var origImgSrc = m.getAttribute("data-btnSrc1");
    var onImgSrc = m.getAttribute("data-btnSrc2");
    var isComplete = false;
    var self = this;
    var forRemoval = null;
    var matchItems = {
      left: {},
      right: {}
    };
    var matches = m.getAttribute("data-match").split(",");

    for (var i = 0; i < matches.length; i++) {
      matches[i] = parseInt(matches[i]) - 1; // update for easier 0 based indexing
    }

    cv1.setAttribute('width', w);
    cv1.setAttribute('height', window.getComputedStyle(cv1).height);
    w = w.slice(0, -2); // prepare for use in drawLine func below

    for (var i = 0; i < matches.length; i += 2) {
      var leftId = matches[i];
      var rightId = matches[i + 1];
      var o = {
        item: l[leftId],
        itemId: leftId,
        match: r[rightId],
        matchId: rightId,
        selected: false,
        matched: false,
        side: "left"
      };
      var o2 = {
        item: r[rightId],
        itemId: rightId,
        match: l[leftId],
        matchId: leftId,
        selected: false,
        matched: false,
        side: "right"
      };
      o.matchObj = o2;
      o2.matchObj = o;
      matchItems.left[leftId] = o;
      matchItems.right[rightId] = o2; // add lookup values to html elements       

      l[leftId].setAttribute("data-itemId", leftId);
      l[leftId].setAttribute("data-matchId", rightId);
      l[leftId].setAttribute("data-side", "left");
      r[rightId].setAttribute("data-itemId", rightId);
      r[rightId].setAttribute("data-matchId", leftId);
      r[rightId].setAttribute("data-side", "right");
    } // IE 10 does not support pointer-events: none, so clicks do not pass through canvas element
    // workaround: create a set of nodes above canvas to proxy clicks to nodes below canvas


    var dL = document.createElement("div");
    dL = m.appendChild(dL);
    var dR = dL.cloneNode();
    dR = m.appendChild(dR);
    dL.className = "matchingProxyLeft";
    dR.className = "matchingProxyRight";
    var num = l.length;
    var s = "";

    for (var i = 0; i < num; i++) {
      s += "<div></div>";
    }

    dL.innerHTML = s;
    dR.innerHTML = s;
    var h = l[0].getBoundingClientRect().height + "px"; // listeners

    var proxyMouseOver = function proxyMouseOver(e) {
      toggleOverState(e, true);
    };

    var proxyMouseOut = function proxyMouseOut(e) {
      toggleOverState(e, false);

      if (forRemoval) {
        removeListeners(forRemoval[0], forRemoval[1]);
        forRemoval = null;
      }
    };

    var proxyClick = function proxyClick(e) {
      var num = parseInt(e.currentTarget.getAttribute("data-itemId"));
      var side = e.currentTarget.getAttribute("data-side") === "left" ? l : r;
      doMatch(side[num]);
    };

    for (var i = 0; i < num; i++) {
      dR.childNodes[i].style.height = dL.childNodes[i].style.height = h;
      dR.childNodes[i].style.width = dL.childNodes[i].style.width = "100%";
      dL.childNodes[i].setAttribute("data-side", "left");
      dL.childNodes[i].setAttribute("data-itemId", i);
      dR.childNodes[i].setAttribute("data-side", "right");
      dR.childNodes[i].setAttribute("data-itemId", i);
      dL.childNodes[i].addEventListener("mouseover", proxyMouseOver, false);
      dL.childNodes[i].addEventListener("mouseout", proxyMouseOut, false);
      dR.childNodes[i].addEventListener("mouseover", proxyMouseOver, false);
      dR.childNodes[i].addEventListener("mouseout", proxyMouseOut, false);
      dL.childNodes[i].addEventListener("click", proxyClick, false);
      dR.childNodes[i].addEventListener("click", proxyClick, false);
    }

    var removeListeners = function removeListeners(ln, rn) {
      var a = [dL.childNodes[ln], dR.childNodes[rn]];

      for (var i = 0; i < a.length; i++) {
        a[i].removeEventListener("mouseover", proxyMouseOver, false);
        a[i].removeEventListener("mouseout", proxyMouseOut, false);
        a[i].removeEventListener("click", proxyClick, false);
        a[i].style.cursor = "default";
      }
    };

    var toggleOverState = function toggleOverState(e, over) {
      e.currentTarget.style.cursor = over ? "pointer" : "default";
      var num = parseInt(e.currentTarget.getAttribute("data-itemId"));
      var side = e.currentTarget.getAttribute("data-side") === "left" ? l : r;
      var f = over ? "add" : "remove";
      side[num].classList[f]("matchingBgColor");
    };

    var doMatch = function doMatch(m) {
      var id = m.getAttribute("data-itemId");
      var side = m.getAttribute("data-side") === "left" ? matchItems.left : matchItems.right;
      var opSide = m.getAttribute("data-side") === "left" ? matchItems.right : matchItems.left;
      var obj = side[id]; // if already matched, do nothing

      if (obj.matched) {
        return;
      }

      if (obj.selected) {
        // deslect
        switchState(false, m, obj);
        return;
      } // select item


      switchState(true, m, obj); // deselect any other items already selected on that side

      for (var p in side) {
        if (p !== id && side[p].selected && !side[p].matched) {
          switchState(false, side[p].item, side[p]);
        }
      } // check for a match


      if (obj.matchObj.selected) {
        //console.log('matched');
        obj.matched = obj.matchObj.matched = true;
        drawLines(obj, obj.matchObj, true); // remove listeners

        var args = obj.side === "left" ? [obj.itemId, obj.matchId] : [obj.matchId, obj.itemId]; //removeListeners(args[0], args[1]);

        forRemoval = args; // check for completion

        for (var p in opSide) {
          if (!side[p].matched) {
            return;
          }
        } // if we get here, exercise complete
        //console.log("all matched!")


        isComplete = true;
        return;
      } // check for a bad match attempt


      for (var p in opSide) {
        if (opSide[p].selected && !opSide[p].matched) {
          //console.log('bad match');
          drawLines(obj, opSide[p], false);
          return;
        }
      } // must be a new attempt, first item of pair selected only
      //console.log('new attempt at match');            

    };

    var switchState = function switchState(select, elm, obj) {
      //elm.querySelector("img").src = select ? onImgSrc : origImgSrc;
      elm.querySelector("div").style.borderColor = select ? "#0c8e3b" : "#fff";
      elm.querySelector("div:first-child").style.backgroundColor = select ? "#01AF40" : "#fff";
      elm.querySelector("div:first-child").style.color = select ? "#01AF40" : "#fff";
      obj.selected = select;
    };

    var clearBadSelectStates = function clearBadSelectStates(o1, o2) {
      switchState(false, o1.item, o1);
      switchState(false, o2.item, o2);
    }; // utility function called when generating screen shots only


    self.doActivity = function () {
      drawLines(null, null, null, "shortCircuit");
    };

    function drawLines(obj1, obj2, success, shortCircuit) {
      // hack to access inner drawMatchLines function when generating screenshots
      if (shortCircuit === "shortCircuit") {
        shortCircuitActivity();
        return;
      }

      var col1 = colors.green;
      var col2 = success ? colors.success : colors.fail;
      var pos = "+=0";
      var finalComplete = success ? goodFinalComplete : badFinalComplete;
      var color;
      var params;
      cY1 = obj1.item.offsetTop + (obj1.item.clientHeight / 2 - lineWidth / 2);
      cY2 = obj2.item.offsetTop + (obj2.item.clientHeight / 2 - lineWidth / 2);

      if (obj1.side === "right") {
        var t = cY1;
        cY1 = cY2;
        cY2 = t;
      }

      tl.clear(); // draw a line 4 times alternating color

      for (var i = 0; i < 5; i++) {
        color = i % 2 ? col2 : col1;
        params = {
          onStart: clearCanvas,
          onComplete: onComplete,
          onCompleteParams: [color]
        };

        if (i) {
          pos = "+=.4"; // position at .4 second on timeline except for first time
        }

        if (i === 4) {
          // special case for final complete callback
          params.onComplete = finalComplete;
          color = col1;
        }

        console.log("drawlines: " + color);
        tl.to(ctx1, 0, params, pos);
      }

      function shortCircuitActivity() {
        for (var i = 0; i < matches.length; i += 2) {
          obj1 = matchItems.left[matches[i]];
          obj2 = matchItems.left[matches[i + 1]];
          matchLines.push([obj1.item.offsetTop + (obj1.item.clientHeight / 2 - lineWidth / 2), obj2.item.offsetTop + (obj2.item.clientHeight / 2 - lineWidth / 2)]);
        }

        drawMatchLines();
        showCompletion();
      }

      function onComplete(c) {
        draw(null, null, c);
        drawMatchLines();
      }

      function goodFinalComplete(c) {
        draw(null, null, c);
        matchLines.push([cY1, cY2]);
        drawMatchLines();

        if (isComplete) {
          showCompletion();
        }
      }

      function badFinalComplete(c) {
        draw(null, null, c);
        clearCanvas();
        drawMatchLines();
        clearBadSelectStates(obj1, obj2);
      }

      function drawMatchLines() {
        for (var i = 0; i < matchLines.length; i++) {
          var y1 = matchLines[i][0];
          var y2 = matchLines[i][1];
          draw(y1, y2, colors.green);
        }
      }

      function draw(y1, y2, c) {
        if (!y1) {
          // if we are not passed coords use current ones in closure
          y1 = cY1;
          y2 = cY2;
        }

        ctx1.strokeStyle = c;
        ctx1.lineWidth = lineWidth;
        ctx1.beginPath();
        ctx1.moveTo(0, y1);
        ctx1.lineTo(w, y2);
        ctx1.stroke();
      }

      function clearCanvas() {
        ctx1.clearRect(0, 0, cv1.width, cv1.height);
      }

      function showCompletion() {
        completeMsg.style.display = "block";
        var audio = completeMsg.getAttribute("data-audio");

        if (audio) {
          lnx.audio.playAudio(audio);
        }
      }
    } // remember for removel of event listerners


    this.items = [dL, dR, proxyMouseOver, proxyMouseOut, proxyClick];
  },
  hasContent: function hasContent() {
    return false;
  },
  destroy: function destroy() {
    var dL = this.items[0];
    var dR = this.items[1];
    var funcs = ["proxyMouseOver", "proxyMouseOut", "proxyClick"];

    for (var i = 0; i < dL.length; i++) {
      for (var j = 0; j < funcs.length; j++) {
        dL[i].removeEventListener(funcs[j], this.items[j + 2], false);
        dR[i].removeEventListener(funcs[j], this.items[j + 2], false);
      }
    }

    this.items = [];
  }
};
lnx.imgSequence = {
  items: [],
  init: function init(node, screenElm, frameElm) {
    var imgs = screenElm.querySelectorAll("img");
    var tl = new TimelineMax({
      repeat: -1
    });
    tl.staggerFromTo(imgs, 2, {
      opacity: 0
    }, {
      opacity: 1
    }, 4);
    this.items.push(tl);
  },
  hasContent: function hasContent() {
    return false;
  },
  destroy: function destroy() {
    this.items[0].kill();
    this.items = [];
  }
};
lnx.selectRevealEmail = {
  items: [],
  init: function init(node, screenElm, frameElm) {
    var elms = screenElm.querySelectorAll(".emailCircs > div");
    var fb = screenElm.querySelector(".slFeedback");
    var txts = screenElm.querySelectorAll(".slTxtContainer > span");
    var scoreElm = screenElm.querySelector(".slScore > span");
    var score = 0;
    var list = [];
    var isComplete = false;
    var id = screenElm.getAttribute("id");

    for (var i = 0; i < elms.length; i++) {
      elms[i].onclick = onClick;
      elms[i].setAttribute("data-num", i);
      list.push(elms[i]);
    }

    function onClick(e) {
      e = e.currentTarget;

      if (list.length) {
        for (var i = 0; i < list.length; i++) {
          if (list[i] === e) {
            //e.style.backgroundColor = "#905a02";
            e.style.opacity = ".5";
            list.splice(i, 1);
            scoreElm.innerHTML = ++score;
            lnx.audio.stopAudio();

            if (!list.length) {
              if (!isComplete) {
                lnx.cache.setValue("complete", id, true);
                lnx.view.onScreenComplete();
                isComplete = true;
              }

              var audio = fb.getAttribute("data-audio");

              if (audio) {
                lnx.audio.playAudio(audio);
              }
            }

            break;
          }
        }
      } //e.style.opacity = "1";


      fb.style.display = "block";
      fb.innerHTML = txts[e.getAttribute("data-num")].outerHTML;
    }

    this.items[0] = elms;
  },
  hasContent: function hasContent() {
    return false;
  },
  destroy: function destroy() {
    if (this.items[0]) {
      var elms = this.items[0];

      for (var i = 0; i < elms.length; i++) {
        elms[i].onclick = null;
      }
    }

    this.items = [];
  }
};
lnx.selectReveal = {
  items: [],
  init: function init(node, screenElm, frameElm) {
    var elms = screenElm.querySelectorAll(".slContainer > div");
    var fb = screenElm.querySelector(".slFeedback");
    var txts = screenElm.querySelectorAll(".slTxtContainer > span");
    var scoreElm = screenElm.querySelector(".slScore > span");
    var score = 0;
    var list = [];
    var isComplete = false;
    var id = screenElm.getAttribute("id");

    for (var i = 0; i < elms.length; i++) {
      elms[i].onclick = onClick;
      elms[i].setAttribute("data-num", i);
      list.push(elms[i]);
    }

    function onClick(e) {
      e = e.target;

      if (list.length) {
        for (var i = 0; i < list.length; i++) {
          if (list[i] === e) {
            list.splice(i, 1);
            scoreElm.innerHTML = ++score;
            lnx.audio.stopAudio();

            if (!list.length) {
              //screenElm.querySelector(".slImageOvelay").style.display = "block";
              var elm = screenElm.querySelector(".slImageOvelay");
              elm.style.opacity = "0";
              elm.style.display = "block";
              var tl = new TimelineMax();
              tl.to(elm, 1, {
                opacity: 1,
                delay: .75
              });

              if (!isComplete) {
                lnx.cache.setValue("complete", id, true);
                lnx.view.onScreenComplete();
                isComplete = true;
              }

              var audio = elm.getAttribute("data-audio");

              if (audio) {
                lnx.audio.playAudio(audio);
              }
            }

            break;
          }
        }
      }

      e.style.opacity = "1";
      fb.style.display = "block";
      fb.innerHTML = txts[e.getAttribute("data-num")].outerHTML;
    }

    this.items[0] = elms;
  },
  hasContent: function hasContent() {
    return false;
  },
  destroy: function destroy() {
    if (this.items[0]) {
      var elms = this.items[0];

      for (var i = 0; i < elms.length; i++) {
        elms[i].onclick = null;
      }
    }

    this.items = [];
  }
};
lnx.overlayMan = {
  overlay: null,
  closeBtn: null,
  payLoad: null,
  initialized: false,
  init: function init() {
    var self = this;
    this.overlay = document.body.querySelector('.overlay');
    if (!this.overlay) return;

    this.overlay.querySelector("img").onclick = function () {
      self.hide();
    };

    this.payLoad = this.overlay.querySelector("div:nth-of-type(2)");
    this.initialized = true;
  },
  show: function show(text) {
    if (!this.initialized) return;
    this.overlay.style.display = "block";
    this.payLoad.innerHTML = text;
  },
  hide: function hide() {
    if (!this.initialized) return;
    this.overlay.style.display = "none";
  }
};
lnx.basicWithVideo = {
  link: null,
  vid: null,
  init: function init(node, screenElm, frameElm) {
    var self = this;
    this.link = screenElm.querySelector("span.underline");
    this.vidDiv = screenElm.querySelector("div.vidOverlay");
    this.vid = screenElm.querySelector("video");

    this.link.onclick = function (e) {
      self.vidDiv.style.display = "block";
      self.vid.play();
      lnx.audio.stopAudio();
    };

    this.vidClose = screenElm.querySelector(".videoClose");

    this.vidClose.onclick = function (e) {
      self.vidDiv.style.display = "none";
      self.vid.pause();
    };
  },
  hasContent: function hasContent() {
    return false;
  },
  destroy: function destroy() {
    this.link.onclick = null;
    this.link = null;
    this.vidClose.onclick = null;
    this.vidClose = null;
  }
};
lnx.steps = {
  items: [],
  imgRefPref: null,
  popUp: null,
  type: null,
  popUpClose: null,
  popUnderlay: null,
  handlers: [],
  hasAudio: false,
  props: [],
  vid: null,
  init: function init(node, screenElm, frameElm) {
    var self = this; // get clickable items

    var a = this.items = Array.prototype.slice.call(screenElm.querySelectorAll("div.clickable"));
    var numSteps = a.length;

    for (var i = 0; i < numSteps; i++) {
      var o = {};
      o.color = window.getComputedStyle(a[i]).backgroundColor;
      o.image = a[i].parentNode.querySelector("div[data-optionNum=\"" + a[i].getAttribute("data-optionNum") + "\"] > img").getAttribute("src");
      this.props.push(o);
    }

    this.popUp = screenElm.querySelector(".popup");
    this.popUpClose = this.popUp.querySelector("img.popupClose");
    this.popUnderlay = this.popUp.querySelector("div.popUnderlay");
    this.type = node.getAttribute("subType");

    if (this.type === "2") {
      this.imgRefPref = node.getAttribute("navId") + "_";
    }

    lnx.util.updateEventListener(this.items, "click", this.onSelection);
    lnx.util.updateEventListener(this.popUpClose, "click", onClose);
    this.handlers.push(onClose);
    lnx.util.updateEventListener(this.popUnderlay, "click", onClickUnderlay);
    this.handlers.push(onClickUnderlay);

    function onClose(e) {
      var video = self.popUp.querySelector("video");

      if (video) {
        video.pause();
        self.vidLink = self.popUp.querySelector("p>span.underline");
        self.vidLink.onclick = null;
        self.videoDiv = self.popUp.querySelector("div.vidOverlay");
        self.videoDiv.style.display = "none";
        self.vidClose.onclick = null;
        self.vidClose = null;
      }

      self.popUp.style.display = "none";

      if (self.hasAudio) {
        lnx.audio.stopAudio();
      }
    }

    function onClickUnderlay(e) {
      if (e.target === e.currentTarget) {
        onClose();
      }
    }

    this.navBtns = Array.prototype.slice.call(this.popUp.querySelectorAll("img.clickable"));
    lnx.util.updateEventListener(this.navBtns, "click", onNav);
    this.handlers.push(onNav);

    if (!this.navBtns[0].getAttribute("data-bupTitle")) {
      this.navBtns[0].setAttribute("data-bupTitle", this.navBtns[0].getAttribute("title"));
      this.navBtns[1].setAttribute("data-bupTitle", this.navBtns[1].getAttribute("title"));
    }

    function onNav(e) {
      var self = lnx.steps;
      var num = parseInt(e.currentTarget.parentNode.getAttribute("data-optionNum"));
      var fwd = e.currentTarget.classList.contains("stepFwd");
      var doNav = false;

      if (fwd && num < numSteps || !fwd && num > 1) {
        num = fwd ? ++num : --num;
        doNav = true;
      }

      if (doNav) {
        var gp = e.currentTarget.parentNode.parentNode;
        var t = gp.querySelector("div.step.clickable[data-optionNum=\"" + num + "\"]");
        onClose();
        sendEvent(t);
      }

      self.setNavBtnStates(num, numSteps);

      function sendEvent(n) {
        var e = document.createEvent('MouseEvents');
        e.initMouseEvent('click', true, false, window, 0, 0, 0, 0, 0, false, false, false, false, 0, null);
        n.dispatchEvent(e);
      }
    }
  },
  setNavBtnStates: function setNavBtnStates(num, numSteps) {
    var self = lnx.steps;

    if (num > numSteps - 1) {
      self.navBtns[1].classList.remove("clickable");
      self.navBtns[1].style.opacity = ".35";
      self.navBtns[1].setAttribute("title", "");
    } else {
      self.navBtns[1].classList.add("clickable");
      self.navBtns[1].style.opacity = "1";
      self.navBtns[1].setAttribute("title", self.navBtns[1].getAttribute("data-bupTitle"));
    }

    if (num < 2) {
      self.navBtns[0].classList.remove("clickable");
      self.navBtns[0].style.opacity = ".35";
      self.navBtns[0].setAttribute("title", "");
    } else {
      self.navBtns[0].classList.add("clickable");
      self.navBtns[0].style.opacity = "1";
      self.navBtns[0].setAttribute("title", self.navBtns[0].getAttribute("data-bupTitle"));
    }
  },
  onSelection: function onSelection(e) {
    e = e || window.event;
    var self = lnx.steps;
    var num = parseInt(e.currentTarget.getAttribute("data-optionNum"));
    var audio = e.currentTarget.getAttribute("data-audio");
    self.popUp.setAttribute("data-optionNum", "" + num);
    self.popUp.querySelector("div.popHeader").style.backgroundColor = self.props[num - 1].color;
    self.popUp.querySelector("div.popHeader > img").setAttribute("src", "images/step" + num + ".png");
    self.popUp.querySelector("div.popBody > img").setAttribute("src", self.props[num - 1].image);
    self.setNavBtnStates(num);

    if (self.type === "2") {
      self.popUp.querySelector("div.popHeader > img").src = "images/" + self.imgRefPref + num + ".jpg";
    } else if (self.type === "3") {
      var hdr = self.popUp.querySelector(".popHeader");
      hdr.style.backgroundColor = window.getComputedStyle(e.currentTarget, null).backgroundColor;
      hdr.innerHTML = e.currentTarget.querySelector("div>span").innerHTML;
    } else if (self.type === "4") {
      var hdr = self.popUp.querySelector(".popHeader");
      var num = e.currentTarget.getAttribute("data-optionNum");
      var color = "#AA0061";

      if (num === "1") {
        color = "#01AF40";
      } else if (num === "2") {
        color = "#F6B331";
      }

      hdr.style.backgroundColor = color;
      hdr.innerHTML = e.currentTarget.querySelector("span").innerHTML;
    }

    if (audio) {
      self.hasAudio = true;
      lnx.audio.playAudio(audio);
    }

    self.popUp.querySelector(".popBody > div").innerHTML = e.currentTarget.querySelector(".target").innerHTML;
    var video = self.popUp.querySelector("video");

    if (video) {
      self.vidLink = self.popUp.querySelector("p>span.underline");
      self.videoDiv = self.popUp.querySelector("div.vidOverlay");

      self.vidLink.onclick = function (e) {
        lnx.audio.stopAudio();
        self.videoDiv.style.display = "block";
        video.play();
      };

      self.vidClose = self.popUp.querySelector(".videoClose");

      self.vidClose.onclick = function (e) {
        self.videoDiv.style.display = "none";
        video.pause();
      };
    }

    self.popUp.style.display = "block";
    self.popUp.focus();
    e.stopPropagation ? e.stopPropagation() : e.cancelBubble = true;
    e.preventDefault ? e.preventDefault() : e.returnValue = false;
    return false;
  },
  hasContent: function hasContent() {
    return false;
  },
  destroy: function destroy() {
    lnx.util.updateEventListener(this.items, "click", this.onSelection, true);
    lnx.util.updateEventListener(this.popUpClose, "click", this.handlers[0], true);
    lnx.util.updateEventListener(this.popUnderlay, "click", this.handlers[1], true);
    lnx.util.updateEventListener(this.navBtns, "click", this.handlers[2], true);
    this.current = this.imgRefPref = this.popUp = this.popUnderlay = this.popUpClose = null;
    this.items = [];
    this.handlers = [];
    this.navBtns = [];
    this.props = [];
    this.hasAudio = false;
  }
};
lnx.scenarioWithVirtualScreens = {
  items: [],
  id: null,
  scenario: null,
  stage: 0,
  type: null,
  resultShown: false,
  audio: [],
  delayAudio: null,
  cancelTimeout: null,
  complete: false,
  init: function init(node, screenElm, frameElm) {
    var self = this;
    this.resultShown = false;
    this.id = screenElm.getAttribute("id");
    this.stage = 1;
    this.type = screenElm.querySelectorAll(".vsPanel").length === 2 ? 1 : 2;
    this.scenario = screenElm.querySelector(".vContainer");
    addAudio();
    this.setUpScenario(this.scenario);

    function addAudio() {
      self.audio.push(screenElm.getAttribute("data-audio"));
      var a = screenElm.querySelectorAll("[data-audio]");

      for (var i = 0; i < a.length; i++) {
        self.audio.push(a[i].getAttribute("data-audio"));
      }

      a = screenElm.querySelector("[data-delayAudio]");

      if (a) {
        self.delayAudio = a.getAttribute("data-delayAudio");
      }
    }
  },
  setUpScenario: function setUpScenario(scenario) {
    lnx.audio.playAudio(this.audio[0]);
    var type = this.type;
    var self = this;
    var isBut = true;
    var isMultiSubmit = null;
    var isBinaryNonBut = false;
    var a = scenario.querySelectorAll(".vsBut"),
        self = this;

    if (a && !a.length) {
      // must be non-button responses
      isBut = false;
      a = scenario.querySelectorAll(".vsAns");
      isMultiSubmit = scenario.querySelector(".vsSubmit");
    }

    for (var i = 0; i < a.length; i++) {
      a[i].onclick = onClick;
    }

    this.items = a;

    if (isMultiSubmit) {
      isMultiSubmit.onclick = onClick;

      if (a.length === 2 || scenario.querySelector(".vsSingleSelect")) {
        isBinaryNonBut = true;
      }

      this.items[this.items.length] = isMultiSubmit;
    }

    var selected;
    adjustHeight();

    function adjustHeight() {
      var p2 = scenario.querySelector(".vsPanel2");

      if (!p2) {
        return;
      }

      var d1 = p2.childNodes[0];
      var d2 = p2.childNodes[1];
      var ans = d2.querySelectorAll(".vsAns");

      if (!ans.length) {
        return;
      }

      var sub = d2.querySelector(".vsSubmit");

      if (sub) {
        var subPosition = window.getComputedStyle(sub).position;
        sub.style.position = "static";
      }

      var pad = parseInt(window.getComputedStyle(ans[0]).paddingTop);

      if (isNaN(pad)) {
        pad = 0;
      }

      var fs = parseInt(window.getComputedStyle(ans[0]).fontSize);
      var i = 0;
      var startOnFont = false;

      while (is2High() && ++i < 20) {
        pad = pad - 2;

        if (pad < 14 && !startOnFont) {
          startOnFont = true;
        }

        if (pad > 3) {
          reduceSize(ans, "paddingTop", pad);
          reduceSize(ans, "paddingBottom", pad);
        }

        if (!is2High()) {
          break;
        }

        if (fs > 11 && startOnFont) {
          fs = fs - 1;
          reduceSize(ans, "fontSize", fs);
        }
      }

      if (sub) {
        sub.style.position = subPosition;
      }

      function is2High() {
        var p2h = getH(p2);
        var d1h = getH(d1);
        var d2h = getH(d2);
        return d1h + d2h > p2h;

        function getH(elm) {
          return parseInt(window.getComputedStyle(elm).height) + parseInt(window.getComputedStyle(elm).paddingTop) + parseInt(window.getComputedStyle(elm).paddingBottom);
        }
      }

      function reduceSize(elms, prop, val) {
        for (var i = 0; i < elms.length; i++) {
          elms[i].style[prop] = val + "px";
        }
      }
    }

    function onClick(e) {
      var isCorrect = false;

      if (isBut) {
        isCorrect = e.currentTarget.getAttribute("data-iscorrect") === "true";
        var c = isBut ? "vsButSelected" : "vsAnsSelected";

        if (selected) {
          selected.classList.remove(c);

          if (!isBut) {
            selected.querySelector("span").classList.remove("vsAnsSpanSelected");
          }
        }

        e.currentTarget.classList.add(c);

        if (!isBut) {
          e.currentTarget.querySelector("span").classList.add("vsAnsSpanSelected");
        }

        selected = e.currentTarget;
        var r = isCorrect ? 2 : 0;
        showResult(r, false);
      } else {
        if (e.currentTarget.classList.contains("vsSubmit")) {
          var oneTrue, oneFalse;
          oneTrue = oneFalse = false;

          for (var i = 0; i < a.length; i++) {
            if (a[i].getAttribute("data-iscorrect")) {
              if (a[i].classList.contains("vsAnsSelected")) {
                oneTrue = true;
              } else {
                oneFalse = true;
              }
            } else {
              if (a[i].classList.contains("vsAnsSelected")) {
                oneFalse = true;
              }
            }
          }

          if (oneTrue && !oneFalse) {
            showResult(2);
          } else if (oneTrue && oneFalse) {
            showResult(1);
          } else {
            showResult(0);
          }
        } else if (isBinaryNonBut) {
          var c = isBut ? "vsButSelected" : "vsAnsSelected";

          if (selected) {
            selected.classList.remove(c);

            if (!isBut) {
              selected.querySelector("span").classList.remove("vsAnsSpanSelected");
            }
          }

          e.currentTarget.classList.add(c);

          if (!isBut) {
            e.currentTarget.querySelector("span").classList.add("vsAnsSpanSelected");
          }

          selected = e.currentTarget;
        } else {
          e.currentTarget.classList.toggle("vsAnsSelected");
          e.currentTarget.querySelector("span").classList.toggle("vsAnsSpanSelected");
        }
      }
    }

    function showResult(r) {
      lnx.audio.stopAudio();
      var p3 = scenario.querySelector(".vsPanel3");
      var cor = p3.querySelector(".vsCorrect");
      var nCor = p3.querySelector(".vsNotCorrect");
      var pCor = p3.querySelector(".vsPartCorrect");
      var audioIdx = self.type === 2 ? 2 : 1;

      if (r === 2) {
        cor.style.display = "block";
        nCor.style.display = "none";
        if (pCor) pCor.style.display = "none";
      } else if (r === 1) {
        if (pCor) pCor.style.display = "block";
        nCor.style.display = "none";
        cor.style.display = "none";
        audioIdx = self.type === 2 ? 4 : 3;
      } else {
        nCor.style.display = "block";
        if (pCor) pCor.style.display = "none";
        cor.style.display = "none";
        audioIdx = self.type === 2 ? 3 : 2;
      }

      var sel = ".vsFeedback";
      var incorFb = p3.querySelector(".vsFeedback2");

      if (incorFb) {
        if (r !== 2) {
          p3.querySelector(".vsFeedback").style.display = "none";
          incorFb.style.display = "block";
        } else {
          p3.querySelector(".vsFeedback").style.display = "block";
          incorFb.style.display = "none";
        }
      } else {
        p3.querySelector(".vsFeedback").style.display = "block";
      }

      self.stage = self.type === 1 ? 2 : 3;
      lnx.audio.playAudio(self.audio[audioIdx]);
      lnx.cache.setValue("complete", this.id, true); //comment out? 2020
      //lnx.view.onScreenComplete();

      if (self.delayAudio) {
        clearTimeout(self.cancelTimeout);
        self.cancelTimeout = setTimeout(function () {
          lnx.audio.playAudio(self.delayAudio);
        }, 2750);
      }

      if (self.resultShown) return;
      var p2 = scenario.querySelector(".vsPanel2");
      var t = new TimelineMax();

      if (type === 2) {
        t.to(p2, .9, {
          left: "-=496px"
        });
      }

      t.to(p3, .9, {
        left: "-=434px"
      }, .5);
      self.resultShown = true;
    }
  },
  hasContent: function hasContent() {
    if (this.initialized) {
      return true;
    } else {
      return false;
    }
  },
  updateVirtualScreen: function updateVirtualScreen(num, dir) {
    var id = this.id;
    slidePanel("-413px", .8);
    this.stage = 2;

    function slidePanel(val, time) {
      var vc = document.querySelector("#" + id + " .vContainer");
      var tl = new TimelineMax();
      tl.to(vc, time, {
        left: val
      });
    }
  },
  insertVirtualScreen: function insertVirtualScreen(dir) {
    if (this.type === 2) {
      if (dir === "next" && this.stage === 1) {
        lnx.audio.stopAudio();
        this.updateVirtualScreen(dir);
        lnx.audio.playAudio(this.audio[1]);
        return true;
      } else if (dir === "prev" && this.stage === 3) {
        this.returnToStart(3);
        return true;
      } else if (dir === "prev" && this.stage === 2) {
        this.returnToStart(2);
        return true;
      } else {
        return false;
      }
    } else {
      return false;
    }
  },
  isFinalScreen: function isFinalScreen() {
    return this.resultShown;
  },
  waitForUnlock: function waitForUnlock() {
    return this.type === 1;
  },
  onAudioComplete: function onAudioComplete() {},
  returnToStart: function returnToStart(st) {
    if (st === 3) {
      var p3 = this.scenario.querySelector(".vsPanel3");
      var cor = p3.querySelector(".vsCorrect");
      var nCor = p3.querySelector(".vsNotCorrect");
      var pCor = p3.querySelector(".vsPartCorrect");
      cor.style.display = "none";
      nCor.style.display = "none";
      if (pCor) pCor.style.display = "none";
      var fb = p3.querySelector(".vsFeedback");
      fb.style.display = "none";
      var p2 = this.scenario.querySelector(".vsPanel2");
      var t = new TimelineMax();
      t.to(p3, .9, {
        left: "+=434px"
      });
      t.to(this.scenario, .9, {
        left: "+=434px"
      }, .9);
      t.to(p2, .9, {
        left: "+=496px"
      }, 1.8);
    } else {
      //var p2 = this.scenario.querySelector(".vsPanel2");
      var t = new TimelineMax();
      t.to(this.scenario, .9, {
        left: "+=434px"
      }); //t.to(p2, .9, {left : "+=496px"}, 1.8);
    }

    this.stage = 1;
    this.resultShown = false;
    var a = this.items;

    for (var i = 0; i < a.length; i++) {
      a[i].classList.remove("vsButSelected");
      a[i].classList.remove("vsAnsSelected");
      a[i].firstChild.classList.remove("vsAnsSpanSelected");
    }

    this.selected = null;
    clearTimeout(this.cancelTimeout);
    lnx.audio.playAudio(this.audio[0]);
  },
  isComplete: function isComplete() {
    return lnx.cache.getValue("complete", this.id);
  },
  destroy: function destroy(newType, navId) {
    if (navId === this.id) {
      return;
    }

    this.removeHandlers();
    this.audio = [];
    clearTimeout(this.cancelTimeout);
    this.resultShown = false;
  },
  removeHandlers: function removeHandlers() {
    var a = this.items;

    for (var i = 0; i < a.length; i++) {
      a[i].onclick = null;
    }

    this.items = [];
  }
};
lnx.verticalScenario = {
  items: [],
  id: null,
  scenario: null,
  stage: 0,
  type: null,
  resultShown: false,
  audio: [],
  delayAudio: null,
  cancelTimeout: null,
  complete: false,
  screenAnims: [],
  timelines: null,
  init: function init(node, screenElm, frameElm) {
    var self = this;
    this.resId = node.getAttribute("resId");
    this.resultShown = false;
    this.id = screenElm.getAttribute("id");
    this.vScreens = screenElm.querySelectorAll(".scenarioVerticalScreen");
    this.vPanel = screenElm.querySelector(".scenarioVerticalPanel");
    this.dummyPanel = screenElm.querySelector(".vsDummyPanel");
    this.stage = 1;
    this.screenAnims = [];
    this.timelines = [];
    var b = this.buttons = []; // var vr = screenElm.querySelectorAll(".scenarioVericalResult > div");
    // for(var i = 0; i < vr.length; i++){
    //     lnx.util.stopHOverflow(vr[i]);
    // }        

    this.createAnims();
    setupQuiz();
    this.playAudio(1);

    function setupQuiz() {
      b = b.concat(Array.prototype.slice.call(screenElm.querySelectorAll(".vs2Ans")));
      b.push(screenElm.querySelector(".vs2But"));

      for (var i = 0; i < b.length; i++) {
        b[i].onclick = onClick;
      }
    }

    function onClick(e) {
      var isCorrect = 0;
      var isSubmit = false;
      var t = e.currentTarget;

      if (t.classList.contains("vs2But")) {
        isSubmit = true;

        for (var i = 0; i < b.length; i++) {
          if (b[i].getAttribute("data-iscorrect") === "true") {
            if (b[i].querySelector("input").checked) {
              isCorrect = 1;
              break;
            }
          }
        }
      } else {
        var r = t.querySelector("input");

        if (r) {
          r.checked = true;
        }
      }

      if (isSubmit) {
        showResult(isCorrect);
      }
    }

    function showResult(isCorrect) {
      var screen = self.vScreens[2];
      var resultPlate = screen.querySelector(".vs2ResultColor");
      var res = screen.querySelectorAll(".scenarioVericalResult > div");
      var fb = screen.querySelectorAll(".vs2Feedback");
      self.resultShown = true;

      for (var i = 0; i < res.length; i++) {
        res[i].style.display = i === isCorrect ? "block" : "none";

        if (fb.length > 1 && i < 2) {
          fb[i].style.display = i === isCorrect ? "block" : "none";
        }
      }

      var audioFile = "Correct";

      if (isCorrect === 1) {
        resultPlate.classList.remove("scenarioVerticalNonCorrect");
      } else {
        resultPlate.classList.add("scenarioVerticalNonCorrect");
        audioFile = "Not_Correct";
      }

      if (self.stage === 3) {
        self.playAudio(audioFile, 250); // user is just re submitting answer, do nothing

        return;
      }

      self.playAudio(audioFile, 750);
      self.insertVirtualScreen("next", "isSubmit");
    }
  },
  playAudio: function playAudio(id, delay) {
    var file = typeof id === "number" ? this.resId + "_" + id : id;
    lnx.audio.playAudio(file, delay);
  },
  createAnims: function createAnims() {
    //screen 1
    var t1 = gsap.timeline();
    var anims = [];
    anims.push(this.vScreens[0]);
    anims = anims.concat(Array.prototype.slice.call(this.vScreens[0].querySelectorAll(".anim")));
    t1.from(anims[0], {
      duration: 1.1,
      y: "+=480"
    }).from(anims[1], {
      duration: .7,
      y: "+=40",
      opacity: 0
    }, "-=.3").from(anims[5], {
      duration: 1,
      x: "+=40",
      opacity: 0
    }, "-=.3").from(anims[2], {
      duration: .5,
      y: "-=40",
      opacity: 0
    }, "-=.6").from(anims[3], {
      duration: .6,
      y: "-=40",
      opacity: 0
    }, "-=.2").from(anims[4], {
      duration: .7,
      y: "-=40",
      opacity: 0
    }, "-=.2");
    this.timelines.push(t1); //screen 2

    var t2 = gsap.timeline({
      paused: true
    });
    anims = [];
    anims.push(this.vScreens[1]);
    anims = anims.concat(Array.prototype.slice.call(this.vScreens[1].querySelectorAll(".anim")));
    anims.push(this.vPanel);
    anims.push(this.vPanel.firstElementChild);
    t2.to(anims[0], {
      duration: 0,
      zIndex: "auto"
    }).to(anims[6], {
      duration: 0,
      display: "block"
    }).to(anims[0], {
      duration: 1.1,
      y: "-=480"
    }).from(anims[1], {
      duration: .7,
      y: "+=20",
      opacity: 0
    }, "-=.3").to(anims[2], {
      duration: .7,
      rotateZ: "-=30",
      transformOrigin: "50% 100%"
    }, "-=.5").from(anims[4], {
      duration: .7,
      y: "+=30",
      opacity: 0
    }, "-=.5").to(anims[5], {
      duration: .7,
      rotateZ: "+=30",
      transformOrigin: "50% 100%"
    }, "-=.5").fromTo(anims[7], {
      opacity: 0,
      x: "+=30"
    }, {
      x: "0",
      duration: .7,
      opacity: 1
    }, "-=.35").from(anims[3], {
      duration: .7,
      y: "+=40",
      opacity: 0
    }, "-=.3");
    this.timelines.push(t2); //screen 3

    var t3 = gsap.timeline({
      paused: true
    });
    anims = [];
    anims.push(this.vScreens[2]);
    anims = anims.concat(Array.prototype.slice.call(this.vScreens[2].querySelectorAll(".anim")));
    anims.push(this.vPanel);
    anims.push(this.vPanel.firstElementChild);
    anims.push(this.dummyPanel);
    t3.to(anims[0], {
      duration: 0,
      zIndex: "auto"
    }).to(anims[0], {
      duration: 1.1,
      y: "-=480"
    }).to(anims[4], {
      duration: 1.1,
      y: "-=480"
    }, "<").to(anims[4], {
      duration: 0,
      display: "none"
    }, "-=.2").fromTo(anims[3], {
      opacity: 0,
      x: "+=70"
    }, {
      duration: .9,
      opacity: 1,
      x: "-=40",
      immediateRender: false
    }, "<").from(anims[1], {
      duration: .7,
      opacity: 0,
      x: "-=40"
    }, "-=.5");
    this.timelines.push(t3);
  },
  hasContent: function hasContent() {
    if (this.initialized) {
      return true;
    } else {
      return false;
    }
  },
  updateVirtualScreen: function updateVirtualScreen(dir) {
    if (dir === "next") {
      this.timelines[this.stage - 1].play();
    } else {
      this.timelines[this.stage].reverse();
    }

    if (this.stage < 3) {
      var delay = this.stage === 2 ? 850 : 0;
      this.playAudio(this.stage, delay);
    }
  },
  insertVirtualScreen: function insertVirtualScreen(dir, isSubmit) {
    var result = true;
    var nextStage = dir === "next" ? ++this.stage : --this.stage;

    if (nextStage < 1 || nextStage > 3) {
      result = false;
    } else if (dir === "next" && nextStage === 3 && isSubmit !== "isSubmit") {
      result = false;
    } else {
      this.updateVirtualScreen(dir);
    }

    return result;
  },
  isComplete: function isComplete() {
    return lnx.cache.getValue("complete", this.id);
  },
  isFinalScreen: function isFinalScreen() {
    return this.resultShown;
  },
  destroy: function destroy(newType, navId) {
    if (navId === this.id) {
      return;
    }

    this.removeHandlers();
    this.audio = [];
    clearTimeout(this.cancelTimeout);
    this.resultShown = false;
  },
  removeHandlers: function removeHandlers() {
    var a = this.items;

    for (var i = 0; i < a.length; i++) {
      a[i].onclick = null;
    }

    this.items = [];
  }
};
lnx.stages = {
  items: [],
  id: null,
  scenario: null,
  stage: 0,
  type: null,
  resultShown: false,
  audio: [],
  delayAudio: null,
  cancelTimeout: null,
  complete: false,
  screenAnims: [],
  timelines: null,
  init: function init(node, screenElm, frameElm, origNavId) {
    var self = this;
    this.resId = node.getAttribute("resId");
    this.resultShown = false;
    this.id = screenElm.getAttribute("id");
    this.stages = screenElm.querySelectorAll("div.stages > div.stage");
    this.stage = 1;
    this.container = screenElm.querySelector("#arrowContainer");
    this.timelines = [];
    this.createAnims();

    if (origNavId === "prev") {
      // must be reversing into activity so show last virtual screen
      this.stage = this.stages.length;
      this.insertVirtualScreen("prev", true);
    }

    this.stages[this.stage - 1].style.display = "block";
    this.playAudio(this.stage);
  },
  playAudio: function playAudio(id, delay) {
    var file = typeof id === "number" ? this.resId + "_" + id : id;
    lnx.audio.playAudio(file, delay);
  },
  createAnims: function createAnims() {
    var arrows = Array.prototype.slice.call(this.container.querySelectorAll("#arrowContainer > div.arrow"));
    var arw4Img = arrows[3].querySelector("img");
    var imageCon = this.container.querySelector("#arrowContainer > div.images");
    var images = imageCon.querySelectorAll("img");
    var stagesCon = this.container;
    var subs = stagesCon.querySelectorAll("div.arrow  ul > li");
    var bgColor = "#B64068"; //windows.getComputedStyle(arrows[3]).backgroundColor;
    //screen 1

    var t1 = gsap.timeline({
      paused: false
    });
    t1.from(arrows[0], {
      duration: .6,
      x: "-60",
      opacity: 0
    }).from(arrows[1], {
      duration: .6,
      x: "-60",
      opacity: 0
    }, "-=.4").from(arrows[2], {
      duration: .6,
      x: "-60",
      opacity: 0
    }, "-=.3").from(arrows[3], {
      duration: .6,
      x: "-60",
      opacity: 0
    }, "-=.2").to(imageCon, {
      duration: 0,
      display: "block"
    });
    this.timelines.push(t1); //screen 2

    var t2 = gsap.timeline({
      paused: true,
      defaults: {
        immediateRender: false
      }
    });
    t2.to(arrows, {
      duration: 0,
      x: 0
    }).to(arrows[3], {
      duration: 3,
      x: "-970",
      ease: "sine.in"
    }).to(arrows[2], {
      duration: 2,
      x: "-750",
      ease: "power1.in"
    }, 1.3).to(arrows[1], {
      duration: 1,
      x: "-500",
      ease: "power1.in"
    }, 2.3).to(subs, {
      duration: .75,
      opacity: 0
    }, "-=1.5");
    this.timelines.push(t2); //screen 3

    var t3 = gsap.timeline({
      paused: true,
      defaults: {
        immediateRender: false
      }
    });
    t3.to(images[9], {
      duration: 1,
      opacity: 0
    }).from(subs[0], {
      duration: .7,
      x: "-100"
    }, "-=.6").to(subs[0], {
      duration: .7,
      opacity: 1
    }, "<");
    this.timelines.push(t3); //screen 4

    var t4 = gsap.timeline({
      paused: true,
      defaults: {
        immediateRender: false
      }
    });
    t4.to(images[8], {
      duration: 1,
      opacity: 0
    }).from(subs[1], {
      duration: .7,
      x: "-100"
    }, "-=.6").to(subs[1], {
      duration: .7,
      opacity: 1
    }, "<");
    this.timelines.push(t4); //screen 5

    var t5 = gsap.timeline({
      paused: true,
      defaults: {
        immediateRender: false
      }
    });
    t5.to(subs, {
      duration: 0,
      opacity: 0
    }).to(stagesCon, {
      duration: 0,
      opacity: 0
    }).to(images[7], {
      duration: 0,
      opacity: 0
    }).to(arrows, {
      duration: 0,
      x: 0
    }).to(stagesCon, {
      duration: .6,
      opacity: 1
    }).to(arrows[3], {
      duration: 1.6,
      x: "-970",
      ease: "sine.in"
    }).to(arrows[2], {
      duration: 1.4,
      x: "-750",
      ease: "power1.in"
    }, 1.42).to(arrows[1], {
      duration: 1,
      x: "-250",
      ease: "power1.in"
    }, 2.24).to(arrows[0], {
      duration: 1,
      x: "-250",
      ease: "power1.in"
    }, "<");
    this.timelines.push(t5); //screen 6

    var t6 = gsap.timeline({
      paused: true,
      defaults: {
        immediateRender: false
      }
    });
    t6.to(images[6], {
      duration: 1,
      opacity: 0
    }).from(subs[2], {
      duration: .7,
      x: "-100"
    }, "-=.6").to(subs[2], {
      duration: .7,
      opacity: 1
    }, "<");
    this.timelines.push(t6); //screen 7

    var t7 = gsap.timeline({
      paused: true,
      defaults: {
        immediateRender: false
      }
    });
    t7.to(images[5], {
      duration: 1,
      opacity: 0
    }).from(subs[3], {
      duration: .7,
      x: "-100"
    }, "-=..6").to(subs[3], {
      duration: .7,
      opacity: 1
    }, "<");
    this.timelines.push(t7); //screen 8

    var t8 = gsap.timeline({
      paused: true,
      defaults: {
        immediateRender: false
      }
    });
    t8.to(subs, {
      duration: 0,
      opacity: 0
    }).to(stagesCon, {
      duration: 0,
      opacity: 0
    }).to(images[4], {
      duration: 0,
      opacity: 0
    }).to(arrows, {
      duration: 0,
      x: 0
    }).to(stagesCon, {
      duration: .6,
      opacity: 1
    }).to(arrows[3], {
      duration: 1.6,
      x: "-970",
      ease: "sine.in"
    }).to(arrows[2], {
      duration: 1.4,
      x: "-487"
    }, 1.41).to(arrows[1], {
      duration: 1.4,
      x: "-487"
    }, "<").to(arrows[0], {
      duration: 1.4,
      x: "-487"
    }, "<");
    this.timelines.push(t8); //GSDevTools.create({animation: t8});
    //screen 9

    var t9 = gsap.timeline({
      paused: true,
      defaults: {
        immediateRender: false
      }
    });
    t9.to(images[3], {
      duration: 1,
      opacity: 0
    }).from(subs[4], {
      duration: .7,
      x: "-100"
    }, "-=.6").to(subs[4], {
      duration: .7,
      opacity: 1
    }, "<");
    this.timelines.push(t9); //screen 10

    var t10 = gsap.timeline({
      paused: true,
      defaults: {
        immediateRender: false
      }
    });
    t10.to(subs, {
      duration: 0,
      opacity: 0
    }).to(stagesCon, {
      duration: 0,
      opacity: 0
    }).to(images[2], {
      duration: 0,
      opacity: 0
    }).to(arrows, {
      duration: 0,
      x: 0
    }).to(stagesCon, {
      duration: .6,
      opacity: 1
    }).to(arrows[3], {
      duration: 1.6,
      x: "-731"
    }).to(arrows[2], {
      duration: 1.6,
      x: "-731"
    }, "<").to(arrows[1], {
      duration: 1.2,
      x: "-550"
    }, "<").to(arrows[0], {
      duration: .8,
      x: "-250"
    }, "<").to(arrows[3], {
      duration: .2,
      backgroundColor: "transparent"
    }, "-=1.4") //.to(arw4Img, {duration:0, src:"images/arrow4b.png"}, "-=.4")         
    ;
    this.timelines.push(t10); //screen 11

    var t11 = gsap.timeline({
      paused: true,
      defaults: {
        immediateRender: false
      }
    });
    t11.to(images[1], {
      duration: 1,
      opacity: 0
    }).from(subs[5], {
      duration: .7,
      x: "-100"
    }, "-=.6").to(subs[5], {
      duration: .7,
      opacity: 1
    }, "<");
    this.timelines.push(t11); //screen 12

    var t12 = gsap.timeline({
      paused: true,
      defaults: {
        immediateRender: false
      }
    });
    t12.to(subs, {
      duration: 0,
      opacity: 1
    }).to(stagesCon, {
      duration: 0,
      opacity: 0
    }).to(arrows, {
      duration: 0,
      x: 0
    }).to(stagesCon, {
      duration: 1,
      opacity: 1
    }).to(arrows[3], {
      duration: .0,
      backgroundColor: bgColor
    }, 0) //.to(arw4Img, {duration:0, src:"images/arrow4.png"}, "<") 
    // .to(subs[0], {duration:.4, scale:1.3, transformOrigin: "50% 100%"}, 11.5)
    // .to(subs[0], {duration:.4, scale:1, transformOrigin: "0% 100%"}, 11.9)
    ;
    this.timelines.push(t12);
  },
  hasContent: function hasContent() {
    if (this.initialized) {
      return true;
    } else {
      return false;
    }
  },
  updateVirtualScreen: function updateVirtualScreen(dir, isReverse) {
    if (isReverse) {
      //nothing to do
      return;
    }

    for (var i = 0; i < this.stages.length; i++) {
      if (this.stage - 1 === i) {
        this.stages[i].style.display = "block";
      } else {
        this.stages[i].style.display = "none";
      }
    }

    var tl;

    if (dir === "prev") {
      // tl = this.timelines[this.stage];
      // if(tl){
      //     tl.kill();
      // }
      this.timelines[this.stage].reverse();
    } else {
      tl = this.timelines[this.stage];

      if (tl) {
        tl.kill(); //seek(tl.duration())
      }

      tl = this.timelines[this.stage - 1];
      tl.play();
    }

    this.playAudio(this.stage);
  },
  insertVirtualScreen: function insertVirtualScreen(dir, isReverse) {
    var result = false;

    if (dir === "next") {
      if (this.stage < 12) {
        this.stage++;
        result = true;
      }
    } else {
      if (this.stage > 1) {
        if (!isReverse) {
          this.stage--;
        }

        result = true;
      }
    }

    if (result) {
      this.updateVirtualScreen(dir, isReverse);
    }

    return result;
  },
  isComplete: function isComplete() {
    return lnx.cache.getValue("complete", this.id);
  },
  isFinalScreen: function isFinalScreen() {
    return this.resultShown;
  },
  destroy: function destroy(newType, navId) {
    if (navId === this.id) {
      return;
    }

    this.audio = [];
    clearTimeout(this.cancelTimeout);
    this.resultShown = false;
  }
};
lnx.dialogueWithVirtualScreens = {
  items: [],
  id: null,
  stage: null,
  audio: [],
  numScreens: null,
  nodeId: null,
  init: function init(node, screenElm, frameElm, origNavId) {
    var SCREENWIDTH = 930;
    var self = this;
    var audioDelay = 600;
    this.id = screenElm.getAttribute("id");
    this.resId = node.getAttribute("resId");
    var vc = document.querySelector("#" + this.id + " .dialogueContainer");

    if (!vc) {
      vc = document.querySelector("#" + this.id + " .newsContainer");
    }

    this.numScreens = vc.querySelectorAll(".newsScreen").length;
    vc.style.width = this.numScreens * SCREENWIDTH + "px"; //this.numScreens = parseInt(screenElm.getAttribute("data-numScreens"));

    this.stage = 1;
    addAudio();
    var audioIdx = 0;

    if (origNavId === "prev") {
      // must be reversing into activity so show last virtual screen
      this.insertVirtualScreen("prev", true);
      audioIdx = this.audio.length - 1;
    }

    lnx.audio.playAudio(this.audio[audioIdx]);
    var elms = screenElm.querySelectorAll("div.newsScreen > div");

    for (var i = 0; i < elms.length; i++) {
      lnx.util.stopVOverflow(elms[i]);
    }

    function addAudio() {
      self.audio = [];
      var a = screenElm.querySelectorAll("[data-audio]");

      for (var i = 0; i < a.length; i++) {
        self.audio.push(a[i].getAttribute("data-audio"));
      }
    }
  },
  getResId: function getResId() {
    return this.resId + "_" + this.stage;
  },
  hasContent: function hasContent() {
    if (this.initialized) {
      return true;
    } else {
      return false;
    }
  },
  updateVirtualScreen: function updateVirtualScreen(dir, showFinalReverse) {
    var screenWidth = 930;
    var vc = document.querySelector("#" + this.id + " .dialogueContainer");

    if (!vc) {
      vc = document.querySelector("#" + this.id + " .newsContainer");
    }

    var tl = new TimelineMax();
    var multiplier = this.stage - 1;
    var val = screenWidth * multiplier;
    var interval = 1.6;

    if (showFinalReverse) {
      //only want to animate from just before final screen if reversing
      //vc.style.left = "-" + ((screenWidth * this.stage) - (screenWidth/2)) + "px";
      interval = 0;
    }

    var sign = val === 0 ? "" : "-";
    val = sign + val + "px";
    tl.to(vc, interval, {
      left: val
    });
  },
  insertVirtualScreen: function insertVirtualScreen(dir, showFinalReverse) {
    if (showFinalReverse) {
      this.stage = this.numScreens + 1; // stage will be decrimented to correct value below in "prev" conditional
    }

    var audioDelay = 1500;

    if (dir === "next") {
      if (this.stage < this.numScreens) {
        this.stage++;
        lnx.audio.stopAudio();
        this.updateVirtualScreen(dir);
        lnx.audio.playAudio(this.audio[this.stage - 1], audioDelay); //check are we now on last screen

        if (!(this.stage + 1 < this.numScreens)) {}

        return true;
      } else {
        return false;
      }
    } else if (dir === "prev") {
      if (this.stage < 2) {
        return false;
      } else {
        this.stage--;
        lnx.audio.stopAudio();
        this.updateVirtualScreen(dir, showFinalReverse);
        lnx.audio.playAudio(this.audio[this.stage - 1], audioDelay);
        return true;
      }
    } else {
      return false;
    }
  },
  isComplete: function isComplete() {
    return lnx.cache.getValue("complete", this.id);
  },
  isFinalScreen: function isFinalScreen() {
    return this.stage >= this.numScreens;
  },
  destroy: function destroy(newType, navId) {
    if (navId === this.id) {
      return;
    }

    this.stage = null;
    this.audio = [];
    this.resId = this.id = null;
  }
};
lnx.hasVirtualScreenOverlay = {
  items: [],
  id: null,
  inOp: false,
  init: function init(node, screenElm, frameElm) {
    var b = screenElm.querySelector(".bullBtn"),
        t = screenElm.querySelector(".bullText"),
        self = this;
    if (!b) return;
    b.onclick = onClick;
    this.items.push(b);
    this.id = screenElm.getAttribute("id");

    function onClick(e) {
      self.inOp = true;
      self.registerAccess(t);
    }
  },
  registerAccess: function registerAccess(t) {
    lnx.overlayMan.show(t.innerHTML);
    lnx.cache.setValue("complete", this.id, true);
    lnx.view.onScreenComplete();
  },
  hasContent: function hasContent() {
    return false;
  },
  insertVirtualScreen: function insertVirtualScreen() {
    // Abbott want this to always display
    //if(lnx.cache.getValue("complete", this.id)) return false;
    if (this.inOp) {
      this.inOp = true;
      return false;
    }

    this.items[0].onclick();
    return true;
  },
  isComplete: function isComplete() {
    return lnx.cache.getValue("complete", this.id);
  },
  destroy: function destroy() {
    this.items[0].onclick = null;
    this.items = [];
    this.id = null;
    this.inOp = false;
  }
};
lnx.acknowledge = {
  items: [],
  init: function init(node, screenElm, frameElm) {
    lnx.util.updateEventListener(document.getElementById("ackBtn"), "click", this.onAcknoledge);
    this.items[0] = screenElm.querySelector(".ackInstruction");
    this.items[1] = screenElm.querySelector(".ackBoxInner");
    this.items[2] = screenElm.querySelector(".ackBoxCongrat");
    this.items[3] = screenElm.getAttribute("data-audioCongrat");
  },
  onAcknoledge: function onAcknoledge(e) {
    var self = lnx.acknowledge;
    var r = lnx.scormApi.lmsCompleteCourse(true);

    if (r) {
      self.items[0].innerHTML = "&nbsp;";
      self.items[1].innerHTML = self.items[2].innerHTML;
      lnx.audio.playAudio(self.items[3]);
    }
  },
  hasContent: function hasContent() {
    return false;
  },
  destroy: function destroy() {
    this.items = [];
  }
};
lnx.autoVideo = {
  items: [],
  init: function init(node, screenElm, frameElm) {
    try {
      screenElm.querySelector(".autoVideo5").play();
    } catch (e) {//
    }
  },
  hasContent: function hasContent() {
    return false;
  },
  destroy: function destroy() {
    this.items = [];
  }
};
lnx.clickAndPopup = {
  items: [],
  imgRefPref: null,
  popUp: null,
  type: null,
  popUpClose: null,
  popUnderlay: null,
  handlers: [],
  hasAudio: false,
  init: function init(node, screenElm, frameElm) {
    var self = this; // get clickable items

    var set = screenElm.querySelectorAll("div.clickable");

    for (var i = 0, len = set.length; i < len; i++) {
      this.items.push(set[i]);
    }

    this.popUp = screenElm.querySelector(".popup");
    this.popUpClose = this.popUp.querySelector("img.popupClose");
    this.popUnderlay = this.popUp.querySelector("div.popUnderlay");
    this.type = node.getAttribute("subType");

    if (this.type === "2") {
      this.imgRefPref = node.getAttribute("navId") + "_";
    }

    lnx.util.updateEventListener(this.items, "click", this.onSelection);
    lnx.util.updateEventListener(this.popUpClose, "click", onClose);
    this.handlers.push(onClose);
    lnx.util.updateEventListener(this.popUnderlay, "click", onClickUnderlay);
    this.handlers.push(onClickUnderlay);

    function onClose(e) {
      var self = lnx.clickAndPopup;
      self.popUp.style.display = "none";

      if (self.type === "2") {
        self.popUp.querySelector("div.popHeader > img").src = "images/blackPx.gif";
      }

      if (self.hasAudio) {
        lnx.audio.stopAudio();
      }
    }

    function onClickUnderlay(e) {
      if (e.target === e.currentTarget) {
        onClose();
      }
    }
  },
  onSelection: function onSelection(e) {
    e = e || window.event;
    var self = lnx.clickAndPopup;
    var num = e.currentTarget.getAttribute("data-optionNum");
    var audio = e.currentTarget.getAttribute("data-audio");

    if (self.type === "2") {
      self.popUp.querySelector("div.popHeader > img").src = "images/" + self.imgRefPref + num + ".jpg";
    } else if (self.type === "3") {
      var hdr = self.popUp.querySelector(".popHeader");
      hdr.style.backgroundColor = window.getComputedStyle(e.currentTarget, null).backgroundColor; //hdr.innerHTML = e.currentTarget.querySelector("div>span").innerHTML;

      hdr.innerHTML = e.currentTarget.querySelector("div > p").innerHTML;
    } else if (self.type === "4") {
      var hdr = self.popUp.querySelector(".popHeader");
      var num = e.currentTarget.getAttribute("data-optionNum");
      var color = "#AA0061";

      if (num === "1") {
        color = "#01AF40";
      } else if (num === "2") {
        color = "#F6B331";
      }

      hdr.style.backgroundColor = color;
      hdr.innerHTML = e.currentTarget.querySelector("span").innerHTML;
    }

    if (audio) {
      self.hasAudio = true;
      lnx.audio.playAudio(audio);
    }

    self.popUp.querySelector(".popBody").innerHTML = e.currentTarget.querySelector(" div.hide").innerHTML;
    self.popUp.style.display = "block";
    self.popUp.focus();
    e.stopPropagation ? e.stopPropagation() : e.cancelBubble = true;
    e.preventDefault ? e.preventDefault() : e.returnValue = false;
    return false;
  },
  hasContent: function hasContent() {
    return false;
  },
  destroy: function destroy() {
    lnx.util.updateEventListener(this.items, "click", this.onSelection, true);
    lnx.util.updateEventListener(this.popUpClose, "click", this.handlers[0], true);
    lnx.util.updateEventListener(this.popUnderlay, "click", this.handlers[1], true);
    this.current = this.imgRefPref = this.popUp = this.popUnderlay = this.popUpClose = null;
    this.items = [];
    this.handlers = [];
    this.hasAudio = false;
  }
};
lnx.clickAndPopupSelection = {
  items: [],
  pairs: [],
  handlers: [],
  popUp: null,
  popUpClose: null,
  popUnderlay: null,
  init: function init(node, screenElm, frameElm) {
    var self = this;
    var isType2 = this.isType2 = node.getAttribute("subType") === "2"; // get clickable items

    this.items = Array.prototype.slice.call(screenElm.querySelectorAll("div.clickable"));
    this.pairs = Array.prototype.slice.call(screenElm.querySelectorAll("div.selectionPair"));
    this.popUp = screenElm.querySelector(".popup");
    this.popUpClose = this.popUp.querySelector("img.popupClose");
    this.popUnderlay = this.popUp.querySelector("div.popUnderlay");
    lnx.util.updateEventListener(this.items, "click", this.onSelection);
    lnx.util.updateEventListener(this.popUpClose, "click", onClose);
    this.handlers.push(onClose);
    lnx.util.updateEventListener(this.popUnderlay, "click", onClickUnderlay);
    this.handlers.push(onClickUnderlay); //show first pair

    this.pairs[0].style.display = "block";
    this.adjustSize();

    function onClose(e) {
      self.popUp.style.display = "none";
      doAnimation(e); //self.nextPair();
    }

    function onClickUnderlay(e) {
      if (e.target === e.currentTarget) {
        onClose();
      }
    }

    function doAnimation(e) {
      var img = self.pairs[0].querySelector("div.selectionPair > img");
      var y = self.pairs[0].getAttribute("data-answer") === "1" ? -60 : 110;
      var x = isType2 ? 280 : 500;
      y = isType2 ? 110 : y;
      var tl = new TimelineMax();
      tl.to(img, .75, {
        scale: .3,
        x: x,
        y: y,
        onComplete: onAnimComplete
      }, '+=0.7');
      handleLastScreen();
    }

    function onAnimComplete() {
      if (self.pairs[0]) {
        var img = self.pairs[0].querySelector("div.selectionPair > img");
        img.style.display = "none";
        self.nextPair();
      }
    }

    function handleLastScreen() {
      if (isType2 && self.pairs.length < 2) {
        self.pairs[0].parentNode.classList.add("abSelectionsBtnsBgImg");
      }
    }
  },
  adjustSize: function adjustSize() {
    if (this.isType2) {
      var divs = this.pairs[0].querySelectorAll("div.abSelection");

      for (var i = 0; i < divs.length; i++) {
        if (divs[i].scrollWidth > divs[i].clientWidth) {
          var sz = parseInt(getComputedStyle(divs[i].firstElementChild).fontSize);

          while (sz > 11) {
            sz -= 1;
            divs[i].firstElementChild.style.fontSize = sz + "px";

            if (divs[i].scrollWidth <= divs[i].clientWidth) {
              break;
            }
          }
        }
      }
    }
  },
  onSelection: function onSelection(e) {
    e = e || window.event;
    var self = lnx.clickAndPopupSelection;
    var corAns = e.currentTarget.parentNode.getAttribute("data-answer");

    if (corAns === null) {
      corAns = e.currentTarget.parentNode.parentNode.getAttribute("data-answer");
    }

    var isCorrect = false;

    if (corAns === e.currentTarget.getAttribute("data-optionNum")) {
      isCorrect = true;
    }

    var img = isCorrect ? "popup_correct.png" : "popup_incorrect.png";
    self.popUp.querySelector(".popHeader > img").src = "images/" + img;
    var sp = self.popUp.querySelector(".popHeader > span");
    sp.innerHTML = self.popUp.querySelectorAll("span")[isCorrect ? 0 : 1].innerHTML;
    sp.style.color = isCorrect ? "#5ADA4F" : "#cc0070";
    var elm = e.currentTarget.querySelector("div.target") || e.currentTarget.parentNode.querySelector("div.target");
    self.popUp.querySelector(".popBody").innerHTML = elm.innerHTML;
    self.popUp.style.display = "block";
    self.popUp.focus();
    lnx.audio.stopAudio();
    lnx.audio.playAudio(isCorrect ? "Correct" : "NotCorrect");
    e.stopPropagation ? e.stopPropagation() : e.cancelBubble = true;
    e.preventDefault ? e.preventDefault() : e.returnValue = false;
    return false;
  },
  nextPair: function nextPair() {
    if (this.pairs.length < 2) {
      // exercise over
      var len = this.items.length;

      for (var i = 0; i < len; i++) {
        lnx.util.updateEventListener(this.items[i], "click", this.onSelection, true);
        this.items[i].classList.remove("clickable");
      }

      var elm = this.pairs[0].parentNode.querySelector(".endMessage");
      var audio = elm.getAttribute("data-audio");

      if (audio) {
        lnx.audio.playAudio(audio);
      }

      this.pairs[0].querySelector(".blueBox").innerHTML = elm.innerHTML;
    } else {
      this.pairs[0].style.display = "none";
      this.pairs.shift();
      this.pairs[0].style.opacity = 0;
      this.pairs[0].style.display = "block";
      var audio = this.pairs[0].getAttribute("data-audio");

      if (audio) {
        lnx.audio.playAudio(audio);
      }

      var tl = new TimelineMax();
      tl.to(this.pairs[0], .75, {
        opacity: 1
      });
      this.adjustSize();
    }
  },
  hasContent: function hasContent() {
    return false;
  },
  destroy: function destroy() {
    lnx.util.updateEventListener(this.items, "click", this.onSelection, true);
    lnx.util.updateEventListener(this.popUpClose, "click", this.handlers[0], true);
    lnx.util.updateEventListener(this.popUnderlay, "click", this.handlers[1], true);
    this.current = this.popUp = this.popUpClose = this.popUnderlay = null;
    this.items = [];
    this.pairs = [];
    this.handlers = [];
  }
};
lnx.videoScenario = {
  options: [],
  correctAns: [],
  resultDiv: null,
  init: function init(node, screenElm, frameElm) {
    var self = this;
    var resultOpen = false;
    this.options = Array.prototype.slice.call(screenElm.querySelectorAll(".scenarioRight > div"));
    this.displayDivs = screenElm.querySelectorAll("div.videoScenario > div");
    this.correctAns = screenElm.getAttribute("data-correctAnswer").split(",");
    var audio = screenElm.getAttribute("data-audio");
    lnx.util.updateEventListener(this.options, "click", onSelection);
    this.vid = screenElm.querySelector("video");
    lnx.util.updateEventListener(this.vid, "ended", onVideoEnd);
    this.vid.play();
    slideInPanels();

    function onVideoEnd(e) {
      lnx.audio.playAudio(audio);
    }

    function slideInPanels() {
      var tl = new TimelineMax();
      var left = parseInt(window.getComputedStyle(self.displayDivs[0]).left) + 310;
      tl.to(self.displayDivs[0], 2, {
        left: left
      });
      var tl2 = new TimelineMax();
      tl2.to(self.displayDivs[2], 2, {
        left: "620px"
      });
    }

    function onSelection(e) {
      lnx.audio.stopAudio();
      var opt = e.currentTarget.getAttribute("data-optionNum");
      var isCorrect = self.correctAns.indexOf(opt) !== -1;
      var result = self.displayDivs[1];
      var tl = new TimelineMax();
      tl.to(result, .75, {
        left: "0px"
      });
      result.classList.remove(isCorrect ? "incorrect" : "correct");
      result.classList.add(isCorrect ? "correct" : "incorrect");
      e.stopPropagation();
      e.preventDefault();
      return false;
    }
  },
  hasContent: function hasContent() {
    return false;
  },
  destroy: function destroy() {
    lnx.util.updateEventListener(this.options, "clcik", this.onSelection, true);
    this.options = [];
    this.correctAns = [];
    this.resultDiv = null;
  }
};
lnx.progBar = {
  progBar: null,
  slider: null,
  init: function init() {
    var isIpad = lnx.config.isIpad;
    var move = isIpad ? "touchmove" : "mousemove";
    var up = isIpad ? "touchend" : "mouseup";
    var down = isIpad ? "touchstart" : "mousedown";
    var slider = this.slider = document.getElementById("progBtn");
    var screenNum = document.getElementById("screenNum");
    var scrNumD = screenNum.getBoundingClientRect().width / 2;
    var progBar = this.progBar = document.getElementById("progressBar");
    var numOfPages = this.numOfPages = lnx.nav.navNodes.length;
    slider.addEventListener(down, onSliderDown, false);

    function onSliderDown(e) {
      document.addEventListener(move, onSliderMove, true);
      document.addEventListener(up, onSliderUp, true);
      onSliderMove(e, true); //calcPage(e, null, true);

      screenNum.style.display = "block";
      e.stopPropagation();
      e.preventDefault();
      return false;
    }

    function onSliderMove(e, noNav) {
      var rect = progBar.getBoundingClientRect();

      if (e.pageX < rect.left || e.pageX > rect.right - 8) {
        //console.log('returning onslidermove');
        return;
      }

      var r = slider.getBoundingClientRect();
      var d = 0; //(e.pageX - r.left)/2;

      var l = parseInt(slider.style.left, 10);
      if (isNaN(l)) l = 0; //console.log(r.left + " : " + e.pageX + " : " + l);

      if (noNav !== true) {
        slider.style.left = l - d + "px";
      }

      screenNum.style.left = l - d - 5 + "px";
      calcPage(e, r.left + r.width / 2, noNav);
      e && e.stopPropagation();
      e && e.preventDefault();
      return false;
    }

    function onSliderUp(e) {
      //onSliderMove(e, false);           
      document.removeEventListener(move, onSliderMove, true);
      document.removeEventListener(up, onSliderUp, true);
      screenNum.style.display = "none";
      e.stopPropagation();
      e.preventDefault();
      return false;
    }

    function calcPage(e, discMid, noNav) {
      var rect = progBar.getBoundingClientRect();
      var d = e.pageX - rect.left;
      var orig = numOfPages * (d / rect.width);
      var p = Math.round(orig);
      if (p < 0) p = 0;
      if (orig < 1) p = 0;
      if (p > numOfPages - 1) p = numOfPages - 1;
      screenNum.innerHTML = p + 1;

      if (noNav) {
        return;
      }

      lnx.nav.goToScreen(p);
    }
  },
  positionSlider: function positionSlider(index) {
    var r = this.progBar.getBoundingClientRect();
    var x = r.width * (index / this.numOfPages);
    this.slider.style.left = x + "px";
  }
};
lnx.certification = {
  items: [],
  init: function init(node, screenElm, frameElm) {
    lnx.util.updateEventListener(document.getElementById("certifyBtn"), "click", this.onCertify);
    var e = screenElm.querySelectorAll("div");
    this.items[0] = e[0];
    this.items[1] = e[1];
    this.items[2] = e[2];
    this.items[3] = screenElm.getAttribute("data-audio");
    this.items[0].style.display = "block";
    this.items[1].style.display = "none";
    this.items[2].style.display = "none";

    if (lnx.scormApi.getTestCompletedSuccessfully()) {
      this.items[1].style.display = "block";
      this.items[0].style.display = "none";

      if (this.items[3]) {
        lnx.audio.playAudio(this.items[3]);
      }
    }
  },
  onCertify: function onCertify(e) {
    var self = lnx.certification;
    var r = lnx.scormApi.lmsCompleteCourse(true);

    if (r) {
      self.items[0].style.display = "none";
      self.items[1].style.display = "none";
      self.items[2].style.display = "block";
    }
  },
  hasContent: function hasContent() {
    return false;
  },
  destroy: function destroy() {
    this.items = [];
  }
};
lnx.home = (_lnx$home = {
  items: [],
  isInited: false,
  isActive: false,
  overlayToc: null,
  firstAccess: true
}, _defineProperty(_lnx$home, "isActive", false), _defineProperty(_lnx$home, "timelines", null), _defineProperty(_lnx$home, "timelinesV", null), _defineProperty(_lnx$home, "origMsg3", null), _defineProperty(_lnx$home, "init", function init(node, screenElm, frameElm) {
  var self = this;
  this.isInited = true; //activate toc btn when user arives at this screen

  var tocBtn = document.querySelector("#tocBtn");
  tocBtn.classList.add("tocBtnActive");
  var playBtns = screenElm.querySelectorAll("#" + lnx.nav.homeId + " div.topic .topicPlayBtn");
  lnx.util.updateEventListener(Array.prototype.slice.call(playBtns), "click", this.onGoToTopic);
  this.isActive = true;
  var tiles = screenElm.querySelectorAll("div.tocTiles > div.tTile");
  lnx.util.updateEventListener(Array.prototype.slice.call(tiles), "click", this.onTileClick);
  var activeTiles = this.showActiveTopics(screenElm);
  this.timelines = this.setUpAnims(this.getTotalTiles(screenElm));

  if (this.firstAccess) {
    this.showMessage(1);
  } else {
    this.playAnims(activeTiles, this.timelines);
  }
}), _defineProperty(_lnx$home, "showMessage", function showMessage(num, val, e) {
  var self = lnx.home;

  if (!self.origMsg3) {
    self.origMsg3 = document.querySelector("div.tocMessages > p:nth-of-type(3)").innerHTML;
  }

  var msg = document.querySelectorAll("div.tocMessages > p");
  var con = document.querySelector("div.tocMessages");

  for (var i = 0; i < msg.length; i++) {
    msg[i].style.display = "none";
  }

  con.style.top = "";
  con.style.left = "";

  if (num) {
    msg = document.querySelector("div.tocMessages > p:nth-of-type(" + num + ")");

    if (num === 3) {
      var rep1 = "";

      if (val.length > 1) {
        rep1 = "s";
      }

      msg.innerHTML = self.origMsg3.replace("{a}", rep1).replace("{b}", val);

      if (e) {
        var t = e.currentTarget;
        con.style.top = t.offsetTop + 60 + "px";
        con.style.left = t.offsetLeft + 40 + "px";
      }
    }

    msg.style.display = "block";
  }
}), _defineProperty(_lnx$home, "getTotalTiles", function getTotalTiles(elm) {
  if (!elm) {
    elm = document;
  }

  var t = elm.querySelectorAll("div.tocTiles > div.tTile");
  return t.length;
}), _defineProperty(_lnx$home, "showActiveTopics", function showActiveTopics(elm) {
  var status = lnx.nav.getTopicsStatus();
  var mods = elm.querySelectorAll("div.tocTiles > div.tTile");
  var activeTiles = [];

  for (var i = 0; i < mods.length; i++) {
    var m = mods[i];
    var mId = m.getAttribute("data-modnavid");

    if (mId && status[mId].available) {
      activeTiles.push(i);
      var topics = m.querySelectorAll("div.topic");

      for (var j = 0; j < topics.length; j++) {
        var t = topics[j];
        var tId = t.getAttribute("data-topicnavid");
        var mt = status[mId].topics[tId];
        t.style.display = "block";
        var imgs = t.querySelectorAll("img");

        if (mt.available) {
          imgs[1].style.display = "inline";

          if (mt.complete) {
            imgs[0].src = "images/topicCheckMark.png";
          }
        } else {
          imgs[1].style.display = "none";
        }
      }
    }
  }

  return activeTiles;
}), _defineProperty(_lnx$home, "setUpAnims", function setUpAnims(totalTiles) {
  var a = [];

  for (var i = 0; i < totalTiles; i++) {
    var num = i + 1;
    var tl = gsap.timeline({
      paused: true
    }).fromTo("div.tocTiles > div.tTile:nth-of-type(" + num + ") > div.tFace:nth-of-type(1)", {
      rotationY: 0
    }, {
      rotationY: -90,
      duration: 1
    }).fromTo("div.tocTiles > div.tTile:nth-of-type(" + num + ") > div.tFace:nth-of-type(2)", {
      rotationY: 90
    }, {
      rotationY: 0,
      duration: 1
    }, "<");
    a.push(tl);
  }

  return a;
}), _defineProperty(_lnx$home, "onTileClick", function onTileClick(e) {
  var self = lnx.home;
  var status = lnx.nav.getTopicsStatus();
  var mId = e.currentTarget.getAttribute("data-modnavid");
  var m = status[mId];

  if (m.available) {
    var i = 0;

    for (var k in status) {
      if (status[k].id === mId) {
        break;
      }

      i++;
    }

    self.playAnims([i], self.isActive ? self.timelines : self.timelinesV, true); //clear msgs - pass null arg

    self.showMessage(null);

    if (self.firstAccess) {
      setTimeout(self.showMessage, 1100, 2);
      self.firstAccess = false;
    }
  } else {
    var num = parseInt(e.currentTarget.querySelector(".tileFrontText > p").innerText.charAt(0));
    var m = getIncompleteModNums(num);
    self.showMessage(3, m, e);
  }

  function getIncompleteModNums(num) {
    var status = lnx.nav.getTopicsStatus();
    var a = orderMods(status);
    var b = [];

    for (var i = 0; i < a.length; i++) {
      if (status[a[i]].index < num && status[a[i]].complete != true) {
        b.push(i + 1);
      }
    }

    if (b.length === 1) {
      return "" + b[0];
    } else {
      return b[0] + "-" + b[b.length - 1];
    }
  }

  function orderMods(mods) {
    var a = [];

    for (var m in mods) {
      a.push(m);
    }

    a.sort(function (a, b) {
      if (a.index < b.index) {
        return -1;
      } else {
        return 1;
      }
    });
    return a;
  }
}), _defineProperty(_lnx$home, "playAnims", function playAnims(tiles, anims, isClick, isVirtual) {
  if (!isClick && isVirtual) {
    var tl = gsap.timeline().fromTo("#overlayToc div.tocTiles", {
      scale: .8,
      opacity: 0
    }, {
      scale: 1,
      opacity: 1,
      duration: .3
    });
  }

  for (var i = 0; i < tiles.length; i++) {
    var anim = anims[tiles[i]];

    if (isClick && anim.progress() > 0) {
      anim.reverse();
    } else {
      anim.seek(0);
      anim.play();
    }
  }
}), _defineProperty(_lnx$home, "createTocModal", function createTocModal() {
  var toc = document.querySelector("#overlayToc");
  var con = toc.querySelector("div.overlayTocContainer");
  var ul = toc.querySelector("#overlayToc div.popUnderlay");
  con.innerHTML = lnx.nav.screensMap[lnx.nav.homeId].innerHTML;
  this.overlayToc = {
    toc: toc,
    con: con,
    ul: ul,
    active: false
  };
  var close = document.querySelector("#overlayToc img.overlayTocCloseBtn");
  lnx.util.updateEventListener(close, "click", this.showTocModal);
  lnx.util.updateEventListener(ul, "click", this.showTocModal);
  var playBtns = toc.querySelectorAll("div.topic .topicPlayBtn");
  lnx.util.updateEventListener(Array.prototype.slice.call(playBtns), "click", this.onGoToTopicFromModal);
  var tiles = toc.querySelectorAll("div.tocTiles > div.tTile");
  lnx.util.updateEventListener(Array.prototype.slice.call(tiles), "click", this.onTileClick);
  this.showActiveTopics(this.overlayToc.toc);
  this.timelinesV = this.setUpAnims(this.getTotalTiles());
}), _defineProperty(_lnx$home, "showTocModal", function showTocModal(e) {
  var self = lnx.home;

  if (self.isActive || !self.isInited) {
    // don't show modal if already on actual home screen
    return;
  }

  if (lnx.nav.isFarIndexBeforeHomeScreen()) {
    //special case - we jump to the actual home screen if user clicks icon for virtual home screen before 
    //they have completed screens prior to home screen - avoids conflict with closed/locked navigation
    lnx.nav.goToHomeScreen();
    return;
  }

  self.overlayToc.toc.style.display = self.overlayToc.active ? "none" : "block";
  self.overlayToc.ul.style.display = self.overlayToc.active ? "none" : "block";
  self.overlayToc.active = self.overlayToc.active ? false : true;

  if (self.overlayToc.active) {
    var activeTilesV = this.showActiveTopics(this.overlayToc.toc);
    self.playAnims(activeTilesV, self.timelinesV, false, true);
  }
}), _defineProperty(_lnx$home, "onGoToTopicFromModal", function onGoToTopicFromModal(e) {
  var self = lnx.home;
  self.onGoToTopic(e);
  self.showTocModal();
}), _defineProperty(_lnx$home, "onGoToTopic", function onGoToTopic(e) {
  var navId = e.currentTarget.parentNode.getAttribute("data-screennavid");
  lnx.nav.navigate(navId);
}), _defineProperty(_lnx$home, "hasContent", function hasContent() {
  return false;
}), _defineProperty(_lnx$home, "destroy", function destroy() {
  this.timelines = [];
  this.isActive = false;
  this.firstAccess = false;
}), _lnx$home);
lnx.flashCard = {
  items: [],
  stage: null,
  id: null,
  total: 0,
  init: function init(node, screenElm, frameElm) {
    var self = this;
    this.stage = 1;
    this.id = screenElm.getAttribute("id");
    var flashCards = this.items = getFlashCards();
    this.total = flashCards.total;
    this.changeCard = changeCard;
    this.showCongrats = showCongrats;
    this.congratsTl;
    var hideMessage = true;
    var currentCardNum = 0;
    var btns = gsap.utils.toArray("div.flashCardButtons > img");

    for (var i = 0; i < btns.length; i++) {
      btns[i].addEventListener("click", onForwardNavEvent);
    }

    initCongratsScreen();
    var circles = initCircles(flashCards.total);
    setActiveCircle(0, 0);

    function onForwardNavEvent(e) {
      var dir = e.target.getAttribute("data-fcNavDir");
      lnx.nav.navigate(dir);
    }

    function changeCard(dir) {
      var r = flashCards;
      var cardItem;
      removeMessage();
      previousActiveCardNum = r.current;

      if (dir === "next") {
        if (!(r.current < r.total - 1)) {
          return;
        }

        cardItem = r.deck[r.current];
        cardItem.tl.play();
        r.current++;
        r.deck[r.current].card.style.display = "flex";
      } else {
        if (!(r.current > 0)) {
          return;
        }

        r.current--;
        cardItem = r.deck[r.current];
        if (cardItem) cardItem.tl.reverse();
      }

      setActiveCircle(previousActiveCardNum, r.current);
    }

    function showCongrats(isReverse) {
      if (isReverse) {
        self.congratsTl.reverse();
      } else {
        var msg = document.querySelector("div.congratsScreen > p");
        msg.focus();
        self.congratsTl.play();
        var file = document.querySelector("div.congratsScreen").getAttribute("data-audio");
        lnx.audio.playAudio(file, 400);
      }
    }

    function getFlashCards() {
      var fcs = gsap.utils.toArray("div.flashCard");
      var deck = [];
      fcs.forEach(function (card, i) {
        var tl = gsap.timeline({
          paused: true
        }).to(card, {
          rotation: -12,
          x: "-=700px",
          opacity: 0,
          duration: 1,
          ease: "power3.in"
        });
        card.style.zIndex = fcs.length - i;
        deck.push({
          card: card,
          tl: tl
        });
      });
      return {
        deck: deck,
        current: 0,
        total: deck.length
      };
    }

    function initCongratsScreen() {
      var congrats = document.querySelectorAll("div.congratsScreen");
      var msg = document.querySelector("div.congratsScreen > p");
      msg.addEventListener("blur", function (e) {
        msg.style.display = "none";
      }); //msg.focus();

      self.congratsTl = gsap.timeline({
        paused: true
      }) //.to(ins, {opacity: 0, duration: .3})
      .set(congrats, {
        display: "flex"
      }).from(congrats, {
        x: 930,
        duration: 1,
        ease: "power2.in"
      }, "<-.2").from(msg, {
        opacity: 0,
        duration: .75
      }, "+=.3");
    }

    function removeMessage() {
      if (hideMessage) {
        var msg = document.querySelector("div.flashCardInstruction"); // msg.classList.remove("flashCardInstructionAnim");
        // msg.classList.add("fadeOutAnimClass");

        msg.style.display = "none";
        hideMessage = false;
      }
    }

    function initCircles(num) {
      var container = document.querySelector("div.circleContainer");
      var s = "";

      for (var i = 0; i < num; i++) {
        s += "<div></div>";
      }

      container.innerHTML = s;
      return container.childNodes;
    }

    function setActiveCircle(old, current) {
      circles[old].style.backgroundColor = "#C3C7D5";
      circles[current].style.backgroundColor = "#353C46";
    }
  },
  updateVirtualScreen: function updateVirtualScreen(dir, isFinal, fromFinal) {
    var self = this;

    if (isFinal) {
      this.showCongrats();
    } else if (fromFinal) {
      this.showCongrats(fromFinal);
    } else {
      this.changeCard(dir);
    }
  },
  insertVirtualScreen: function insertVirtualScreen(dir) {
    if (dir == "next") {
      if (this.stage < this.total + 1) {
        this.stage++;
        var isFinal = this.stage > this.total;
        this.updateVirtualScreen(dir, isFinal);
        return true;
      } else {
        return false;
      }
    } else {
      if (this.stage > 1) {
        this.stage--;
        var fromFinal = this.stage === this.total;
        this.updateVirtualScreen(dir, false, fromFinal);
        return true;
      } else {
        return false;
      }
    }
  },
  isFinalScreen: function isFinalScreen() {
    if (this.stage > this.total) {
      return true;
    } else {
      return false;
    }
  },
  hasContent: function hasContent() {
    if (this.initialized) {
      return true;
    } else {
      return false;
    }
  },
  destroy: function destroy() {
    this.items = [];
  }
}; // handle window events

window.onerror = function (m, u, l) {
  lnx.util.onWinErr(m, u, l);
};

window.onload = function () {
  lnx.init(); //lnx.util.init();
};

window.onbeforeunload = function (e) {
  var elm = e.target && e.target.activeElement || document.activeElement;

  if (elm && elm.tagName.toLowerCase() === 'a' && elm.href.indexOf('mailto:') > -1) {
    return;
  }

  lnx.exit();
};

window.onunload = function () {
  lnx.exit();
}; //force update

/*window.onbeforeunload = function(){
	
	//lnx.exit();
}*/